@@ -15,13 +15,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.hadoop.hbase.testclassification;
 
 /**
- * Tag a test as related to  mapreduce and taking longer than 5 minutes to run on public build
+ * Tag a test as related to mapreduce and taking longer than 5 minutes to run on public build
  * infrastructure.
- *
  * @see org.apache.hadoop.hbase.testclassification.ClientTests
  * @see org.apache.hadoop.hbase.testclassification.CoprocessorTests
  * @see org.apache.hadoop.hbase.testclassification.FilterTests
@@ -38,4 +36,6 @@ package org.apache.hadoop.hbase.testclassification;
  * @see org.apache.hadoop.hbase.testclassification.VerySlowMapReduceTests
  */
 public interface VerySlowMapReduceTests {
+  public static final String TAG =
+    "org.apache.hadoop.hbase.testclassification.VerySlowMapReduceTests";
 }
