@@ -430,6 +430,11 @@ public class ServerLoad implements ServerMetrics {
     return metrics.getTasks();
   }
 
+  @Override
+  public Map<String, Integer> getRegionCachedInfo() {
+    return metrics.getRegionCachedInfo();
+  }
+
   /**
    * Originally, this method factored in the effect of requests going to the server as well.
    * However, this does not interact very well with the current region rebalancing code, which only
