@@ -99,6 +99,11 @@ class AsyncHBaseAdmin implements AsyncAdmin {
     return wrap(rawAdmin.listTableDescriptorsByNamespace(name));
   }
 
+  @Override
+  public CompletableFuture<List<TableDescriptor>> listTableDescriptorsByState(boolean isEnabled) {
+    return wrap(rawAdmin.listTableDescriptorsByState(isEnabled));
+  }
+
   @Override
   public CompletableFuture<List<TableName>> listTableNames(boolean includeSysTables) {
     return wrap(rawAdmin.listTableNames(includeSysTables));
@@ -115,6 +120,11 @@ class AsyncHBaseAdmin implements AsyncAdmin {
     return wrap(rawAdmin.listTableNamesByNamespace(name));
   }
 
+  @Override
+  public CompletableFuture<List<TableName>> listTableNamesByState(boolean isEnabled) {
+    return wrap(rawAdmin.listTableNamesByState(isEnabled));
+  }
+
   @Override
   public CompletableFuture<TableDescriptor> getDescriptor(TableName tableName) {
     return wrap(rawAdmin.getDescriptor(tableName));
@@ -264,6 +274,11 @@ class AsyncHBaseAdmin implements AsyncAdmin {
     return wrap(rawAdmin.flush(tableName, columnFamily));
   }
 
+  @Override
+  public CompletableFuture<Void> flush(TableName tableName, List<byte[]> columnFamilies) {
+    return wrap(rawAdmin.flush(tableName, columnFamilies));
+  }
+
   @Override
   public CompletableFuture<Void> flushRegion(byte[] regionName) {
     return wrap(rawAdmin.flushRegion(regionName));
@@ -376,6 +391,11 @@ class AsyncHBaseAdmin implements AsyncAdmin {
     return wrap(rawAdmin.splitRegion(regionName, splitPoint));
   }
 
+  @Override
+  public CompletableFuture<Void> truncateRegion(byte[] regionName) {
+    return wrap(rawAdmin.truncateRegion(regionName));
+  }
+
   @Override
   public CompletableFuture<Void> assign(byte[] regionName) {
     return wrap(rawAdmin.assign(regionName));
@@ -480,6 +500,22 @@ class AsyncHBaseAdmin implements AsyncAdmin {
     return wrap(rawAdmin.disableTableReplication(tableName));
   }
 
+  @Override
+  public CompletableFuture<Boolean> isReplicationPeerEnabled(String peerId) {
+    return wrap(rawAdmin.isReplicationPeerEnabled(peerId));
+  }
+
+  @Override
+  public CompletableFuture<Boolean> replicationPeerModificationSwitch(boolean on,
+    boolean drainProcedures) {
+    return wrap(rawAdmin.replicationPeerModificationSwitch(on, drainProcedures));
+  }
+
+  @Override
+  public CompletableFuture<Boolean> isReplicationPeerModificationEnabled() {
+    return wrap(rawAdmin.isReplicationPeerModificationEnabled());
+  }
+
   @Override
   public CompletableFuture<Void> snapshot(SnapshotDescription snapshot) {
     return wrap(rawAdmin.snapshot(snapshot));
@@ -880,4 +916,9 @@ class AsyncHBaseAdmin implements AsyncAdmin {
   public CompletableFuture<Void> flushMasterStore() {
     return wrap(rawAdmin.flushMasterStore());
   }
+
+  @Override
+  public CompletableFuture<List<String>> getCachedFilesList(ServerName serverName) {
+    return wrap(rawAdmin.getCachedFilesList(serverName));
+  }
 }
