@@ -45,6 +45,7 @@ import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.client.backoff.ServerStatistics;
 import org.apache.hadoop.hbase.client.coprocessor.Batch;
 import org.apache.hadoop.hbase.exceptions.ClientExceptionsUtil;
+import org.apache.hadoop.hbase.quotas.RpcThrottlingException;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.yetus.audience.InterfaceAudience;
@@ -284,6 +285,7 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
   private final int operationTimeout;
   private final int rpcTimeout;
   private final AsyncProcess asyncProcess;
+  private final Map<String, byte[]> requestAttributes;
 
   /**
    * For {@link AsyncRequestFutureImpl#manageError(int, Row, Retry, Throwable, ServerName)}. Only
@@ -398,6 +400,7 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
     if (task.getCallable() == null) {
       tracker = new RetryingTimeTracker().start();
     }
+    this.requestAttributes = task.getRequestAttributes();
   }
 
   protected Set<CancellableRegionServerCallable> getCallsInProgress() {
@@ -414,21 +417,25 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
    * timeout against the appropriate tracker, or returns false if no tracker.
    */
   private boolean isOperationTimeoutExceeded() {
+    // return value of 1 is special to mean exceeded, to differentiate from 0
+    // which is no timeout. see implementation of RetryingTimeTracker.getRemainingTime
+    return getRemainingTime() == 1;
+  }
+
+  private long getRemainingTime() {
     RetryingTimeTracker currentTracker;
     if (tracker != null) {
       currentTracker = tracker;
     } else if (currentCallable != null && currentCallable.getTracker() != null) {
       currentTracker = currentCallable.getTracker();
     } else {
-      return false;
+      return 0;
     }
 
     // no-op if already started, this is just to ensure it was initialized (usually true)
     currentTracker.start();
 
-    // return value of 1 is special to mean exceeded, to differentiate from 0
-    // which is no timeout. see implementation of getRemainingTime
-    return currentTracker.getRemainingTime(operationTimeout) == 1;
+    return currentTracker.getRemainingTime(operationTimeout);
   }
 
   /**
@@ -847,6 +854,8 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
     long backOffTime;
     if (retryImmediately) {
       backOffTime = 0;
+    } else if (throwable instanceof RpcThrottlingException) {
+      backOffTime = ((RpcThrottlingException) throwable).getWaitInterval();
     } else if (HBaseServerException.isServerOverloaded(throwable)) {
       // Give a special check when encountering an exception indicating the server is overloaded.
       // see #HBASE-17114 and HBASE-26807
@@ -869,6 +878,19 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
         backOffTime, true, null, -1, -1));
     }
 
+    long remainingTime = getRemainingTime();
+    // 1 is a special value meaning exceeded and 0 means no timeout.
+    // throw if timeout already exceeded, or if backoff is larger than non-zero remaining
+    if (remainingTime == 1 || (remainingTime > 0 && backOffTime > remainingTime)) {
+      OperationTimeoutExceededException ex = new OperationTimeoutExceededException(
+        "Backoff time of " + backOffTime + "ms would exceed operation timeout", throwable);
+      for (Action actionToFail : toReplay) {
+        manageError(actionToFail.getOriginalIndex(), actionToFail.getAction(),
+          Retry.NO_NOT_RETRIABLE, ex, null);
+      }
+      return;
+    }
+
     try {
       if (backOffTime > 0) {
         Thread.sleep(backOffTime);
@@ -1345,7 +1367,8 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
   private MultiServerCallable createCallable(final ServerName server, TableName tableName,
     final MultiAction multi) {
     return new MultiServerCallable(asyncProcess.connection, tableName, server, multi,
-      asyncProcess.rpcFactory.newController(), rpcTimeout, tracker, multi.getPriority());
+      asyncProcess.rpcFactory.newController(), rpcTimeout, tracker, multi.getPriority(),
+      requestAttributes);
   }
 
   private void updateResult(int index, Object result) {
