@@ -27,6 +27,8 @@ public class CheckAndMutateResult {
   private final boolean success;
   private final Result result;
 
+  private QueryMetrics metrics = null;
+
   public CheckAndMutateResult(boolean success, Result result) {
     this.success = success;
     this.result = result;
@@ -41,4 +43,13 @@ public class CheckAndMutateResult {
   public Result getResult() {
     return result;
   }
+
+  public CheckAndMutateResult setMetrics(QueryMetrics metrics) {
+    this.metrics = metrics;
+    return this;
+  }
+
+  public QueryMetrics getMetrics() {
+    return metrics;
+  }
 }
