@@ -27,12 +27,12 @@ import io.opentelemetry.context.Scope;
 import java.io.IOException;
 import java.io.InterruptedIOException;
 import java.util.ArrayDeque;
+import java.util.Map;
 import java.util.Queue;
 import java.util.concurrent.ExecutorService;
 import org.apache.commons.lang3.mutable.MutableBoolean;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.DoNotRetryIOException;
-import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.HRegionInfo;
 import org.apache.hadoop.hbase.HRegionLocation;
 import org.apache.hadoop.hbase.NotServingRegionException;
@@ -63,6 +63,12 @@ public abstract class ClientScanner extends AbstractClientScanner {
   private static final Logger LOG = LoggerFactory.getLogger(ClientScanner.class);
 
   protected final Scan scan;
+  // We clone the original client Scan to avoid modifying user object from scan internals.
+  // The below scanForMetrics is the client's object, which we mutate only for returning
+  // ScanMetrics.
+  // See https://issues.apache.org/jira/browse/HBASE-27402.
+  private final Scan scanForMetrics;
+
   protected boolean closed = false;
   // Current region scanner is against. Gets cleared if current region goes
   // wonky: e.g. if it splits on us.
@@ -79,6 +85,7 @@ public abstract class ClientScanner extends AbstractClientScanner {
   protected final TableName tableName;
   protected final int readRpcTimeout;
   protected final int scannerTimeout;
+  private final boolean useScannerTimeoutForNextCalls;
   protected boolean scanMetricsPublished = false;
   protected RpcRetryingCaller<Result[]> caller;
   protected RpcControllerFactory rpcControllerFactory;
@@ -92,6 +99,7 @@ public abstract class ClientScanner extends AbstractClientScanner {
   protected final int primaryOperationTimeout;
   private int retries;
   protected final ExecutorService pool;
+  protected final Map<String, byte[]> requestAttributes;
 
   /**
    * Create a new ClientScanner for the specified table Note that the passed {@link Scan}'s start
@@ -101,10 +109,13 @@ public abstract class ClientScanner extends AbstractClientScanner {
    * @param tableName  The table that we wish to scan
    * @param connection Connection identifying the cluster
    */
-  public ClientScanner(final Configuration conf, final Scan scan, final TableName tableName,
-    ClusterConnection connection, RpcRetryingCallerFactory rpcFactory,
+  public ClientScanner(final Configuration conf, final Scan scan, final Scan scanForMetrics,
+    final TableName tableName, ClusterConnection connection, RpcRetryingCallerFactory rpcFactory,
     RpcControllerFactory controllerFactory, ExecutorService pool, int scanReadRpcTimeout,
-    int scannerTimeout, int primaryOperationTimeout) throws IOException {
+    int scannerTimeout, int primaryOperationTimeout,
+    ConnectionConfiguration connectionConfiguration, Map<String, byte[]> requestAttributes)
+    throws IOException {
+    this.scanForMetrics = scanForMetrics;
     if (LOG.isTraceEnabled()) {
       LOG.trace(
         "Scan table=" + tableName + ", startRow=" + Bytes.toStringBinary(scan.getStartRow()));
@@ -115,19 +126,20 @@ public abstract class ClientScanner extends AbstractClientScanner {
     this.connection = connection;
     this.pool = pool;
     this.primaryOperationTimeout = primaryOperationTimeout;
-    this.retries = conf.getInt(HConstants.HBASE_CLIENT_RETRIES_NUMBER,
-      HConstants.DEFAULT_HBASE_CLIENT_RETRIES_NUMBER);
+    this.retries = connectionConfiguration.getRetriesNumber();
     if (scan.getMaxResultSize() > 0) {
       this.maxScannerResultSize = scan.getMaxResultSize();
     } else {
-      this.maxScannerResultSize = conf.getLong(HConstants.HBASE_CLIENT_SCANNER_MAX_RESULT_SIZE_KEY,
-        HConstants.DEFAULT_HBASE_CLIENT_SCANNER_MAX_RESULT_SIZE);
+      this.maxScannerResultSize = connectionConfiguration.getScannerMaxResultSize();
     }
     this.readRpcTimeout = scanReadRpcTimeout;
     this.scannerTimeout = scannerTimeout;
+    this.useScannerTimeoutForNextCalls = connectionConfiguration.isUseScannerTimeoutForNextCalls();
+    this.requestAttributes = requestAttributes;
 
     if (scan.isScanMetricsByRegionEnabled() && scan.getConsistency() == Consistency.TIMELINE) {
       scan.setEnableScanMetricsByRegion(false);
+      scanForMetrics.setEnableScanMetricsByRegion(false);
       LOG.warn("Scan metrics by region is not supported for timeline consistency in HBase 2");
     }
     // check if application wants to collect scan metrics
@@ -137,8 +149,7 @@ public abstract class ClientScanner extends AbstractClientScanner {
     if (this.scan.getCaching() > 0) {
       this.caching = this.scan.getCaching();
     } else {
-      this.caching = conf.getInt(HConstants.HBASE_CLIENT_SCANNER_CACHING,
-        HConstants.DEFAULT_HBASE_CLIENT_SCANNER_CACHING);
+      this.caching = connectionConfiguration.getScannerCaching();
     }
 
     this.caller = rpcFactory.<Result[]> newCaller();
@@ -260,7 +271,7 @@ public abstract class ClientScanner extends AbstractClientScanner {
     }
     this.callable = new ScannerCallableWithReplicas(getTable(), getConnection(),
       createScannerCallable(), pool, primaryOperationTimeout, scan, getRetries(), readRpcTimeout,
-      scannerTimeout, caching, conf, caller);
+      scannerTimeout, useScannerTimeoutForNextCalls, caching, conf, caller);
     this.callable.setCaching(this.caching);
     incRegionCountMetrics(scanMetrics);
     return true;
@@ -301,7 +312,7 @@ public abstract class ClientScanner extends AbstractClientScanner {
     // As we have claimed in the comment of Scan.getScanMetrics, this relies on that user will not
     // call ResultScanner.getScanMetrics and reset the ScanMetrics. Otherwise the metrics published
     // to Scan will be messed up.
-    scan.setAttribute(Scan.SCAN_ATTRIBUTES_METRICS_DATA,
+    scanForMetrics.setAttribute(Scan.SCAN_ATTRIBUTES_METRICS_DATA,
       ProtobufUtil.toScanMetrics(scanMetrics, false).toByteArray());
     scanMetricsPublished = true;
   }
