@@ -26,9 +26,7 @@ import org.apache.yetus.audience.InterfaceAudience;
 @InterfaceAudience.Public
 public enum CompactType {
 
-  NORMAL(0),
-  MOB(1);
+  NORMAL,
+  MOB
 
-  CompactType(int value) {
-  }
 }
