@@ -39,6 +39,7 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Date;
 import java.util.List;
+import java.util.Map;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
@@ -50,6 +51,8 @@ import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.locks.ReentrantLock;
+import java.util.function.Supplier;
+import java.util.stream.Collectors;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.AuthUtil;
 import org.apache.hadoop.hbase.CatalogReplicaMode;
@@ -94,7 +97,6 @@ import org.apache.zookeeper.KeeperException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import org.apache.hbase.thirdparty.com.google.common.base.Supplier;
 import org.apache.hbase.thirdparty.com.google.common.base.Suppliers;
 import org.apache.hbase.thirdparty.com.google.common.base.Throwables;
 import org.apache.hbase.thirdparty.com.google.common.util.concurrent.ThreadFactoryBuilder;
@@ -110,6 +112,9 @@ import org.apache.hadoop.hbase.shaded.protobuf.generated.AccessControlProtos.Get
 import org.apache.hadoop.hbase.shaded.protobuf.generated.AccessControlProtos.HasUserPermissionsRequest;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.AccessControlProtos.HasUserPermissionsResponse;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.AdminProtos;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.BootstrapNodeProtos.BootstrapNodeService;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.BootstrapNodeProtos.GetAllBootstrapNodesRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.BootstrapNodeProtos.GetAllBootstrapNodesResponse;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.ClientService.BlockingInterface;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.HBaseProtos;
@@ -146,6 +151,9 @@ import org.apache.hadoop.hbase.shaded.protobuf.generated.QuotaProtos.GetQuotaSta
 import org.apache.hadoop.hbase.shaded.protobuf.generated.QuotaProtos.GetQuotaStatesResponse;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.QuotaProtos.GetSpaceQuotaRegionSizesRequest;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.QuotaProtos.GetSpaceQuotaRegionSizesResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.RegionServerStatusProtos.GetLiveRegionServersRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.RegionServerStatusProtos.GetLiveRegionServersResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.RegionServerStatusProtos.RegionServerStatusService;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.AddReplicationPeerRequest;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.AddReplicationPeerResponse;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.DisableReplicationPeerRequest;
@@ -154,10 +162,18 @@ import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.Enabl
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.EnableReplicationPeerResponse;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.GetReplicationPeerConfigRequest;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.GetReplicationPeerConfigResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.GetReplicationPeerModificationProceduresRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.GetReplicationPeerModificationProceduresResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.GetReplicationPeerStateRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.GetReplicationPeerStateResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.IsReplicationPeerModificationEnabledRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.IsReplicationPeerModificationEnabledResponse;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.ListReplicationPeersRequest;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.ListReplicationPeersResponse;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.RemoveReplicationPeerRequest;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.RemoveReplicationPeerResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.ReplicationPeerModificationSwitchRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.ReplicationPeerModificationSwitchResponse;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.UpdateReplicationPeerConfigRequest;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ReplicationProtos.UpdateReplicationPeerConfigResponse;
 
@@ -269,7 +285,16 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
    * @param conf Configuration object
    */
   ConnectionImplementation(Configuration conf, ExecutorService pool, User user) throws IOException {
-    this(conf, pool, user, null);
+    this(conf, pool, user, null, Collections.emptyMap());
+  }
+
+  /**
+   * constructor
+   * @param conf Configuration object
+   */
+  ConnectionImplementation(Configuration conf, ExecutorService pool, User user,
+    Map<String, byte[]> connectionAttributes) throws IOException {
+    this(conf, pool, user, null, connectionAttributes);
   }
 
   /**
@@ -277,6 +302,14 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
    */
   ConnectionImplementation(Configuration conf, ExecutorService pool, User user,
     ConnectionRegistry registry) throws IOException {
+    this(conf, pool, user, registry, Collections.emptyMap());
+  }
+
+  /**
+   * Constructor, for creating cluster connection with provided ConnectionRegistry.
+   */
+  ConnectionImplementation(Configuration conf, ExecutorService pool, User user,
+    ConnectionRegistry registry, Map<String, byte[]> connectionAttributes) throws IOException {
     this.conf = conf;
     this.user = user;
     if (user != null && user.isLoginFromKeytab()) {
@@ -318,7 +351,7 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
 
     try {
       if (registry == null) {
-        this.registry = ConnectionRegistryFactory.getRegistry(conf);
+        this.registry = ConnectionRegistryFactory.getRegistry(conf, user);
       } else {
         this.registry = registry;
       }
@@ -326,14 +359,15 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
 
       if (conf.getBoolean(CLIENT_SIDE_METRICS_ENABLED_KEY, false)) {
         this.metricsScope = MetricsConnection.getScope(conf, clusterId, this);
-        this.metrics = MetricsConnection.getMetricsConnection(this.metricsScope, this::getBatchPool,
-          this::getMetaLookupPool);
+        this.metrics = MetricsConnection.getMetricsConnection(conf, this.metricsScope,
+          this::getBatchPool, this::getMetaLookupPool);
       } else {
         this.metrics = null;
       }
       this.metaCache = new MetaCache(this.metrics);
 
-      this.rpcClient = RpcClientFactory.createClient(this.conf, this.clusterId, this.metrics);
+      this.rpcClient = RpcClientFactory.createClient(this.conf, this.clusterId, this.metrics,
+        connectionAttributes);
       this.rpcControllerFactory = RpcControllerFactory.instantiate(conf);
       this.rpcCallerFactory = RpcRetryingCallerFactory.instantiate(conf, connectionConfig,
         interceptor, this.stats, this.metrics);
@@ -403,7 +437,7 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
     if (masterStateCacheTimeout <= 0L) {
       this.masterStateSupplier = masterConnSupplier;
     } else {
-      this.masterStateSupplier = Suppliers.memoizeWithExpiration(masterConnSupplier,
+      this.masterStateSupplier = Suppliers.memoizeWithExpiration(masterConnSupplier::get,
         masterStateCacheTimeout, TimeUnit.SECONDS);
     }
   }
@@ -461,7 +495,7 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
       @Override
       public Table build() {
         return new HTable(ConnectionImplementation.this, this, rpcCallerFactory,
-          rpcControllerFactory, pool);
+          rpcControllerFactory, pool, requestAttributes);
       }
     };
   }
@@ -1019,9 +1053,10 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
           .setTableName(TableName.META_TABLE_NAME).setOperation(s).build();
         try (Scope ignored = span.makeCurrent();
           ReversedClientScanner rcs =
-            new ReversedClientScanner(conf, s, TableName.META_TABLE_NAME, this, rpcCallerFactory,
+            new ReversedClientScanner(conf, s, s, TableName.META_TABLE_NAME, this, rpcCallerFactory,
               rpcControllerFactory, getMetaLookupPool(), connectionConfig.getMetaReadRpcTimeout(),
-              connectionConfig.getMetaScanTimeout(), metaReplicaCallTimeoutScanInMicroSecond)) {
+              connectionConfig.getMetaScanTimeout(), metaReplicaCallTimeoutScanInMicroSecond,
+              connectionConfig, Collections.emptyMap())) {
           boolean tableNotFound = true;
           for (;;) {
             Result regionInfoRow = rcs.next();
@@ -1528,6 +1563,12 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
         return stub.splitRegion(controller, request);
       }
 
+      @Override
+      public MasterProtos.TruncateRegionResponse truncateRegion(RpcController controller,
+        MasterProtos.TruncateRegionRequest request) throws ServiceException {
+        return stub.truncateRegion(controller, request);
+      }
+
       @Override
       public MasterProtos.DeleteTableResponse deleteTable(RpcController controller,
         MasterProtos.DeleteTableRequest request) throws ServiceException {
@@ -1803,12 +1844,32 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
         return stub.getTableDescriptors(controller, request);
       }
 
+      @Override
+      public MasterProtos.ListTableDescriptorsByStateResponse listTableDescriptorsByState(
+        RpcController controller, MasterProtos.ListTableDescriptorsByStateRequest request)
+        throws ServiceException {
+        return stub.listTableDescriptorsByState(controller, request);
+      }
+
       @Override
       public MasterProtos.GetTableNamesResponse getTableNames(RpcController controller,
         MasterProtos.GetTableNamesRequest request) throws ServiceException {
         return stub.getTableNames(controller, request);
       }
 
+      @Override
+      public MasterProtos.FlushTableResponse flushTable(RpcController controller,
+        MasterProtos.FlushTableRequest request) throws ServiceException {
+        return stub.flushTable(controller, request);
+      }
+
+      @Override
+      public MasterProtos.ListTableNamesByStateResponse listTableNamesByState(
+        RpcController controller, MasterProtos.ListTableNamesByStateRequest request)
+        throws ServiceException {
+        return stub.listTableNamesByState(controller, request);
+      }
+
       @Override
       public MasterProtos.GetClusterStatusResponse getClusterStatus(RpcController controller,
         MasterProtos.GetClusterStatusRequest request) throws ServiceException {
@@ -1929,6 +1990,12 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
         return stub.listReplicationPeers(controller, request);
       }
 
+      @Override
+      public GetReplicationPeerStateResponse isReplicationPeerEnabled(RpcController controller,
+        GetReplicationPeerStateRequest request) throws ServiceException {
+        return stub.isReplicationPeerEnabled(controller, request);
+      }
+
       @Override
       public GetSpaceQuotaRegionSizesResponse getSpaceQuotaRegionSizes(RpcController controller,
         GetSpaceQuotaRegionSizesRequest request) throws ServiceException {
@@ -2014,6 +2081,27 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
         FlushMasterStoreRequest request) throws ServiceException {
         return stub.flushMasterStore(controller, request);
       }
+
+      @Override
+      public ReplicationPeerModificationSwitchResponse replicationPeerModificationSwitch(
+        RpcController controller, ReplicationPeerModificationSwitchRequest request)
+        throws ServiceException {
+        return stub.replicationPeerModificationSwitch(controller, request);
+      }
+
+      @Override
+      public GetReplicationPeerModificationProceduresResponse
+        getReplicationPeerModificationProcedures(RpcController controller,
+          GetReplicationPeerModificationProceduresRequest request) throws ServiceException {
+        return stub.getReplicationPeerModificationProcedures(controller, request);
+      }
+
+      @Override
+      public IsReplicationPeerModificationEnabledResponse isReplicationPeerModificationEnabled(
+        RpcController controller, IsReplicationPeerModificationEnabledRequest request)
+        throws ServiceException {
+        return stub.isReplicationPeerModificationEnabled(controller, request);
+      }
     };
   }
 
@@ -2281,6 +2369,39 @@ public class ConnectionImplementation implements ClusterConnection, Closeable {
     }
   }
 
+  @Override
+  public List<ServerName> getLiveRegionServers(Supplier<ServerName> masterAddrTracker, int count)
+    throws IOException {
+    RegionServerStatusService.BlockingInterface stub = RegionServerStatusService.newBlockingStub(
+      rpcClient.createBlockingRpcChannel(masterAddrTracker.get(), user, rpcTimeout));
+    GetLiveRegionServersResponse resp;
+    try {
+      resp = stub.getLiveRegionServers(null,
+        GetLiveRegionServersRequest.newBuilder().setCount(count).build());
+    } catch (ServiceException e) {
+      Throwable t = ConnectionUtils.translateException(e);
+      Throwables.propagateIfPossible(t, IOException.class);
+      throw new IOException(t);
+    }
+    return resp.getServerList().stream().map(ProtobufUtil::toServerName)
+      .collect(Collectors.toList());
+  }
+
+  @Override
+  public List<ServerName> getAllBootstrapNodes(ServerName regionServer) throws IOException {
+    BootstrapNodeService.BlockingInterface stub = BootstrapNodeService
+      .newBlockingStub(rpcClient.createBlockingRpcChannel(regionServer, user, rpcTimeout));
+    GetAllBootstrapNodesResponse resp;
+    try {
+      resp = stub.getAllBootstrapNodes(null, GetAllBootstrapNodesRequest.getDefaultInstance());
+    } catch (ServiceException e) {
+      Throwable t = ConnectionUtils.translateException(e);
+      Throwables.propagateIfPossible(t, IOException.class);
+      throw new IOException(t);
+    }
+    return resp.getNodeList().stream().map(ProtobufUtil::toServerName).collect(Collectors.toList());
+  }
+
   @Override
   public String getClusterId() {
     try {
