@@ -17,19 +17,32 @@
  */
 package org.apache.hadoop.hbase.client;
 
+import static org.apache.hadoop.hbase.trace.HBaseSemanticAttributes.REGION_NAMES_KEY;
+
+import io.opentelemetry.api.trace.Span;
+import io.opentelemetry.api.trace.StatusCode;
+import io.opentelemetry.context.Scope;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
+import java.util.Objects;
+import java.util.function.Function;
+import java.util.function.Supplier;
+import java.util.stream.Collectors;
 import org.apache.hadoop.hbase.CatalogReplicaMode;
 import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.HRegionLocation;
 import org.apache.hadoop.hbase.MetaTableAccessor;
 import org.apache.hadoop.hbase.RegionLocations;
 import org.apache.hadoop.hbase.TableName;
+import org.apache.hadoop.hbase.client.trace.TableSpanBuilder;
+import org.apache.hadoop.hbase.trace.TraceUtil;
 import org.apache.yetus.audience.InterfaceAudience;
 
+import org.apache.hbase.thirdparty.org.apache.commons.collections4.CollectionUtils;
+
 /**
  * An implementation of {@link RegionLocator}. Used to view region location information for a single
  * HBase table. Lightweight. Get as needed and just close when done. Instances of this class SHOULD
@@ -61,36 +74,56 @@ public class HRegionLocator implements RegionLocator {
   @Override
   public HRegionLocation getRegionLocation(byte[] row, int replicaId, boolean reload)
     throws IOException {
-    return connection.locateRegion(tableName, row, !reload, true, replicaId)
-      .getRegionLocation(replicaId);
+    final Supplier<Span> supplier = new TableSpanBuilder(connection)
+      .setName("HRegionLocator.getRegionLocation").setTableName(tableName);
+    return tracedLocationFuture(() -> connection
+      .locateRegion(tableName, row, !reload, true, replicaId).getRegionLocation(replicaId),
+      AsyncRegionLocator::getRegionNames, supplier);
   }
 
   @Override
   public List<HRegionLocation> getRegionLocations(byte[] row, boolean reload) throws IOException {
-    RegionLocations locs =
-      connection.locateRegion(tableName, row, !reload, true, RegionInfo.DEFAULT_REPLICA_ID);
+    final Supplier<Span> supplier = new TableSpanBuilder(connection)
+      .setName("HRegionLocator.getRegionLocations").setTableName(tableName);
+    final RegionLocations locs = tracedLocationFuture(
+      () -> connection.locateRegion(tableName, row, !reload, true, RegionInfo.DEFAULT_REPLICA_ID),
+      AsyncRegionLocator::getRegionNames, supplier);
     return Arrays.asList(locs.getRegionLocations());
   }
 
   @Override
   public List<HRegionLocation> getAllRegionLocations() throws IOException {
-    ArrayList<HRegionLocation> regions = new ArrayList<>();
-    for (RegionLocations locations : listRegionLocations()) {
-      for (HRegionLocation location : locations.getRegionLocations()) {
-        regions.add(location);
-      }
-      RegionLocations cleaned = locations.removeElementsWithNullLocation();
-      // above can return null if all locations had null location
-      if (cleaned != null) {
-        connection.cacheLocation(tableName, cleaned);
+    final Supplier<Span> supplier = new TableSpanBuilder(connection)
+      .setName("HRegionLocator.getAllRegionLocations").setTableName(tableName);
+    return tracedLocationFuture(() -> {
+      ArrayList<HRegionLocation> regions = new ArrayList<>();
+      for (RegionLocations locations : listRegionLocations()) {
+        for (HRegionLocation location : locations.getRegionLocations()) {
+          regions.add(location);
+        }
+        RegionLocations cleaned = locations.removeElementsWithNullLocation();
+        // above can return null if all locations had null location
+        if (cleaned != null) {
+          connection.cacheLocation(tableName, cleaned);
+        }
       }
+      return regions;
+    }, HRegionLocator::getRegionNames, supplier);
+  }
+
+  private static List<String> getRegionNames(List<HRegionLocation> locations) {
+    if (CollectionUtils.isEmpty(locations)) {
+      return Collections.emptyList();
     }
-    return regions;
+    return locations.stream().filter(Objects::nonNull).map(AsyncRegionLocator::getRegionNames)
+      .filter(Objects::nonNull).flatMap(List::stream).collect(Collectors.toList());
   }
 
   @Override
   public void clearRegionLocationCache() {
-    connection.clearRegionCache(tableName);
+    final Supplier<Span> supplier = new TableSpanBuilder(connection)
+      .setName("HRegionLocator.clearRegionLocationCache").setTableName(tableName);
+    TraceUtil.trace(() -> connection.clearRegionCache(tableName), supplier);
   }
 
   @Override
@@ -98,6 +131,7 @@ public class HRegionLocator implements RegionLocator {
     return this.tableName;
   }
 
+  @SuppressWarnings("MixedMutabilityReturnType")
   private List<RegionLocations> listRegionLocations() throws IOException {
     if (TableName.isMetaTableName(tableName)) {
       return Collections
@@ -120,4 +154,23 @@ public class HRegionLocator implements RegionLocator {
     MetaTableAccessor.scanMetaForTableRegions(connection, visitor, tableName, metaReplicaMode);
     return regions;
   }
+
+  private <R, T extends Throwable> R tracedLocationFuture(TraceUtil.ThrowingCallable<R, T> action,
+    Function<R, List<String>> getRegionNames, Supplier<Span> spanSupplier) throws T {
+    final Span span = spanSupplier.get();
+    try (Scope ignored = span.makeCurrent()) {
+      final R result = action.call();
+      final List<String> regionNames = getRegionNames.apply(result);
+      if (!CollectionUtils.isEmpty(regionNames)) {
+        span.setAttribute(REGION_NAMES_KEY, regionNames);
+      }
+      span.setStatus(StatusCode.OK);
+      span.end();
+      return result;
+    } catch (Throwable e) {
+      TraceUtil.setError(span, e);
+      span.end();
+      throw e;
+    }
+  }
 }
