@@ -286,6 +286,7 @@ public class Scan extends Query {
     setPriority(scan.getPriority());
     readType = scan.getReadType();
     super.setReplicaId(scan.getReplicaId());
+    super.setQueryMetricsEnabled(scan.isQueryMetricsEnabled());
   }
 
   /**
@@ -318,6 +319,7 @@ public class Scan extends Query {
     this.mvccReadPoint = -1L;
     setPriority(get.getPriority());
     super.setReplicaId(get.getReplicaId());
+    super.setQueryMetricsEnabled(get.isQueryMetricsEnabled());
   }
 
   public boolean isGetScan() {
@@ -985,6 +987,7 @@ public class Scan extends Query {
       map.put("colFamTimeRangeMap", colFamTimeRangeMapStr);
     }
     map.put("priority", getPriority());
+    map.put("queryMetricsEnabled", queryMetricsEnabled);
     return map;
   }
 
