@@ -64,8 +64,10 @@ public class BinaryComparator extends org.apache.hadoop.hbase.filter.ByteArrayCo
   }
 
   /**
+   * Parse a serialized representation of {@link BinaryComparator}
    * @param pbBytes A pb serialized {@link BinaryComparator} instance
    * @return An instance of {@link BinaryComparator} made from <code>bytes</code>
+   * @throws DeserializationException if an error occurred
    * @see #toByteArray
    */
   public static BinaryComparator parseFrom(final byte[] pbBytes) throws DeserializationException {
@@ -79,14 +81,17 @@ public class BinaryComparator extends org.apache.hadoop.hbase.filter.ByteArrayCo
   }
 
   /**
-   * @return true if and only if the fields of the comparator that are serialized are equal to the
-   *         corresponding fields in other. Used for testing.
+   * Returns true if and only if the fields of the comparator that are serialized are equal to the
+   * corresponding fields in other. Used for testing.
    */
   @Override
   boolean areSerializedFieldsEqual(ByteArrayComparable other) {
-    if (other == this) return true;
-    if (!(other instanceof BinaryComparator)) return false;
-
+    if (other == this) {
+      return true;
+    }
+    if (!(other instanceof BinaryComparator)) {
+      return false;
+    }
     return super.areSerializedFieldsEqual(other);
   }
 }
