@@ -42,7 +42,7 @@ import org.apache.hadoop.hbase.shaded.protobuf.generated.FilterProtos;
  * with columns like 'ball', 'act'.
  */
 @InterfaceAudience.Public
-public class MultipleColumnPrefixFilter extends FilterBase {
+public class MultipleColumnPrefixFilter extends FilterBase implements HintingFilter {
   private static final Logger LOG = LoggerFactory.getLogger(MultipleColumnPrefixFilter.class);
   protected byte[] hint = null;
   protected TreeSet<byte[]> sortedPrefixes = createTreeSet();
@@ -134,8 +134,10 @@ public class MultipleColumnPrefixFilter extends FilterBase {
   }
 
   /**
+   * Parse a serialized representation of {@link MultipleColumnPrefixFilter}
    * @param pbBytes A pb serialized {@link MultipleColumnPrefixFilter} instance
    * @return An instance of {@link MultipleColumnPrefixFilter} made from <code>bytes</code>
+   * @throws DeserializationException if an error occurred
    * @see #toByteArray
    */
   public static MultipleColumnPrefixFilter parseFrom(final byte[] pbBytes)
@@ -156,15 +158,17 @@ public class MultipleColumnPrefixFilter extends FilterBase {
   }
 
   /**
-   * @param o the other filter to compare with
-   * @return true if and only if the fields of the filter that are serialized are equal to the
-   *         corresponding fields in other. Used for testing.
+   * Returns true if and only if the fields of the filter that are serialized are equal to the
+   * corresponding fields in other. Used for testing.
    */
   @Override
   boolean areSerializedFieldsEqual(Filter o) {
-    if (o == this) return true;
-    if (!(o instanceof MultipleColumnPrefixFilter)) return false;
-
+    if (o == this) {
+      return true;
+    }
+    if (!(o instanceof MultipleColumnPrefixFilter)) {
+      return false;
+    }
     MultipleColumnPrefixFilter other = (MultipleColumnPrefixFilter) o;
     return this.sortedPrefixes.equals(other.sortedPrefixes);
   }
