@@ -18,6 +18,8 @@
 package org.apache.hadoop.hbase.ipc;
 
 import java.net.SocketAddress;
+import java.util.Collections;
+import java.util.Map;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.client.MetricsConnection;
 import org.apache.hadoop.hbase.util.ReflectionUtils;
@@ -59,7 +61,7 @@ public final class RpcClientFactory {
    */
   public static RpcClient createClient(Configuration conf, String clusterId,
     MetricsConnection metrics) {
-    return createClient(conf, clusterId, null, metrics);
+    return createClient(conf, clusterId, metrics, Collections.emptyMap());
   }
 
   private static String getRpcClientClass(Configuration conf) {
@@ -74,17 +76,33 @@ public final class RpcClientFactory {
   /**
    * Creates a new RpcClient by the class defined in the configuration or falls back to
    * RpcClientImpl
-   * @param conf      configuration
-   * @param clusterId the cluster id
-   * @param localAddr client socket bind address.
-   * @param metrics   the connection metrics
+   * @param conf                 configuration
+   * @param clusterId            the cluster id
+   * @param metrics              the connection metrics
+   * @param connectionAttributes attributes to be sent along to server during connection establish
    * @return newly created RpcClient
    */
   public static RpcClient createClient(Configuration conf, String clusterId,
-    SocketAddress localAddr, MetricsConnection metrics) {
+    MetricsConnection metrics, Map<String, byte[]> connectionAttributes) {
+    return createClient(conf, clusterId, null, metrics, connectionAttributes);
+  }
+
+  /**
+   * Creates a new RpcClient by the class defined in the configuration or falls back to
+   * RpcClientImpl
+   * @param conf                 configuration
+   * @param clusterId            the cluster id
+   * @param localAddr            client socket bind address.
+   * @param metrics              the connection metrics
+   * @param connectionAttributes attributes to be sent along to server during connection establish
+   * @return newly created RpcClient
+   */
+  public static RpcClient createClient(Configuration conf, String clusterId,
+    SocketAddress localAddr, MetricsConnection metrics, Map<String, byte[]> connectionAttributes) {
     String rpcClientClass = getRpcClientClass(conf);
-    return ReflectionUtils.instantiateWithCustomCtor(rpcClientClass, new Class[] {
-      Configuration.class, String.class, SocketAddress.class, MetricsConnection.class },
-      new Object[] { conf, clusterId, localAddr, metrics });
+    return ReflectionUtils.instantiateWithCustomCtor(
+      rpcClientClass, new Class[] { Configuration.class, String.class, SocketAddress.class,
+        MetricsConnection.class, Map.class },
+      new Object[] { conf, clusterId, localAddr, metrics, connectionAttributes });
   }
 }
