@@ -36,6 +36,7 @@ import org.apache.hadoop.hbase.HBaseConfiguration;
 import org.apache.hadoop.hbase.ServerName;
 import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.ipc.HBaseRpcController;
+import org.apache.hadoop.hbase.security.User;
 import org.apache.hadoop.hbase.security.UserProvider;
 import org.apache.hadoop.hbase.testclassification.ClientTests;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
@@ -142,19 +143,20 @@ public class TestAsyncAdminRpcPriority {
     }).when(adminStub).stopServer(any(HBaseRpcController.class), any(StopServerRequest.class),
       any());
 
-    conn = new AsyncConnectionImpl(CONF, new DoNothingConnectionRegistry(CONF), "test",
-      UserProvider.instantiate(CONF).getCurrent()) {
+    User user = UserProvider.instantiate(CONF).getCurrent();
+    conn =
+      new AsyncConnectionImpl(CONF, new DoNothingConnectionRegistry(CONF, user), "test", user) {
 
-      @Override
-      CompletableFuture<MasterService.Interface> getMasterStub() {
-        return CompletableFuture.completedFuture(masterStub);
-      }
+        @Override
+        CompletableFuture<MasterService.Interface> getMasterStub() {
+          return CompletableFuture.completedFuture(masterStub);
+        }
 
-      @Override
-      Interface getAdminStub(ServerName serverName) throws IOException {
-        return adminStub;
-      }
-    };
+        @Override
+        Interface getAdminStub(ServerName serverName) throws IOException {
+          return adminStub;
+        }
+      };
   }
 
   private HBaseRpcController assertPriority(int priority) {
