@@ -138,8 +138,8 @@ public class TestClientNoCluster extends Configured implements Tool {
   static class SimpleRegistry extends DoNothingConnectionRegistry {
     final ServerName META_HOST = META_SERVERNAME;
 
-    public SimpleRegistry(Configuration conf) {
-      super(conf);
+    public SimpleRegistry(Configuration conf, User user) {
+      super(conf, user);
     }
 
     @Override
@@ -329,9 +329,9 @@ public class TestClientNoCluster extends Configured implements Tool {
   static class RegionServerStoppedOnScannerOpenConnection extends ConnectionImplementation {
     final ClientService.BlockingInterface stub;
 
-    RegionServerStoppedOnScannerOpenConnection(Configuration conf, ExecutorService pool, User user)
-      throws IOException {
-      super(conf, pool, user);
+    RegionServerStoppedOnScannerOpenConnection(Configuration conf, ExecutorService pool, User user,
+      Map<String, byte[]> requestAttributes) throws IOException {
+      super(conf, pool, user, requestAttributes);
       // Mock up my stub so open scanner returns a scanner id and then on next, we throw
       // exceptions for three times and then after that, we return no more to scan.
       this.stub = Mockito.mock(ClientService.BlockingInterface.class);
@@ -360,8 +360,9 @@ public class TestClientNoCluster extends Configured implements Tool {
   static class RpcTimeoutConnection extends ConnectionImplementation {
     final ClientService.BlockingInterface stub;
 
-    RpcTimeoutConnection(Configuration conf, ExecutorService pool, User user) throws IOException {
-      super(conf, pool, user);
+    RpcTimeoutConnection(Configuration conf, ExecutorService pool, User user,
+      Map<String, byte[]> requestAttributes) throws IOException {
+      super(conf, pool, user, requestAttributes);
       // Mock up my stub so an exists call -- which turns into a get -- throws an exception
       this.stub = Mockito.mock(ClientService.BlockingInterface.class);
       try {
@@ -384,8 +385,8 @@ public class TestClientNoCluster extends Configured implements Tool {
    */
   static class RpcTimeoutAsyncConnection extends AsyncConnectionImpl {
     RpcTimeoutAsyncConnection(Configuration configuration, ConnectionRegistry registry,
-      String clusterId, User user) {
-      super(configuration, registry, clusterId, user);
+      String clusterId, User user, Map<String, byte[]> connectionAttributes) {
+      super(configuration, registry, clusterId, user, connectionAttributes);
     }
   }
 
@@ -403,9 +404,9 @@ public class TestClientNoCluster extends Configured implements Tool {
     final AtomicLong sequenceids = new AtomicLong(0);
     private final Configuration conf;
 
-    ManyServersManyRegionsConnection(Configuration conf, ExecutorService pool, User user)
-      throws IOException {
-      super(conf, pool, user);
+    ManyServersManyRegionsConnection(Configuration conf, ExecutorService pool, User user,
+      Map<String, byte[]> requestAttributes) throws IOException {
+      super(conf, pool, user, requestAttributes);
       int serverCount = conf.getInt("hbase.test.servers", 10);
       this.serversByClient = new HashMap<>(serverCount);
       this.meta =
