@@ -101,7 +101,7 @@ public class CellComparatorImpl implements CellComparator {
     return ignoreSequenceid ? diff : Long.compare(r.getSequenceId(), l.getSequenceId());
   }
 
-  private static int compareKeyValues(final KeyValue left, final KeyValue right) {
+  private int compareKeyValues(final KeyValue left, final KeyValue right) {
     int diff;
     // Compare Rows. Cache row length.
     int leftRowLength = left.getRowLength();
@@ -154,8 +154,8 @@ public class CellComparatorImpl implements CellComparator {
     // Compare families.
     int leftFamilyPosition = left.getFamilyOffset(leftFamilyLengthPosition);
     int rightFamilyPosition = right.getFamilyOffset(rightFamilyLengthPosition);
-    diff = Bytes.compareTo(left.getFamilyArray(), leftFamilyPosition, leftFamilyLength,
-      right.getFamilyArray(), rightFamilyPosition, rightFamilyLength);
+    diff = compareFamilies(left, leftFamilyPosition, leftFamilyLength, right, rightFamilyPosition,
+      rightFamilyLength);
     if (diff != 0) {
       return diff;
     }
@@ -184,7 +184,7 @@ public class CellComparatorImpl implements CellComparator {
     return (0xff & rightType) - (0xff & leftType);
   }
 
-  private static int compareBBKV(final ByteBufferKeyValue left, final ByteBufferKeyValue right) {
+  private int compareBBKV(final ByteBufferKeyValue left, final ByteBufferKeyValue right) {
     int diff;
     // Compare Rows. Cache row length.
     int leftRowLength = left.getRowLength();
@@ -237,8 +237,8 @@ public class CellComparatorImpl implements CellComparator {
     // Compare families.
     int leftFamilyPosition = left.getFamilyPosition(leftFamilyLengthPosition);
     int rightFamilyPosition = right.getFamilyPosition(rightFamilyLengthPosition);
-    diff = ByteBufferUtils.compareTo(left.getFamilyByteBuffer(), leftFamilyPosition,
-      leftFamilyLength, right.getFamilyByteBuffer(), rightFamilyPosition, rightFamilyLength);
+    diff = compareFamilies(left, leftFamilyPosition, leftFamilyLength, right, rightFamilyPosition,
+      rightFamilyLength);
     if (diff != 0) {
       return diff;
     }
@@ -266,7 +266,7 @@ public class CellComparatorImpl implements CellComparator {
     return (0xff & rightType) - (0xff & leftType);
   }
 
-  private static int compareKVVsBBKV(final KeyValue left, final ByteBufferKeyValue right) {
+  private int compareKVVsBBKV(final KeyValue left, final ByteBufferKeyValue right) {
     int diff;
     // Compare Rows. Cache row length.
     int leftRowLength = left.getRowLength();
@@ -319,8 +319,8 @@ public class CellComparatorImpl implements CellComparator {
     // Compare families.
     int leftFamilyPosition = left.getFamilyOffset(leftFamilyLengthPosition);
     int rightFamilyPosition = right.getFamilyPosition(rightFamilyLengthPosition);
-    diff = ByteBufferUtils.compareTo(left.getFamilyArray(), leftFamilyPosition, leftFamilyLength,
-      right.getFamilyByteBuffer(), rightFamilyPosition, rightFamilyLength);
+    diff = compareFamilies(left, leftFamilyPosition, leftFamilyLength, right, rightFamilyPosition,
+      rightFamilyLength);
     if (diff != 0) {
       return diff;
     }
@@ -369,7 +369,10 @@ public class CellComparatorImpl implements CellComparator {
     return compareQualifiers(left, leftQualLen, right, rightQualLen);
   }
 
-  private int compareFamilies(Cell left, int leftFamLen, Cell right, int rightFamLen) {
+  /**
+   * This method will be overridden when we compare cells inner store to bypass family comparing.
+   */
+  protected int compareFamilies(Cell left, int leftFamLen, Cell right, int rightFamLen) {
     if (left instanceof ByteBufferExtendedCell && right instanceof ByteBufferExtendedCell) {
       return ByteBufferUtils.compareTo(((ByteBufferExtendedCell) left).getFamilyByteBuffer(),
         ((ByteBufferExtendedCell) left).getFamilyPosition(), leftFamLen,
@@ -449,6 +452,34 @@ public class CellComparatorImpl implements CellComparator {
       right.getFamilyArray(), right.getFamilyOffset(), right.getFamilyLength());
   }
 
+  /**
+   * This method will be overridden when we compare cells inner store to bypass family comparing.
+   */
+  protected int compareFamilies(KeyValue left, int leftFamilyPosition, int leftFamilyLength,
+    KeyValue right, int rightFamilyPosition, int rightFamilyLength) {
+    return Bytes.compareTo(left.getFamilyArray(), leftFamilyPosition, leftFamilyLength,
+      right.getFamilyArray(), rightFamilyPosition, rightFamilyLength);
+  }
+
+  /**
+   * This method will be overridden when we compare cells inner store to bypass family comparing.
+   */
+  protected int compareFamilies(ByteBufferKeyValue left, int leftFamilyPosition,
+    int leftFamilyLength, ByteBufferKeyValue right, int rightFamilyPosition,
+    int rightFamilyLength) {
+    return ByteBufferUtils.compareTo(left.getFamilyByteBuffer(), leftFamilyPosition,
+      leftFamilyLength, right.getFamilyByteBuffer(), rightFamilyPosition, rightFamilyLength);
+  }
+
+  /**
+   * This method will be overridden when we compare cells inner store to bypass family comparing.
+   */
+  protected int compareFamilies(KeyValue left, int leftFamilyPosition, int leftFamilyLength,
+    ByteBufferKeyValue right, int rightFamilyPosition, int rightFamilyLength) {
+    return ByteBufferUtils.compareTo(left.getFamilyArray(), leftFamilyPosition, leftFamilyLength,
+      right.getFamilyByteBuffer(), rightFamilyPosition, rightFamilyLength);
+  }
+
   static int compareQualifiers(KeyValue left, KeyValue right) {
     // NOTE: Same method is in CellComparatorImpl, also private, not shared, intentionally. Not
     // sharing gets us a few percent more throughput in compares. If changes here or there, make
