@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -18,21 +18,20 @@
 package org.apache.hadoop.hbase.conf;
 
 import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
-import org.apache.hadoop.hbase.classification.InterfaceStability;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.apache.yetus.audience.InterfaceStability;
 
 /**
- * Every class that wants to observe changes in Configuration properties,
- * must implement interface (and also, register itself with the
- * <code>ConfigurationManager</code> object.
+ * Every class that wants to observe changes in Configuration properties, must implement interface
+ * (and also, register itself with the {@link ConfigurationManager}.
  */
 @InterfaceAudience.Private
 @InterfaceStability.Evolving
 public interface ConfigurationObserver {
 
   /**
-   * This method would be called by the {@link ConfigurationManager}
-   * object when the {@link Configuration} object is reloaded from disk.
+   * This method would be called by the {@link ConfigurationManager} object when the
+   * {@link Configuration} object is reloaded from disk.
    */
   void onConfigurationChange(Configuration conf);
 }
