@@ -29,7 +29,6 @@ import org.apache.hadoop.hbase.CellUtil;
 import org.apache.hadoop.hbase.ExtendedCell;
 import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.KeyValue;
-import org.apache.hadoop.hbase.KeyValue.Type;
 import org.apache.hadoop.hbase.KeyValueUtil;
 import org.apache.hadoop.hbase.PrivateCellUtil;
 import org.apache.hadoop.hbase.io.TagCompressionContext;
@@ -358,14 +357,14 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
   protected static class OnheapDecodedCell implements ExtendedCell {
     private static final long FIXED_OVERHEAD = ClassSize.align(ClassSize.OBJECT
       + (3 * ClassSize.REFERENCE) + (2 * Bytes.SIZEOF_LONG) + (7 * Bytes.SIZEOF_INT)
-      + (Bytes.SIZEOF_SHORT) + (2 * Bytes.SIZEOF_BYTE) + (3 * ClassSize.ARRAY));
+      + Bytes.SIZEOF_SHORT + (2 * Bytes.SIZEOF_BYTE) + (3 * ClassSize.ARRAY));
     private byte[] keyOnlyBuffer;
     private short rowLength;
     private int familyOffset;
     private byte familyLength;
     private int qualifierOffset;
     private int qualifierLength;
-    private long timestamp;
+    private long timeStamp;
     private byte typeByte;
     private byte[] valueBuffer;
     private int valueOffset;
@@ -385,7 +384,7 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
       this.familyLength = familyLength;
       this.qualifierOffset = qualOffset;
       this.qualifierLength = qualLength;
-      this.timestamp = timeStamp;
+      this.timeStamp = timeStamp;
       this.typeByte = typeByte;
       this.valueBuffer = valueBuffer;
       this.valueOffset = valueOffset;
@@ -443,7 +442,7 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
 
     @Override
     public long getTimestamp() {
-      return timestamp;
+      return timeStamp;
     }
 
     @Override
@@ -554,16 +553,17 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
   }
 
   protected static class OffheapDecodedExtendedCell extends ByteBufferExtendedCell {
-    private static final long FIXED_OVERHEAD = ClassSize.align(ClassSize.OBJECT
-      + (3 * ClassSize.REFERENCE) + (2 * Bytes.SIZEOF_LONG) + (7 * Bytes.SIZEOF_INT)
-      + (Bytes.SIZEOF_SHORT) + (2 * Bytes.SIZEOF_BYTE) + (3 * ClassSize.BYTE_BUFFER));
+    private static final long FIXED_OVERHEAD =
+      (long) ClassSize.align(ClassSize.OBJECT + (3 * ClassSize.REFERENCE) + (2 * Bytes.SIZEOF_LONG)
+        + (7 * Bytes.SIZEOF_INT) + Bytes.SIZEOF_SHORT) + (2 * Bytes.SIZEOF_BYTE)
+        + (3 * ClassSize.BYTE_BUFFER);
     private ByteBuffer keyBuffer;
     private short rowLength;
     private int familyOffset;
     private byte familyLength;
     private int qualifierOffset;
     private int qualifierLength;
-    private long timestamp;
+    private long timeStamp;
     private byte typeByte;
     private ByteBuffer valueBuffer;
     private int valueOffset;
@@ -586,7 +586,7 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
       this.familyLength = familyLength;
       this.qualifierOffset = qualOffset;
       this.qualifierLength = qualLength;
-      this.timestamp = timeStamp;
+      this.timeStamp = timeStamp;
       this.typeByte = typeByte;
       this.valueBuffer = valueBuffer;
       this.valueOffset = valueOffset;
@@ -598,6 +598,7 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
     }
 
     @Override
+    @SuppressWarnings("ByteBufferBackingArray")
     public byte[] getRowArray() {
       return this.keyBuffer.array();
     }
@@ -613,6 +614,7 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
     }
 
     @Override
+    @SuppressWarnings("ByteBufferBackingArray")
     public byte[] getFamilyArray() {
       return this.keyBuffer.array();
     }
@@ -628,6 +630,7 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
     }
 
     @Override
+    @SuppressWarnings("ByteBufferBackingArray")
     public byte[] getQualifierArray() {
       return this.keyBuffer.array();
     }
@@ -644,7 +647,7 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
 
     @Override
     public long getTimestamp() {
-      return this.timestamp;
+      return this.timeStamp;
     }
 
     @Override
@@ -750,10 +753,10 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
     @Override
     public int write(OutputStream out, boolean withTags) throws IOException {
       int lenToWrite = getSerializedSize(withTags);
-      ByteBufferUtils.putInt(out, keyBuffer.capacity());
+      ByteBufferUtils.putInt(out, keyBuffer.remaining());
       ByteBufferUtils.putInt(out, valueLength);
       // Write key
-      out.write(keyBuffer.array());
+      out.write(keyBuffer.array(), keyBuffer.arrayOffset(), keyBuffer.remaining());
       // Write value
       ByteBufferUtils.copyBufferToStream(out, this.valueBuffer, this.valueOffset, this.valueLength);
       if (withTags && this.tagsLength > 0) {
@@ -1020,14 +1023,14 @@ abstract class BufferedDataBlockEncoder extends AbstractDataBlockEncoder {
     private int compareTypeBytes(Cell key, Cell right) {
       if (
         key.getFamilyLength() + key.getQualifierLength() == 0
-          && key.getTypeByte() == Type.Minimum.getCode()
+          && key.getTypeByte() == KeyValue.Type.Minimum.getCode()
       ) {
         // left is "bigger", i.e. it appears later in the sorted order
         return 1;
       }
       if (
         right.getFamilyLength() + right.getQualifierLength() == 0
-          && right.getTypeByte() == Type.Minimum.getCode()
+          && right.getTypeByte() == KeyValue.Type.Minimum.getCode()
       ) {
         return -1;
       }
