@@ -78,6 +78,7 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
     }
   }
 
+  @Override
   public int refCnt() {
     return refCnt.refCnt();
   }
@@ -113,28 +114,16 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
   /** Returns the total capacity of this ByteBuff. */
   public abstract int capacity();
 
-  /**
-   * Returns the limit of this ByteBuff
-   * @return limit of the ByteBuff
-   */
+  /** Returns the limit of this ByteBuff */
   public abstract int limit();
 
-  /**
-   * Marks the limit of this ByteBuff.
-   * @return This ByteBuff
-   */
+  /** Marks the limit of this ByteBuff */
   public abstract ByteBuff limit(int limit);
 
-  /**
-   * Rewinds this ByteBuff and the position is set to 0
-   * @return this object
-   */
+  /** Rewinds this ByteBuff and the position is set to 0 */
   public abstract ByteBuff rewind();
 
-  /**
-   * Marks the current position of the ByteBuff
-   * @return this object
-   */
+  /** Marks the current position of the ByteBuff */
   public abstract ByteBuff mark();
 
   /**
@@ -167,16 +156,10 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
    */
   public abstract void asSubByteBuffer(int offset, int length, ObjectIntPair<ByteBuffer> pair);
 
-  /**
-   * Returns the number of elements between the current position and the limit.
-   * @return the remaining elements in this ByteBuff
-   */
+  /** Returns the number of elements between the current position and the limit. */
   public abstract int remaining();
 
-  /**
-   * Returns true if there are elements between the current position and the limt
-   * @return true if there are elements, false otherwise
-   */
+  /** Returns true if there are elements between the current position and the limit. */
   public abstract boolean hasRemaining();
 
   /**
@@ -238,6 +221,7 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
   /**
    * Copies the specified number of bytes from this ByteBuff's current position to the byte[]'s
    * offset. Also advances the position of the ByteBuff by the given length.
+   * @param dst    the byte[] to which the ByteBuff's content is to be copied
    * @param offset within the current array
    * @param length upto which the bytes to be copied
    */
@@ -256,11 +240,13 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
   /**
    * Copies the content from this ByteBuff's current position to the byte array and fills it. Also
    * advances the position of the ByteBuff by the length of the byte[].
+   * @param dst the byte[] to which the ByteBuff's content is to be copied
    */
   public abstract void get(byte[] dst);
 
   /**
    * Copies from the given byte[] to this ByteBuff
+   * @param src    source byte array
    * @param offset the position in the byte array from which the copy should be done
    * @param length the length upto which the copy should happen
    * @return this ByteBuff
@@ -269,6 +255,7 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
 
   /**
    * Copies from the given byte[] to this ByteBuff
+   * @return this ByteBuff
    * @param src source byte array
    * @return this ByteBuff
    */
@@ -285,8 +272,7 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
 
   /**
    * Returns the short value at the current position. Also advances the position by the size of
-   * short
-   * @return the short value at the current position
+   * short.
    */
   public abstract short getShort();
 
@@ -307,45 +293,37 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
   public abstract short getShortAfterPosition(int offset);
 
   /**
-   * Returns the int value at the current position. Also advances the position by the size of int
-   * @return the int value at the current position
+   * Returns the int value at the current position. Also advances the position by the size of int.
    */
   public abstract int getInt();
 
   /**
    * Writes an int to this ByteBuff at its current position. Also advances the position by size of
-   * int
-   * @param value Int value to write
-   * @return this object
+   * int.
    */
   public abstract ByteBuff putInt(int value);
 
   /**
    * Fetches the int at the given index. Does not change position of the underlying ByteBuffers.
    * Even if the current int does not fit in the current item we can safely move to the next item
-   * and fetch the remaining bytes forming the int
-   * @return the int value at the given index
+   * and fetch the remaining bytes forming the int.
    */
   public abstract int getInt(int index);
 
   /**
    * Fetches the int value at the given offset from current position. Does not change position of
    * the underlying ByteBuffers.
-   * @return the int value at the given index.
    */
   public abstract int getIntAfterPosition(int offset);
 
   /**
-   * Returns the long value at the current position. Also advances the position by the size of long
-   * @return the long value at the current position
+   * Returns the long value at the current position. Also advances the position by the size of long.
    */
   public abstract long getLong();
 
   /**
    * Writes a long to this ByteBuff at its current position. Also advances the position by size of
-   * long
-   * @param value Long value to write
-   * @return this object
+   * long.
    */
   public abstract ByteBuff putLong(long value);
 
@@ -367,16 +345,13 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
 
   /**
    * Copy the content from this ByteBuff to a byte[].
-   * @return byte[] with the copied contents from this ByteBuff.
    */
   public byte[] toBytes() {
     return toBytes(0, this.limit());
   }
 
   /**
-   * Copy the content from this ByteBuff to a byte[] based on the given offset and length the
-   * position from where the copy should start the length upto which the copy has to be done
-   * @return byte[] with the copied contents from this ByteBuff.
+   * Copy the content from this ByteBuff to a byte[] based on the given offset and length.
    */
   public abstract byte[] toBytes(int offset, int length);
 
@@ -399,25 +374,16 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
    */
   public abstract ByteBuff put(int offset, ByteBuff src, int srcOffset, int length);
 
-  /**
-   * Reads bytes from the given channel into this ByteBuff
-   * @return The number of bytes read from the channel
-   */
+  /** Reads bytes from the given channel into this ByteBuf. */
   public abstract int read(ReadableByteChannel channel) throws IOException;
 
-  /**
-   * Reads bytes from FileChannel into this ByteBuff
-   */
+  /** Reads bytes from FileChannel into this ByteBuff */
   public abstract int read(FileChannel channel, long offset) throws IOException;
 
-  /**
-   * Write this ByteBuff's data into target file
-   */
+  /** Write this ByteBuff's data into target file */
   public abstract int write(FileChannel channel, long offset) throws IOException;
 
-  /**
-   * function interface for Channel read
-   */
+  /** Functional interface for Channel read */
   @FunctionalInterface
   interface ChannelReader {
     int read(ReadableByteChannel channel, ByteBuffer buf, long offset) throws IOException;
@@ -458,10 +424,7 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
     return (nBytes > 0) ? nBytes : ret;
   }
 
-  /**
-   * Read integer from ByteBuff coded in 7 bits and increment position.
-   * @return Read integer.
-   */
+  /** Read integer from ByteBuff coded in 7 bits and increment position. */
   public static int readCompressedInt(ByteBuff buf) {
     byte b = buf.get();
     if ((b & ByteBufferUtils.NEXT_BIT_MASK) != 0) {
@@ -580,9 +543,7 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
     return refCnt;
   }
 
-  /**
-   * Make this private because we don't want to expose the refCnt related wrap method to upstream.
-   */
+  // Make this private because we don't want to expose the refCnt related wrap method to upstream.
   private static ByteBuff wrap(List<ByteBuffer> buffers, RefCnt refCnt) {
     if (buffers == null || buffers.size() == 0) {
       throw new IllegalArgumentException("buffers shouldn't be null or empty");
@@ -592,9 +553,7 @@ public abstract class ByteBuff implements HBaseReferenceCounted {
       : new MultiByteBuff(refCnt, buffers.toArray(new ByteBuffer[0]));
   }
 
-  /**
-   * Make this private because we don't want to expose the refCnt related wrap method to upstream.
-   */
+  // Make this private because we don't want to expose the refCnt related wrap method to upstream.
   private static ByteBuff wrap(ByteBuffer buffer, RefCnt refCnt) {
     return new SingleByteBuff(refCnt, buffer);
   }
