@@ -24,7 +24,7 @@ import org.apache.yetus.audience.InterfaceAudience;
  * in a circle array.
  */
 @InterfaceAudience.Private
-public class WindowMovingAverage extends MovingAverage {
+public class WindowMovingAverage<T> extends MovingAverage<T> {
   protected final static int DEFAULT_SIZE = 5;
 
   // The last n statistics.
@@ -47,7 +47,7 @@ public class WindowMovingAverage extends MovingAverage {
 
   @Override
   protected void updateMostRecentTime(long elapsed) {
-    int index = moveForwardMostRecentPosistion();
+    int index = moveForwardMostRecentPosition();
     lastN[index] = elapsed;
   }
 
@@ -55,7 +55,7 @@ public class WindowMovingAverage extends MovingAverage {
   public double getAverageTime() {
     return enoughStatistics()
       ? (double) sum(getNumberOfStatistics()) / getNumberOfStatistics()
-      : (double) sum(getMostRecentPosistion() + 1) / (getMostRecentPosistion() + 1);
+      : (double) sum(getMostRecentPosition() + 1) / (getMostRecentPosition() + 1);
   }
 
   /**
@@ -84,7 +84,7 @@ public class WindowMovingAverage extends MovingAverage {
   }
 
   /** Returns index of most recent */
-  protected int getMostRecentPosistion() {
+  protected int getMostRecentPosition() {
     return mostRecent;
   }
 
@@ -92,7 +92,7 @@ public class WindowMovingAverage extends MovingAverage {
    * Move forward the most recent index.
    * @return the most recent index
    */
-  protected int moveForwardMostRecentPosistion() {
+  protected int moveForwardMostRecentPosition() {
     int index = ++mostRecent;
     if (!oneRound && index == getNumberOfStatistics()) {
       // Back to the head of the lastN, from now on will
