@@ -73,12 +73,10 @@ public class TestTableName {
       if (this == o) {
         return true;
       }
-      if (o == null || getClass() != o.getClass()) {
+      if (!(o instanceof Names)) {
         return false;
       }
-
       Names names = (Names) o;
-
       if (!ns.equals(names.ns)) {
         return false;
       }
@@ -135,7 +133,7 @@ public class TestTableName {
   @Test
   public void testIllegalHTableNames() {
     for (String tn : illegalTableNames) {
-      assertThrows(Exception.class,
+      assertThrows(IllegalArgumentException.class,
         () -> TableName.isLegalFullyQualifiedTableName(Bytes.toBytes(tn)));
     }
   }
