@@ -73,15 +73,12 @@ public class TestUnion2 {
     public int encodedLength(Object val) {
       Integer i = null;
       String s = null;
-      try {
+      if (val instanceof Integer) {
         i = (Integer) val;
-      } catch (ClassCastException ignored) {
       }
-      try {
+      if (val instanceof String) {
         s = (String) val;
-      } catch (ClassCastException ignored) {
       }
-
       if (null != i) {
         return 1 + typeA.encodedLength(i);
       }
@@ -89,7 +86,6 @@ public class TestUnion2 {
       if (null != s) {
         return 1 + typeB.encodedLength(s);
       }
-
       throw new IllegalArgumentException("val is not a valid member of this union.");
     }
 
@@ -97,15 +93,12 @@ public class TestUnion2 {
     public int encode(PositionedByteRange dst, Object val) {
       Integer i = null;
       String s = null;
-      try {
+      if (val instanceof Integer) {
         i = (Integer) val;
-      } catch (ClassCastException ignored) {
       }
-      try {
+      if (val instanceof String) {
         s = (String) val;
-      } catch (ClassCastException ignored) {
       }
-
       if (null != i) {
         dst.put(IS_INTEGER);
         return 1 + typeA.encode(dst, i);
