@@ -23,8 +23,8 @@ import com.google.protobuf.ServiceException;
 import java.io.File;
 import java.io.IOException;
 import java.security.PrivilegedExceptionAction;
+import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import org.apache.hadoop.conf.Configuration;
@@ -40,7 +40,6 @@ import org.apache.hadoop.hbase.client.ColumnFamilyDescriptorBuilder;
 import org.apache.hadoop.hbase.client.Connection;
 import org.apache.hadoop.hbase.client.ConnectionFactory;
 import org.apache.hadoop.hbase.client.Put;
-import org.apache.hadoop.hbase.client.Result;
 import org.apache.hadoop.hbase.client.ResultScanner;
 import org.apache.hadoop.hbase.client.Scan;
 import org.apache.hadoop.hbase.client.Table;
@@ -65,6 +64,7 @@ import org.apache.hadoop.hbase.security.visibility.VisibilityConstants;
 import org.apache.hadoop.hbase.security.visibility.VisibilityTestUtil;
 import org.apache.hadoop.hbase.testclassification.MediumTests;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.Pair;
 import org.apache.hadoop.minikdc.MiniKdc;
 import org.apache.hadoop.security.UserGroupInformation;
@@ -81,6 +81,8 @@ import org.junit.rules.TestName;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.hbase.thirdparty.com.google.common.collect.Iterables;
+
 @Category({ MediumTests.class })
 public class TestSecureExport {
   @ClassRule
@@ -103,7 +105,7 @@ public class TestSecureExport {
   private static final byte[] ROW3 = Bytes.toBytes("row3");
   private static final byte[] QUAL = Bytes.toBytes("qual");
   private static final String LOCALHOST = "localhost";
-  private static final long NOW = System.currentTimeMillis();
+  private static final long NOW = EnvironmentEdgeManager.currentTime();
   // user granted with all global permission
   private static final String USER_ADMIN = "admin";
   // user is table owner. will have all permissions on table
@@ -335,7 +337,7 @@ public class TestSecureExport {
       return null;
     };
     SecureTestUtil.verifyAllowed(putAction, getUserByLogin(USER_OWNER));
-    List<Pair<List<String>, Integer>> labelsAndRowCounts = new LinkedList<>();
+    List<Pair<List<String>, Integer>> labelsAndRowCounts = new ArrayList<>(5);
     labelsAndRowCounts.add(new Pair<>(Arrays.asList(SECRET), 1));
     labelsAndRowCounts.add(new Pair<>(Arrays.asList(PRIVATE, CONFIDENTIAL), 1));
     labelsAndRowCounts.add(new Pair<>(Arrays.asList(TOPSECRET), 1));
@@ -387,11 +389,7 @@ public class TestSecureExport {
         try (Connection conn = ConnectionFactory.createConnection(UTIL.getConfiguration());
           Table table = conn.getTable(importHtd.getTableName());
           ResultScanner scanner = table.getScanner(scan)) {
-          int count = 0;
-          for (Result r : scanner) {
-            ++count;
-          }
-          assertEquals(rowCount, count);
+          assertEquals(rowCount, Iterables.size(scanner));
         }
         return null;
       };
