@@ -17,6 +17,7 @@
  */
 package org.apache.hadoop.hbase.ipc;
 
+import org.apache.hadoop.hbase.util.Pair;
 import org.apache.yetus.audience.InterfaceAudience;
 
 @InterfaceAudience.Private
@@ -27,6 +28,8 @@ public interface MetricsHBaseServerWrapper {
 
   int getReplicationQueueLength();
 
+  int getBulkLoadQueueLength();
+
   int getPriorityQueueLength();
 
   int getMetaPriorityQueueLength();
@@ -41,6 +44,8 @@ public interface MetricsHBaseServerWrapper {
 
   int getActiveReplicationRpcHandlerCount();
 
+  int getActiveBulkLoadRpcHandlerCount();
+
   int getActiveMetaPriorityRpcHandlerCount();
 
   long getNumGeneralCallsDropped();
@@ -60,4 +65,10 @@ public interface MetricsHBaseServerWrapper {
   int getActiveScanRpcHandlerCount();
 
   long getNettyDmUsage();
+
+  /**
+   * These two metrics are calculated together, so we want to return them in one call
+   * @return pair containing total (first) and max (second) pending outbound bytes.
+   */
+  Pair<Long, Long> getTotalAndMaxNettyOutboundBytes();
 }
