@@ -43,7 +43,9 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
- * The implementation of the {@link Terminal} interface.
+ * An implementation of the {@link Terminal} interface for normal display mode. This implementation
+ * produces output intended for human viewing. In particular, it only displays one screenful of
+ * data. The output contains some escape sequences for formatting.
  */
 @InterfaceAudience.Private
 public class TerminalImpl implements Terminal {
@@ -211,7 +213,9 @@ public class TerminalImpl implements Terminal {
 
       try {
         process.waitFor();
-      } catch (InterruptedException ignored) {
+      } catch (InterruptedException e) {
+        // Restore interrupt status
+        Thread.currentThread().interrupt();
       }
 
       int exitValue = process.exitValue();
