@@ -54,9 +54,6 @@ import org.apache.hadoop.security.authorize.AccessControlList;
 import org.apache.hadoop.security.ssl.SSLFactory;
 import org.apache.hadoop.test.GenericTestUtils;
 import org.apache.hadoop.util.StringUtils;
-import org.apache.log4j.Level;
-import org.apache.log4j.LogManager;
-import org.apache.log4j.Logger;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
 import org.junit.ClassRule;
@@ -80,7 +77,8 @@ public class TestLogLevel {
   private static final String logName = TestLogLevel.class.getName();
   private static final String protectedPrefix = "protected";
   private static final String protectedLogName = protectedPrefix + "." + logName;
-  private static final Logger log = LogManager.getLogger(logName);
+  private static final org.apache.logging.log4j.Logger log =
+    org.apache.logging.log4j.LogManager.getLogger(logName);
   private final static String PRINCIPAL = "loglevel.principal";
   private final static String KEYTAB = "loglevel.keytab";
 
@@ -265,7 +263,8 @@ public class TestLogLevel {
 
   private void testDynamicLogLevel(final String bindProtocol, final String connectProtocol,
     final boolean isSpnego) throws Exception {
-    testDynamicLogLevel(bindProtocol, connectProtocol, isSpnego, logName, Level.DEBUG.toString());
+    testDynamicLogLevel(bindProtocol, connectProtocol, isSpnego, logName,
+      org.apache.logging.log4j.Level.DEBUG.toString());
   }
 
   private void testDynamicLogLevel(final String bindProtocol, final String connectProtocol,
@@ -288,9 +287,10 @@ public class TestLogLevel {
     if (!LogLevel.isValidProtocol(connectProtocol)) {
       throw new Exception("Invalid client protocol " + connectProtocol);
     }
-    Logger log = LogManager.getLogger(loggerName);
-    Level oldLevel = log.getLevel();
-    assertNotEquals("Get default Log Level which shouldn't be ERROR.", Level.ERROR, oldLevel);
+    org.apache.logging.log4j.Logger log = org.apache.logging.log4j.LogManager.getLogger(loggerName);
+    org.apache.logging.log4j.Level oldLevel = log.getLevel();
+    assertNotEquals("Get default Log Level which shouldn't be ERROR.",
+      org.apache.logging.log4j.Level.ERROR, oldLevel);
 
     // configs needed for SPNEGO at server side
     if (isSpnego) {
@@ -326,7 +326,7 @@ public class TestLogLevel {
     }
 
     // restore log level
-    GenericTestUtils.setLogLevel(log, oldLevel);
+    org.apache.logging.log4j.core.config.Configurator.setLevel(log.getName(), oldLevel);
   }
 
   /**
@@ -353,10 +353,10 @@ public class TestLogLevel {
     CLI cli = new CLI(protocol.equalsIgnoreCase("https") ? sslConf : clientConf);
     cli.run(setLevelArgs);
 
-    Logger log = LogManager.getLogger(logName);
+    org.apache.logging.log4j.Logger logger = org.apache.logging.log4j.LogManager.getLogger(logName);
 
     assertEquals("new level not equal to expected: ", newLevel.toUpperCase(),
-      log.getEffectiveLevel().toString());
+      logger.getLevel().toString());
   }
 
   @Test
