@@ -41,6 +41,7 @@ import org.apache.hadoop.hbase.client.Result;
 import org.apache.hadoop.hbase.client.Table;
 import org.apache.hadoop.hbase.regionserver.StorefileRefresherChore;
 import org.apache.hadoop.hbase.testclassification.IntegrationTests;
+import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.LoadTestTool;
 import org.apache.hadoop.hbase.util.MultiThreadedReader;
 import org.apache.hadoop.hbase.util.Threads;
@@ -143,7 +144,7 @@ public class IntegrationTestTimeBoundedRequestsWithRegionReplicas extends Integr
     LOG.info("Cluster size:"
       + util.getHBaseClusterInterface().getClusterMetrics().getLiveServerMetrics().size());
 
-    long start = System.currentTimeMillis();
+    long start = EnvironmentEdgeManager.currentTime();
     String runtimeKey = String.format(RUN_TIME_KEY, this.getClass().getSimpleName());
     long runtime = util.getConfiguration().getLong(runtimeKey, defaultRunTime);
     long startKey = 0;
@@ -198,7 +199,7 @@ public class IntegrationTestTimeBoundedRequestsWithRegionReplicas extends Integr
 
     // set the intended run time for the reader. The reader will do read requests
     // to random keys for this amount of time.
-    long remainingTime = runtime - (System.currentTimeMillis() - start);
+    long remainingTime = runtime - (EnvironmentEdgeManager.currentTime() - start);
     if (remainingTime <= 0) {
       LOG.error("The amount of time left for the test to perform random reads is "
         + "non-positive. Increase the test execution time via "
@@ -267,7 +268,8 @@ public class IntegrationTestTimeBoundedRequestsWithRegionReplicas extends Integr
       try {
         this.timeoutThread.join();
       } catch (InterruptedException e) {
-        e.printStackTrace();
+        // Restore interrupt status
+        Thread.currentThread().interrupt();
       }
       this.aborted = true;
       super.waitForFinish();
@@ -293,7 +295,7 @@ public class IntegrationTestTimeBoundedRequestsWithRegionReplicas extends Integr
       return new TimeBoundedMultiThreadedReaderThread(readerId);
     }
 
-    private class TimeoutThread extends Thread {
+    private static class TimeoutThread extends Thread {
       long timeout;
       long reportInterval = 60000;
 
