@@ -32,6 +32,7 @@ import org.apache.hadoop.conf.Configured;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hbase.Cell;
 import org.apache.hadoop.hbase.CellUtil;
+import org.apache.hadoop.hbase.ExtendedCell;
 import org.apache.hadoop.hbase.HBaseConfiguration;
 import org.apache.hadoop.hbase.KeyValue;
 import org.apache.hadoop.hbase.KeyValueUtil;
@@ -53,6 +54,7 @@ import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.MapReduceExtendedCell;
 import org.apache.hadoop.hbase.wal.WALEdit;
 import org.apache.hadoop.hbase.wal.WALKey;
+import org.apache.hadoop.io.WritableComparable;
 import org.apache.hadoop.mapreduce.Job;
 import org.apache.hadoop.mapreduce.Mapper;
 import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
@@ -138,9 +140,10 @@ public class WALPlayer extends Configured implements Tool {
   /**
    * A mapper that just writes out Cells. This one can be used together with {@link CellSortReducer}
    */
-  static class WALCellMapper extends Mapper<WALKey, WALEdit, ImmutableBytesWritable, Cell> {
+  static class WALCellMapper extends Mapper<WALKey, WALEdit, WritableComparable<?>, Cell> {
     private Set<String> tableSet = new HashSet<>();
     private boolean multiTableSupport = false;
+    private boolean diskBasedSortingEnabled = false;
 
     @Override
     public void map(WALKey key, WALEdit value, Context context) throws IOException {
@@ -161,7 +164,8 @@ public class WALPlayer extends Configured implements Tool {
             byte[] outKey = multiTableSupport
               ? Bytes.add(table.getName(), Bytes.toBytes(tableSeparator), CellUtil.cloneRow(cell))
               : CellUtil.cloneRow(cell);
-            context.write(new ImmutableBytesWritable(outKey), new MapReduceExtendedCell(cell));
+            ExtendedCell extendedCell = (ExtendedCell) cell;
+            context.write(wrapKey(outKey, extendedCell), new MapReduceExtendedCell(extendedCell));
           }
         }
       } catch (InterruptedException e) {
@@ -174,8 +178,23 @@ public class WALPlayer extends Configured implements Tool {
       Configuration conf = context.getConfiguration();
       String[] tables = conf.getStrings(TABLES_KEY);
       this.multiTableSupport = conf.getBoolean(MULTI_TABLES_SUPPORT, false);
+      this.diskBasedSortingEnabled = HFileOutputFormat2.diskBasedSortingEnabled(conf);
       Collections.addAll(tableSet, tables);
     }
+
+    private WritableComparable<?> wrapKey(byte[] key, ExtendedCell cell) {
+      if (this.diskBasedSortingEnabled) {
+        // Important to build a new cell with the updated key to maintain multi-table support
+        KeyValue kv = new KeyValue(key, 0, key.length, cell.getFamilyArray(),
+          cell.getFamilyOffset(), cell.getFamilyLength(), cell.getQualifierArray(),
+          cell.getQualifierOffset(), cell.getQualifierLength(), cell.getTimestamp(),
+          KeyValue.Type.codeToType(cell.getTypeByte()), null, 0, 0);
+        kv.setSequenceId(cell.getSequenceId());
+        return new KeyOnlyCellComparable(kv);
+      } else {
+        return new ImmutableBytesWritable(key);
+      }
+    }
   }
 
   /**
@@ -353,7 +372,13 @@ public class WALPlayer extends Configured implements Tool {
     job.setJarByClass(WALPlayer.class);
 
     job.setInputFormatClass(WALInputFormat.class);
-    job.setMapOutputKeyClass(ImmutableBytesWritable.class);
+    boolean diskBasedSortingEnabled = HFileOutputFormat2.diskBasedSortingEnabled(conf);
+    if (diskBasedSortingEnabled) {
+      job.setMapOutputKeyClass(KeyOnlyCellComparable.class);
+      job.setSortComparatorClass(KeyOnlyCellComparable.KeyOnlyCellComparator.class);
+    } else {
+      job.setMapOutputKeyClass(ImmutableBytesWritable.class);
+    }
 
     String hfileOutPath = conf.get(BULK_OUTPUT_CONF_KEY);
     if (hfileOutPath != null) {
@@ -372,7 +397,11 @@ public class WALPlayer extends Configured implements Tool {
       List<TableName> tableNames = getTableNameList(tables);
 
       job.setMapperClass(WALCellMapper.class);
-      job.setReducerClass(CellSortReducer.class);
+      if (diskBasedSortingEnabled) {
+        job.setReducerClass(PreSortedCellsReducer.class);
+      } else {
+        job.setReducerClass(CellSortReducer.class);
+      }
       Path outputDir = new Path(hfileOutPath);
       FileOutputFormat.setOutputPath(job, outputDir);
       job.setMapOutputValueClass(MapReduceExtendedCell.class);
