@@ -24,15 +24,21 @@ import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
+import java.util.HashMap;
+import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
 import java.util.function.BiConsumer;
+import java.util.stream.Collectors;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.fs.FSDataOutputStream;
@@ -60,7 +66,6 @@ import org.apache.hadoop.hbase.util.HFileArchiveUtil;
 import org.apache.hadoop.hbase.util.Pair;
 import org.apache.hadoop.hbase.util.Strings;
 import org.apache.hadoop.io.BytesWritable;
-import org.apache.hadoop.io.IOUtils;
 import org.apache.hadoop.io.NullWritable;
 import org.apache.hadoop.io.Writable;
 import org.apache.hadoop.mapreduce.InputFormat;
@@ -72,12 +77,15 @@ import org.apache.hadoop.mapreduce.RecordReader;
 import org.apache.hadoop.mapreduce.TaskAttemptContext;
 import org.apache.hadoop.mapreduce.lib.output.NullOutputFormat;
 import org.apache.hadoop.mapreduce.security.TokenCache;
+import org.apache.hadoop.util.ReflectionUtils;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.hadoop.util.Tool;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.hbase.thirdparty.com.google.common.collect.ImmutableList;
+import org.apache.hbase.thirdparty.com.google.common.collect.ImmutableSet;
 import org.apache.hbase.thirdparty.org.apache.commons.cli.CommandLine;
 import org.apache.hbase.thirdparty.org.apache.commons.cli.Option;
 
@@ -115,11 +123,16 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
   private static final String CONF_MAP_GROUP = "snapshot.export.default.map.group";
   private static final String CONF_BANDWIDTH_MB = "snapshot.export.map.bandwidth.mb";
   private static final String CONF_MR_JOB_NAME = "mapreduce.job.name";
+  private static final String CONF_INPUT_FILE_GROUPER_CLASS =
+    "snapshot.export.input.file.grouper.class";
+  private static final String CONF_INPUT_FILE_LOCATION_RESOLVER_CLASS =
+    "snapshot.export.input.file.location.resolver.class";
   protected static final String CONF_SKIP_TMP = "snapshot.export.skip.tmp";
   private static final String CONF_COPY_MANIFEST_THREADS =
     "snapshot.export.copy.references.threads";
   private static final int DEFAULT_COPY_MANIFEST_THREADS =
     Runtime.getRuntime().availableProcessors();
+  private static final String CONF_STORAGE_POLICY = "snapshot.export.storage.policy.family";
 
   static class Testing {
     static final String CONF_TEST_FAILURE = "test.snapshot.export.failure";
@@ -152,11 +165,21 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
     static final Option CHMOD =
       new Option(null, "chmod", true, "Change the permission of the files to the specified one.");
     static final Option MAPPERS = new Option(null, "mappers", true,
-      "Number of mappers to use during the copy (mapreduce.job.maps).");
+      "Number of mappers to use during the copy (mapreduce.job.maps). "
+        + "If you provide a --custom-file-grouper, "
+        + "then --mappers is interpreted as the number of mappers per group.");
     static final Option BANDWIDTH =
       new Option(null, "bandwidth", true, "Limit bandwidth to this value in MB/second.");
     static final Option RESET_TTL =
       new Option(null, "reset-ttl", false, "Do not copy TTL for the snapshot");
+    static final Option STORAGE_POLICY = new Option(null, "storage-policy", true,
+      "Storage policy for export snapshot output directory, with format like: f=HOT&g=ALL_SDD");
+    static final Option CUSTOM_FILE_GROUPER = new Option(null, "custom-file-grouper", true,
+      "Fully qualified class name of an implementation of ExportSnapshot.CustomFileGrouper. "
+        + "See JavaDoc on that class for more information.");
+    static final Option FILE_LOCATION_RESOLVER = new Option(null, "file-location-resolver", true,
+      "Fully qualified class name of an implementation of ExportSnapshot.FileLocationResolver. "
+        + "See JavaDoc on that class for more information.");
   }
 
   // Export Map-Reduce Counters, to keep track of the progress
@@ -179,6 +202,54 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
     INCOMPATIBLE, // checksum comparison is not compatible.
   }
 
+  /**
+   * If desired, you may implement a CustomFileGrouper in order to influence how ExportSnapshot
+   * chooses which input files go into the MapReduce job's {@link InputSplit}s. Your implementation
+   * must return a data structure that contains each input file exactly once. Files that appear in
+   * separate entries in the top-level returned Collection are guaranteed to not be placed in the
+   * same InputSplit. This can be used to segregate your input files by the rack or host on which
+   * they are available, which, used in conjunction with {@link FileLocationResolver}, can improve
+   * the performance of your ExportSnapshot runs. To use this, pass the --custom-file-grouper
+   * argument with the fully qualified class name of an implementation of CustomFileGrouper that's
+   * on the classpath. If this argument is not used, no particular grouping logic will be applied.
+   */
+  @InterfaceAudience.Public
+  public interface CustomFileGrouper {
+    Collection<Collection<Pair<SnapshotFileInfo, Long>>>
+      getGroupedInputFiles(final Collection<Pair<SnapshotFileInfo, Long>> snapshotFiles);
+  }
+
+  private static class NoopCustomFileGrouper implements CustomFileGrouper {
+    @Override
+    public Collection<Collection<Pair<SnapshotFileInfo, Long>>>
+      getGroupedInputFiles(final Collection<Pair<SnapshotFileInfo, Long>> snapshotFiles) {
+      return ImmutableList.of(snapshotFiles);
+    }
+  }
+
+  /**
+   * If desired, you may implement a FileLocationResolver in order to influence the _location_
+   * metadata attached to each {@link InputSplit} that ExportSnapshot will submit to YARN. The
+   * method {@link #getLocationsForInputFiles(Collection)} method is called once for each InputSplit
+   * being constructed. Whatever is returned will ultimately be reported by that split's
+   * {@link InputSplit#getLocations()} method. This can be used to encourage YARN to schedule the
+   * ExportSnapshot's mappers on rack-local or host-local NodeManagers. To use this, pass the
+   * --file-location-resolver argument with the fully qualified class name of an implementation of
+   * FileLocationResolver that's on the classpath. If this argument is not used, no locations will
+   * be attached to the InputSplits.
+   */
+  @InterfaceAudience.Public
+  public interface FileLocationResolver {
+    Set<String> getLocationsForInputFiles(final Collection<Pair<SnapshotFileInfo, Long>> files);
+  }
+
+  static class NoopFileLocationResolver implements FileLocationResolver {
+    @Override
+    public Set<String> getLocationsForInputFiles(Collection<Pair<SnapshotFileInfo, Long>> files) {
+      return ImmutableSet.of();
+    }
+  }
+
   private static class ExportMapper
     extends Mapper<BytesWritable, NullWritable, NullWritable, NullWritable> {
     private static final Logger LOG = LoggerFactory.getLogger(ExportMapper.class);
@@ -220,14 +291,12 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
       outputArchive = new Path(outputRoot, HConstants.HFILE_ARCHIVE_DIRECTORY);
 
       try {
-        srcConf.setBoolean("fs." + inputRoot.toUri().getScheme() + ".impl.disable.cache", true);
         inputFs = FileSystem.get(inputRoot.toUri(), srcConf);
       } catch (IOException e) {
         throw new IOException("Could not get the input FileSystem with root=" + inputRoot, e);
       }
 
       try {
-        destConf.setBoolean("fs." + outputRoot.toUri().getScheme() + ".impl.disable.cache", true);
         outputFs = FileSystem.get(outputRoot.toUri(), destConf);
       } catch (IOException e) {
         throw new IOException("Could not get the output FileSystem with root=" + outputRoot, e);
@@ -249,12 +318,6 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
       }
     }
 
-    @Override
-    protected void cleanup(Context context) {
-      IOUtils.closeStream(inputFs);
-      IOUtils.closeStream(outputFs);
-    }
-
     @Override
     public void map(BytesWritable key, NullWritable value, Context context)
       throws InterruptedException, IOException {
@@ -332,6 +395,13 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
 
         // Ensure that the output folder is there and copy the file
         createOutputPath(outputPath.getParent());
+        String family = new Path(inputInfo.getHfile()).getParent().getName();
+        String familyStoragePolicy = generateFamilyStoragePolicyKey(family);
+        if (stringIsNotEmpty(context.getConfiguration().get(familyStoragePolicy))) {
+          String key = context.getConfiguration().get(familyStoragePolicy);
+          LOG.info("Setting storage policy {} for {}", key, outputPath.getParent());
+          outputFs.setStoragePolicy(outputPath.getParent(), key);
+        }
         FSDataOutputStream out = outputFs.create(outputPath, true);
 
         long stime = EnvironmentEdgeManager.currentTime();
@@ -430,7 +500,7 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
     }
 
     private boolean stringIsNotEmpty(final String str) {
-      return str != null && str.length() > 0;
+      return str != null && !str.isEmpty();
     }
 
     private long copyData(final Context context, final Path inputPath, final InputStream in,
@@ -594,8 +664,10 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
             errMessage
               .append(" You can choose file-level checksum validation via "
                 + "-Ddfs.checksum.combine.mode=COMPOSITE_CRC when block-sizes"
-                + " or filesystems are different.")
-              .append(" Or you can skip checksum-checks altogether with --no-checksum-verify.\n")
+                + " or filesystems are different.\n")
+              .append(" Or you can skip checksum-checks altogether with -no-checksum-verify,")
+              .append(
+                " for the table backup scenario, you should use -i option to skip checksum-checks.\n")
               .append(" (NOTE: By skipping checksums, one runs the risk of "
                 + "masking data-corruption during file-transfer.)\n");
           }
@@ -662,6 +734,7 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
 
     // Get snapshot files
     LOG.info("Loading Snapshot '" + snapshotDesc.getName() + "' hfile list");
+    Set<String> addedFiles = new HashSet<>();
     SnapshotReferenceUtil.visitReferencedFiles(conf, fs, snapshotDir, snapshotDesc,
       new SnapshotReferenceUtil.SnapshotVisitor() {
         @Override
@@ -681,7 +754,13 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
             snapshotFileAndSize = getSnapshotFileAndSize(fs, conf, table, referencedRegion, family,
               referencedHFile, storeFile.hasFileSize() ? storeFile.getFileSize() : -1);
           }
-          files.add(snapshotFileAndSize);
+          String fileToExport = snapshotFileAndSize.getFirst().getHfile();
+          if (!addedFiles.contains(fileToExport)) {
+            files.add(snapshotFileAndSize);
+            addedFiles.add(fileToExport);
+          } else {
+            LOG.debug("Skip the existing file: {}.", fileToExport);
+          }
         }
       });
 
@@ -707,8 +786,9 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
    * The algorithm used is pretty straightforward; the file list is sorted by size, and then each
    * group fetch the bigger file available, iterating through groups alternating the direction.
    */
-  static List<List<Pair<SnapshotFileInfo, Long>>>
-    getBalancedSplits(final List<Pair<SnapshotFileInfo, Long>> files, final int ngroups) {
+  static List<List<Pair<SnapshotFileInfo, Long>>> getBalancedSplits(
+    final Collection<Pair<SnapshotFileInfo, Long>> unsortedFiles, final int ngroups) {
+    List<Pair<SnapshotFileInfo, Long>> files = new ArrayList<>(unsortedFiles);
     // Sort files by size, from small to big
     Collections.sort(files, new Comparator<Pair<SnapshotFileInfo, Long>>() {
       public int compare(Pair<SnapshotFileInfo, Long> a, Pair<SnapshotFileInfo, Long> b) {
@@ -719,7 +799,6 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
 
     // create balanced groups
     List<List<Pair<SnapshotFileInfo, Long>>> fileGroups = new LinkedList<>();
-    long[] sizeGroups = new long[ngroups];
     int hi = files.size() - 1;
     int lo = 0;
 
@@ -738,7 +817,6 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
       Pair<SnapshotFileInfo, Long> fileInfo = files.get(hi--);
 
       // add the hi one
-      sizeGroups[g] += fileInfo.getSecond();
       group.add(fileInfo);
 
       // change direction when at the end or the beginning
@@ -752,16 +830,10 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
       }
     }
 
-    if (LOG.isDebugEnabled()) {
-      for (int i = 0; i < sizeGroups.length; ++i) {
-        LOG.debug("export split=" + i + " size=" + Strings.humanReadableInt(sizeGroups[i]));
-      }
-    }
-
     return fileGroups;
   }
 
-  private static class ExportSnapshotInputFormat extends InputFormat<BytesWritable, NullWritable> {
+  static class ExportSnapshotInputFormat extends InputFormat<BytesWritable, NullWritable> {
     @Override
     public RecordReader<BytesWritable, NullWritable> createRecordReader(InputSplit split,
       TaskAttemptContext tac) throws IOException, InterruptedException {
@@ -775,37 +847,78 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
       FileSystem fs = FileSystem.get(snapshotDir.toUri(), conf);
 
       List<Pair<SnapshotFileInfo, Long>> snapshotFiles = getSnapshotFiles(conf, fs, snapshotDir);
+
+      Collection<List<Pair<SnapshotFileInfo, Long>>> balancedGroups =
+        groupFilesForSplits(conf, snapshotFiles);
+
+      Class<? extends FileLocationResolver> fileLocationResolverClass =
+        conf.getClass(CONF_INPUT_FILE_LOCATION_RESOLVER_CLASS, NoopFileLocationResolver.class,
+          FileLocationResolver.class);
+      FileLocationResolver fileLocationResolver =
+        ReflectionUtils.newInstance(fileLocationResolverClass, conf);
+      LOG.info("FileLocationResolver {} will provide location metadata for each InputSplit",
+        fileLocationResolverClass);
+
+      List<InputSplit> splits = new ArrayList<>(balancedGroups.size());
+      for (Collection<Pair<SnapshotFileInfo, Long>> files : balancedGroups) {
+        splits.add(new ExportSnapshotInputSplit(files, fileLocationResolver));
+      }
+      return splits;
+    }
+
+    Collection<List<Pair<SnapshotFileInfo, Long>>> groupFilesForSplits(Configuration conf,
+      List<Pair<SnapshotFileInfo, Long>> snapshotFiles) {
       int mappers = conf.getInt(CONF_NUM_SPLITS, 0);
-      if (mappers == 0 && snapshotFiles.size() > 0) {
+      if (mappers == 0 && !snapshotFiles.isEmpty()) {
         mappers = 1 + (snapshotFiles.size() / conf.getInt(CONF_MAP_GROUP, 10));
         mappers = Math.min(mappers, snapshotFiles.size());
         conf.setInt(CONF_NUM_SPLITS, mappers);
         conf.setInt(MR_NUM_MAPS, mappers);
       }
 
-      List<List<Pair<SnapshotFileInfo, Long>>> groups = getBalancedSplits(snapshotFiles, mappers);
-      List<InputSplit> splits = new ArrayList(groups.size());
-      for (List<Pair<SnapshotFileInfo, Long>> files : groups) {
-        splits.add(new ExportSnapshotInputSplit(files));
-      }
-      return splits;
+      Class<? extends CustomFileGrouper> inputFileGrouperClass = conf.getClass(
+        CONF_INPUT_FILE_GROUPER_CLASS, NoopCustomFileGrouper.class, CustomFileGrouper.class);
+      CustomFileGrouper customFileGrouper =
+        ReflectionUtils.newInstance(inputFileGrouperClass, conf);
+      Collection<Collection<Pair<SnapshotFileInfo, Long>>> groups =
+        customFileGrouper.getGroupedInputFiles(snapshotFiles);
+
+      LOG.info("CustomFileGrouper {} split input files into {} groups", inputFileGrouperClass,
+        groups.size());
+      int mappersPerGroup = groups.isEmpty() ? 1 : Math.max(mappers / groups.size(), 1);
+      LOG.info(
+        "Splitting each group into {} InputSplits, "
+          + "to achieve closest possible amount of mappers to target of {}",
+        mappersPerGroup, mappers);
+
+      // Within each group, create splits of equal size. Groups are not mixed together.
+      return groups.stream().map(g -> getBalancedSplits(g, mappersPerGroup))
+        .flatMap(Collection::stream).collect(Collectors.toList());
     }
 
-    private static class ExportSnapshotInputSplit extends InputSplit implements Writable {
+    static class ExportSnapshotInputSplit extends InputSplit implements Writable {
+
       private List<Pair<BytesWritable, Long>> files;
+      private String[] locations;
       private long length;
 
       public ExportSnapshotInputSplit() {
         this.files = null;
+        this.locations = null;
       }
 
-      public ExportSnapshotInputSplit(final List<Pair<SnapshotFileInfo, Long>> snapshotFiles) {
-        this.files = new ArrayList(snapshotFiles.size());
+      public ExportSnapshotInputSplit(final Collection<Pair<SnapshotFileInfo, Long>> snapshotFiles,
+        FileLocationResolver fileLocationResolver) {
+        this.files = new ArrayList<>(snapshotFiles.size());
         for (Pair<SnapshotFileInfo, Long> fileInfo : snapshotFiles) {
           this.files.add(
             new Pair<>(new BytesWritable(fileInfo.getFirst().toByteArray()), fileInfo.getSecond()));
           this.length += fileInfo.getSecond();
         }
+        this.locations =
+          fileLocationResolver.getLocationsForInputFiles(snapshotFiles).toArray(new String[0]);
+        LOG.trace("This ExportSnapshotInputSplit has files {} of collective size {}, "
+          + "with location hints: {}", files, length, locations);
       }
 
       private List<Pair<BytesWritable, Long>> getSplitKeys() {
@@ -819,7 +932,7 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
 
       @Override
       public String[] getLocations() throws IOException, InterruptedException {
-        return new String[] {};
+        return locations;
       }
 
       @Override
@@ -834,6 +947,12 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
           files.add(new Pair<>(fileInfo, size));
           length += size;
         }
+        int locationCount = in.readInt();
+        List<String> locations = new ArrayList<>(locationCount);
+        for (int i = 0; i < locationCount; ++i) {
+          locations.add(in.readUTF());
+        }
+        this.locations = locations.toArray(new String[0]);
       }
 
       @Override
@@ -843,6 +962,10 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
           fileInfo.getFirst().write(out);
           out.writeLong(fileInfo.getSecond());
         }
+        out.writeInt(locations.length);
+        for (String location : locations) {
+          out.writeUTF(location);
+        }
       }
     }
 
@@ -902,7 +1025,8 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
    */
   private void runCopyJob(final Path inputRoot, final Path outputRoot, final String snapshotName,
     final Path snapshotDir, final boolean verifyChecksum, final String filesUser,
-    final String filesGroup, final int filesMode, final int mappers, final int bandwidthMB)
+    final String filesGroup, final int filesMode, final int mappers, final int bandwidthMB,
+    final String storagePolicy, final String customFileGrouper, final String fileLocationResolver)
     throws IOException, InterruptedException, ClassNotFoundException {
     Configuration conf = getConf();
     if (filesGroup != null) conf.set(CONF_FILES_GROUP, filesGroup);
@@ -918,6 +1042,17 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
     conf.setInt(CONF_BANDWIDTH_MB, bandwidthMB);
     conf.set(CONF_SNAPSHOT_NAME, snapshotName);
     conf.set(CONF_SNAPSHOT_DIR, snapshotDir.toString());
+    if (storagePolicy != null) {
+      for (Map.Entry<String, String> entry : storagePolicyPerFamily(storagePolicy).entrySet()) {
+        conf.set(generateFamilyStoragePolicyKey(entry.getKey()), entry.getValue());
+      }
+    }
+    if (customFileGrouper != null) {
+      conf.set(CONF_INPUT_FILE_GROUPER_CLASS, customFileGrouper);
+    }
+    if (fileLocationResolver != null) {
+      conf.set(CONF_INPUT_FILE_LOCATION_RESOLVER_CLASS, fileLocationResolver);
+    }
 
     String jobname = conf.get(CONF_MR_JOB_NAME, "ExportSnapshot-" + snapshotName);
     Job job = new Job(conf);
@@ -1004,6 +1139,23 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
     }, conf);
   }
 
+  private Map<String, String> storagePolicyPerFamily(String storagePolicy) {
+    Map<String, String> familyStoragePolicy = new HashMap<>();
+    for (String familyConf : storagePolicy.split("&")) {
+      String[] familySplit = familyConf.split("=");
+      if (familySplit.length != 2) {
+        continue;
+      }
+      // family is key, storage policy is value
+      familyStoragePolicy.put(familySplit[0], familySplit[1]);
+    }
+    return familyStoragePolicy;
+  }
+
+  private static String generateFamilyStoragePolicyKey(String family) {
+    return CONF_STORAGE_POLICY + "." + family;
+  }
+
   private boolean verifyTarget = true;
   private boolean verifySource = true;
   private boolean verifyChecksum = true;
@@ -1018,6 +1170,9 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
   private int filesMode = 0;
   private int mappers = 0;
   private boolean resetTtl = false;
+  private String storagePolicy = null;
+  private String customFileGrouper = null;
+  private String fileLocationResolver = null;
 
   @Override
   protected void processOptions(CommandLine cmd) {
@@ -1040,6 +1195,15 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
     verifyTarget = !cmd.hasOption(Options.NO_TARGET_VERIFY.getLongOpt());
     verifySource = !cmd.hasOption(Options.NO_SOURCE_VERIFY.getLongOpt());
     resetTtl = cmd.hasOption(Options.RESET_TTL.getLongOpt());
+    if (cmd.hasOption(Options.STORAGE_POLICY.getLongOpt())) {
+      storagePolicy = cmd.getOptionValue(Options.STORAGE_POLICY.getLongOpt());
+    }
+    if (cmd.hasOption(Options.CUSTOM_FILE_GROUPER.getLongOpt())) {
+      customFileGrouper = cmd.getOptionValue(Options.CUSTOM_FILE_GROUPER.getLongOpt());
+    }
+    if (cmd.hasOption(Options.FILE_LOCATION_RESOLVER.getLongOpt())) {
+      fileLocationResolver = cmd.getOptionValue(Options.FILE_LOCATION_RESOLVER.getLongOpt());
+    }
   }
 
   /**
@@ -1074,10 +1238,8 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
     }
 
     Configuration srcConf = HBaseConfiguration.createClusterConf(conf, null, CONF_SOURCE_PREFIX);
-    srcConf.setBoolean("fs." + inputRoot.toUri().getScheme() + ".impl.disable.cache", true);
     FileSystem inputFs = FileSystem.get(inputRoot.toUri(), srcConf);
     Configuration destConf = HBaseConfiguration.createClusterConf(conf, null, CONF_DEST_PREFIX);
-    destConf.setBoolean("fs." + outputRoot.toUri().getScheme() + ".impl.disable.cache", true);
     FileSystem outputFs = FileSystem.get(outputRoot.toUri(), destConf);
     boolean skipTmp = conf.getBoolean(CONF_SKIP_TMP, false)
       || conf.get(SnapshotDescriptionUtils.SNAPSHOT_WORKING_DIR) != null;
@@ -1210,7 +1372,8 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
     // by the HFileArchiver, since they have no references.
     try {
       runCopyJob(inputRoot, outputRoot, snapshotName, snapshotDir, verifyChecksum, filesUser,
-        filesGroup, filesMode, mappers, bandwidthMB);
+        filesGroup, filesMode, mappers, bandwidthMB, storagePolicy, customFileGrouper,
+        fileLocationResolver);
 
       LOG.info("Finalize the Snapshot Export");
       if (!skipTmp) {
@@ -1238,9 +1401,6 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
       }
       outputFs.delete(outputSnapshotDir, true);
       return EXIT_FAILURE;
-    } finally {
-      IOUtils.closeStream(inputFs);
-      IOUtils.closeStream(outputFs);
     }
   }
 
@@ -1271,6 +1431,8 @@ public class ExportSnapshot extends AbstractHBaseTool implements Tool {
     addOption(Options.MAPPERS);
     addOption(Options.BANDWIDTH);
     addOption(Options.RESET_TTL);
+    addOption(Options.CUSTOM_FILE_GROUPER);
+    addOption(Options.FILE_LOCATION_RESOLVER);
   }
 
   public static void main(String[] args) {
