@@ -56,7 +56,12 @@ public class RowResultGenerator extends ResultGenerator {
           }
         }
       }
-      get.setTimeRange(rowspec.getStartTime(), rowspec.getEndTime());
+      if (rowspec.isPartialTimeRange()) {
+        get.setTimestamp(rowspec.getTimestamp());
+      } else {
+        get.setTimeRange(rowspec.getStartTime(), rowspec.getEndTime());
+      }
+
       get.setMaxVersions(rowspec.getMaxVersions());
       if (filter != null) {
         get.setFilter(filter);
