@@ -22,6 +22,8 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import org.apache.hadoop.hbase.TableName;
+import org.apache.hadoop.hbase.client.BalanceRequest;
+import org.apache.hadoop.hbase.client.BalanceResponse;
 import org.apache.hadoop.hbase.net.Address;
 import org.apache.yetus.audience.InterfaceAudience;
 
@@ -65,7 +67,16 @@ public interface RSGroupAdmin {
    * Balance regions in the given RegionServer group.
    * @return boolean Whether balance ran or not
    */
-  boolean balanceRSGroup(String groupName) throws IOException;
+  default BalanceResponse balanceRSGroup(String groupName) throws IOException {
+    return balanceRSGroup(groupName, BalanceRequest.defaultInstance());
+  }
+
+  /**
+   * Balance regions in the given RegionServer group, running based on the given
+   * {@link BalanceRequest}.
+   * @return boolean Whether balance ran or not
+   */
+  BalanceResponse balanceRSGroup(String groupName, BalanceRequest request) throws IOException;
 
   /**
    * Lists current set of RegionServer groups.
@@ -111,4 +122,12 @@ public interface RSGroupAdmin {
    * @throws IOException if a remote or network exception occurs
    */
   void updateRSGroupConfig(String groupName, Map<String, String> configuration) throws IOException;
+
+  /**
+   * Update the configuration and trigger an online config change on all the regionservers in the
+   * RSGroup.
+   * @param groupName the group name
+   * @throws IOException if a remote or network exception occurs
+   */
+  void updateConfiguration(String groupName) throws IOException;
 }
