@@ -52,7 +52,10 @@ public enum ExecutorType {
   RS_REFRESH_PEER(31),
   RS_SWITCH_RPC_THROTTLE(33),
   RS_IN_MEMORY_COMPACTION(34),
-  RS_CLAIM_REPLICATION_QUEUE(35);
+  RS_CLAIM_REPLICATION_QUEUE(35),
+  RS_SNAPSHOT_OPERATIONS(36),
+  RS_FLUSH_OPERATIONS(37),
+  RS_RELOAD_QUOTAS_OPERATIONS(38);
 
   ExecutorType(int value) {
   }
