@@ -120,7 +120,6 @@ public final class PrefetchExecutor {
     if (LOG.isDebugEnabled()) {
       LOG.debug("Prefetch completed for {}", path.getName());
     }
-    LOG.debug("Prefetch completed for {}", path);
   }
 
   public static void cancel(Path path) {
@@ -134,14 +133,7 @@ public final class PrefetchExecutor {
     }
   }
 
-  public static void interrupt(Path path) {
-    Future<?> future = prefetchFutures.get(path);
-    if (future != null) {
-      prefetchFutures.remove(path);
-      // ok to race with other cancellation attempts
-      future.cancel(true);
-      LOG.debug("Prefetch cancelled for {}", path);
-    }
+  private PrefetchExecutor() {
   }
 
   public static boolean isCompleted(Path path) {
@@ -152,9 +144,6 @@ public final class PrefetchExecutor {
     return true;
   }
 
-  private PrefetchExecutor() {
-  }
-
   /* Visible for testing only */
   @RestrictedApi(explanation = "Should only be called in tests", link = "",
       allowedOnPath = ".*/src/test/.*")
@@ -201,10 +190,18 @@ public final class PrefetchExecutor {
     prefetchFutures.forEach((k, v) -> {
       ScheduledFuture sf = (ScheduledFuture) prefetchFutures.get(k);
       if (!(sf.getDelay(TimeUnit.MILLISECONDS) > 0)) {
-        // the thread is still pending delay expiration and has not started to run yet, so can be
-        // re-scheduled at no cost.
-        interrupt(k);
-        request(k, prefetchRunnable.get(k));
+        Runnable runnable = prefetchRunnable.get(k);
+        Future<?> future = prefetchFutures.get(k);
+        if (future != null) {
+          prefetchFutures.remove(k);
+          // ok to race with other cancellation attempts
+          boolean canceled = future.cancel(true);
+          LOG.debug("Prefetch {} for {}",
+            canceled ? "cancelled" : "cancel attempted but it was already finished", k);
+        }
+        if (runnable != null && future != null && future.isCancelled()) {
+          request(k, runnable);
+        }
       }
       LOG.debug("Reset called on Prefetch of file {} with delay {}, delay variation {}", k,
         prefetchDelayMillis, prefetchDelayVariation);
