@@ -132,6 +132,11 @@ public class FifoRpcScheduler extends RpcScheduler {
     return 0;
   }
 
+  @Override
+  public int getBulkLoadQueueLength() {
+    return 0;
+  }
+
   @Override
   public int getActiveRpcHandlerCount() {
     return executor.getActiveCount();
@@ -152,6 +157,11 @@ public class FifoRpcScheduler extends RpcScheduler {
     return 0;
   }
 
+  @Override
+  public int getActiveBulkLoadRpcHandlerCount() {
+    return 0;
+  }
+
   @Override
   public int getActiveMetaPriorityRpcHandlerCount() {
     return 0;
