@@ -102,7 +102,6 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
 
   private long responseCellSize = 0;
   private long responseBlockSize = 0;
-  private long fsReadTimeMillis = 0;
   // cumulative size of serialized exceptions
   private long exceptionSize = 0;
   private final boolean retryImmediatelySupported;
@@ -339,6 +338,7 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
       bc = new BufferChain(responseBufs);
     } catch (IOException e) {
       RpcServer.LOG.warn("Exception while creating response " + e);
+      bc = createFallbackErrorResponse(e);
     }
     this.response = bc;
     // Once a response message is created and set to this.response, this Call can be treated as
@@ -375,6 +375,32 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     headerBuilder.setException(exceptionBuilder.build());
   }
 
+  /*
+   * Creates a fallback error response when the primary response creation fails. This method is
+   * invoked as a last resort when an IOException occurs during the normal response creation
+   * process. It attempts to create a minimal error response containing only the error information,
+   * without any cell blocks or additional data. The purpose is to ensure that the client receives
+   * some indication of the failure rather than experiencing a silent connection drop. This provides
+   * better error handling on the client side.
+   */
+  private BufferChain createFallbackErrorResponse(IOException originalException) {
+    try {
+      ResponseHeader.Builder headerBuilder = ResponseHeader.newBuilder();
+      headerBuilder.setCallId(this.id);
+      String responseErrorMsg =
+        "Failed to create response due to: " + originalException.getMessage();
+      setExceptionResponse(originalException, responseErrorMsg, headerBuilder);
+      Message header = headerBuilder.build();
+      ByteBuffer headerBuf = createHeaderAndMessageBytes(null, header, 0, null);
+      this.isError = true;
+      return new BufferChain(new ByteBuffer[] { headerBuf });
+    } catch (IOException e) {
+      RpcServer.LOG.error("Failed to create error response for client, connection may be dropped",
+        e);
+      return null;
+    }
+  }
+
   static ByteBuffer createHeaderAndMessageBytes(Message result, Message header, int cellBlockSize,
     List<ByteBuffer> cellBlock) throws IOException {
     // Organize the response as a set of bytebuffers rather than collect it all together inside
@@ -583,14 +609,4 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
   public synchronized RpcCallback getCallBack() {
     return this.rpcCallback;
   }
-
-  @Override
-  public void updateFsReadTime(long latencyMillis) {
-    fsReadTimeMillis += latencyMillis;
-  }
-
-  @Override
-  public long getFsReadTime() {
-    return fsReadTimeMillis;
-  }
 }
