@@ -45,14 +45,17 @@ public class CloseRegionProcedure extends RegionRemoteProcedureBase {
   // wrong(but do not make it wrong intentionally). The client can handle this error.
   private ServerName assignCandidate;
 
+  private boolean evictCache;
+
   public CloseRegionProcedure() {
     super();
   }
 
   public CloseRegionProcedure(TransitRegionStateProcedure parent, RegionInfo region,
-    ServerName targetServer, ServerName assignCandidate) {
+    ServerName targetServer, ServerName assignCandidate, boolean evictCache) {
     super(parent, region, targetServer);
     this.assignCandidate = assignCandidate;
+    this.evictCache = evictCache;
   }
 
   @Override
@@ -62,7 +65,7 @@ public class CloseRegionProcedure extends RegionRemoteProcedureBase {
 
   @Override
   public RemoteOperation newRemoteOperation(MasterProcedureEnv env) {
-    return new RegionCloseOperation(this, region, getProcId(), assignCandidate,
+    return new RegionCloseOperation(this, region, getProcId(), assignCandidate, evictCache,
       env.getMasterServices().getMasterActiveTime());
   }
 
@@ -73,6 +76,7 @@ public class CloseRegionProcedure extends RegionRemoteProcedureBase {
     if (assignCandidate != null) {
       builder.setAssignCandidate(ProtobufUtil.toServerName(assignCandidate));
     }
+    builder.setEvictCache(evictCache);
     serializer.serialize(builder.build());
   }
 
@@ -84,6 +88,7 @@ public class CloseRegionProcedure extends RegionRemoteProcedureBase {
     if (data.hasAssignCandidate()) {
       assignCandidate = ProtobufUtil.toServerName(data.getAssignCandidate());
     }
+    evictCache = data.getEvictCache();
   }
 
   @Override
