@@ -18,6 +18,7 @@
 package org.apache.hadoop.hbase.master.procedure;
 
 import org.apache.hadoop.hbase.ServerName;
+import org.apache.hadoop.hbase.quotas.QuotaCache;
 import org.apache.yetus.audience.InterfaceAudience;
 
 /**
@@ -50,7 +51,17 @@ public interface ServerProcedureInterface {
     /**
      * send the claim replication queue request to region server to actually assign it
      */
-    CLAIM_REPLICATION_QUEUE_REMOTE
+    CLAIM_REPLICATION_QUEUE_REMOTE,
+
+    /**
+     * send verify snapshot request to region server and handle the response
+     */
+    VERIFY_SNAPSHOT,
+
+    /**
+     * Re-read the hbase:quotas table and update {@link QuotaCache}.
+     */
+    RELOAD_QUOTAS,
   }
 
   /** Returns Name of this server instance. */
