@@ -42,6 +42,8 @@ class ServerQueue extends Queue<ServerName> {
       case SPLIT_WAL_REMOTE:
       case CLAIM_REPLICATION_QUEUES:
       case CLAIM_REPLICATION_QUEUE_REMOTE:
+      case VERIFY_SNAPSHOT:
+      case RELOAD_QUOTAS:
         return false;
       default:
         break;
