@@ -84,6 +84,9 @@ public final class ThreadLocalServerSideScanMetrics {
   private static final ThreadLocal<AtomicLong> BLOCK_READ_OPS_COUNT =
     ThreadLocal.withInitial(() -> new AtomicLong(0));
 
+  private static final ThreadLocal<AtomicLong> FS_READ_TIME =
+    ThreadLocal.withInitial(() -> new AtomicLong(0));
+
   public static void setScanMetricsEnabled(boolean enable) {
     IS_SCAN_METRICS_ENABLED.set(enable);
   }
@@ -104,6 +107,10 @@ public final class ThreadLocalServerSideScanMetrics {
     return BLOCK_READ_OPS_COUNT.get().addAndGet(count);
   }
 
+  public static long addFsReadTime(long time) {
+    return FS_READ_TIME.get().addAndGet(time);
+  }
+
   public static boolean isScanMetricsEnabled() {
     return IS_SCAN_METRICS_ENABLED.get();
   }
@@ -124,6 +131,10 @@ public final class ThreadLocalServerSideScanMetrics {
     return BLOCK_READ_OPS_COUNT.get();
   }
 
+  public static AtomicLong getFsReadTimeCounter() {
+    return FS_READ_TIME.get();
+  }
+
   public static long getBytesReadFromFsAndReset() {
     return getBytesReadFromFsCounter().getAndSet(0);
   }
@@ -140,11 +151,16 @@ public final class ThreadLocalServerSideScanMetrics {
     return getBlockReadOpsCountCounter().getAndSet(0);
   }
 
+  public static long getFsReadTimeAndReset() {
+    return getFsReadTimeCounter().getAndSet(0);
+  }
+
   public static void reset() {
     getBytesReadFromFsAndReset();
     getBytesReadFromBlockCacheAndReset();
     getBytesReadFromMemstoreAndReset();
     getBlockReadOpsCountAndReset();
+    getFsReadTimeAndReset();
   }
 
   public static void populateServerSideScanMetrics(ServerSideScanMetrics metrics) {
@@ -159,5 +175,7 @@ public final class ThreadLocalServerSideScanMetrics {
       getBytesReadFromMemstoreCounter().get());
     metrics.addToCounter(ServerSideScanMetrics.BLOCK_READ_OPS_COUNT_METRIC_NAME,
       getBlockReadOpsCountCounter().get());
+    metrics.addToCounter(ServerSideScanMetrics.FS_READ_TIME_METRIC_NAME,
+      getFsReadTimeCounter().get());
   }
 }
