@@ -17,18 +17,12 @@
  */
 package org.apache.hadoop.hbase.quotas;
 
-import static org.apache.hadoop.hbase.util.ConcurrentMapUtils.computeIfAbsent;
-
 import java.io.IOException;
 import java.time.Duration;
-import java.util.ArrayList;
 import java.util.EnumSet;
-import java.util.List;
 import java.util.Map;
 import java.util.Optional;
-import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.TimeUnit;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.ClusterMetrics;
@@ -36,14 +30,11 @@ import org.apache.hadoop.hbase.ClusterMetrics.Option;
 import org.apache.hadoop.hbase.ScheduledChore;
 import org.apache.hadoop.hbase.Stoppable;
 import org.apache.hadoop.hbase.TableName;
-import org.apache.hadoop.hbase.client.Get;
 import org.apache.hadoop.hbase.client.RegionStatesCount;
 import org.apache.hadoop.hbase.ipc.RpcCall;
 import org.apache.hadoop.hbase.ipc.RpcServer;
-import org.apache.hadoop.hbase.regionserver.HRegionServer;
 import org.apache.hadoop.hbase.regionserver.RegionServerServices;
 import org.apache.hadoop.hbase.util.Bytes;
-import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.apache.yetus.audience.InterfaceStability;
@@ -56,10 +47,7 @@ import org.apache.hbase.thirdparty.com.google.common.cache.LoadingCache;
 
 /**
  * Cache that keeps track of the quota settings for the users and tables that are interacting with
- * it. To avoid blocking the operations if the requested quota is not in cache an "empty quota" will
- * be returned and the request to fetch the quota information will be enqueued for the next refresh.
- * TODO: At the moment the Cache has a Chore that will be triggered every 5min or on cache-miss
- * events. Later the Quotas will be pushed using the notification system.
+ * it.
  */
 @InterfaceAudience.Private
 @InterfaceStability.Evolving
@@ -77,18 +65,15 @@ public class QuotaCache implements Stoppable {
   public static final String QUOTA_USER_REQUEST_ATTRIBUTE_OVERRIDE_KEY =
     "hbase.quota.user.override.key";
   private static final int REFRESH_DEFAULT_PERIOD = 43_200_000; // 12 hours
-  private static final int EVICT_PERIOD_FACTOR = 5;
 
-  // for testing purpose only, enforce the cache to be always refreshed
-  static boolean TEST_FORCE_REFRESH = false;
-  // for testing purpose only, block cache refreshes to reliably verify state
-  static boolean TEST_BLOCK_REFRESH = false;
+  private final Object initializerLock = new Object();
+  private volatile boolean initialized = false;
+
+  private volatile Map<String, QuotaState> namespaceQuotaCache = new ConcurrentHashMap<>();
+  private volatile Map<TableName, QuotaState> tableQuotaCache = new ConcurrentHashMap<>();
+  private volatile Map<String, UserQuotaState> userQuotaCache = new ConcurrentHashMap<>();
+  private volatile Map<String, QuotaState> regionServerQuotaCache = new ConcurrentHashMap<>();
 
-  private final ConcurrentMap<String, QuotaState> namespaceQuotaCache = new ConcurrentHashMap<>();
-  private final ConcurrentMap<TableName, QuotaState> tableQuotaCache = new ConcurrentHashMap<>();
-  private final ConcurrentMap<String, UserQuotaState> userQuotaCache = new ConcurrentHashMap<>();
-  private final ConcurrentMap<String, QuotaState> regionServerQuotaCache =
-    new ConcurrentHashMap<>();
   private volatile boolean exceedThrottleQuotaEnabled = false;
   // factors used to divide cluster scope quota into machine scope quota
   private volatile double machineQuotaFactor = 1;
@@ -111,10 +96,8 @@ public class QuotaCache implements Stoppable {
 
     Configuration conf = rsServices.getConfiguration();
     // Refresh the cache every 12 hours, and every time a quota is changed, and every time a
-    // configuration
-    // reload is triggered. Periodic reloads are kept to a minimum to avoid flooding the
-    // RegionServer
-    // holding the hbase:quota table with requests.
+    // configuration reload is triggered. Periodic reloads are kept to a minimum to avoid
+    // flooding the RegionServer holding the hbase:quota table with requests.
     int period = conf.getInt(REFRESH_CONF_KEY, REFRESH_DEFAULT_PERIOD);
     refreshChore = new QuotaRefresherChore(conf, period, this);
     rsServices.getChoreService().scheduleChore(refreshChore);
@@ -134,6 +117,37 @@ public class QuotaCache implements Stoppable {
     return stopped;
   }
 
+  private void ensureInitialized() {
+    if (!initialized) {
+      synchronized (initializerLock) {
+        if (!initialized) {
+          refreshChore.chore();
+          initialized = true;
+        }
+      }
+    }
+  }
+
+  private Map<String, UserQuotaState> fetchUserQuotaStateEntries() throws IOException {
+    return QuotaUtil.fetchUserQuotas(rsServices.getConfiguration(), rsServices.getConnection(),
+      tableMachineQuotaFactors, machineQuotaFactor);
+  }
+
+  private Map<String, QuotaState> fetchRegionServerQuotaStateEntries() throws IOException {
+    return QuotaUtil.fetchRegionServerQuotas(rsServices.getConfiguration(),
+      rsServices.getConnection());
+  }
+
+  private Map<TableName, QuotaState> fetchTableQuotaStateEntries() throws IOException {
+    return QuotaUtil.fetchTableQuotas(rsServices.getConfiguration(), rsServices.getConnection(),
+      tableMachineQuotaFactors);
+  }
+
+  private Map<String, QuotaState> fetchNamespaceQuotaStateEntries() throws IOException {
+    return QuotaUtil.fetchNamespaceQuotas(rsServices.getConfiguration(), rsServices.getConnection(),
+      machineQuotaFactor);
+  }
+
   /**
    * Returns the limiter associated to the specified user/table.
    * @param ugi   the user to limit
@@ -153,8 +167,14 @@ public class QuotaCache implements Stoppable {
    * @return the quota info associated to specified user
    */
   public UserQuotaState getUserQuotaState(final UserGroupInformation ugi) {
-    return computeIfAbsent(userQuotaCache, getQuotaUserName(ugi),
-      () -> QuotaUtil.buildDefaultUserQuotaState(rsServices.getConfiguration(), 0L));
+    String user = getQuotaUserName(ugi);
+    ensureInitialized();
+    // local reference because the chore thread may assign to userQuotaCache
+    Map<String, UserQuotaState> cache = userQuotaCache;
+    if (!cache.containsKey(user)) {
+      cache.put(user, QuotaUtil.buildDefaultUserQuotaState(rsServices.getConfiguration()));
+    }
+    return cache.get(user);
   }
 
   /**
@@ -163,7 +183,13 @@ public class QuotaCache implements Stoppable {
    * @return the limiter associated to the specified table
    */
   public QuotaLimiter getTableLimiter(final TableName table) {
-    return getQuotaState(this.tableQuotaCache, table).getGlobalLimiter();
+    ensureInitialized();
+    // local reference because the chore thread may assign to tableQuotaCache
+    Map<TableName, QuotaState> cache = tableQuotaCache;
+    if (!cache.containsKey(table)) {
+      cache.put(table, new QuotaState());
+    }
+    return cache.get(table).getGlobalLimiter();
   }
 
   /**
@@ -172,7 +198,13 @@ public class QuotaCache implements Stoppable {
    * @return the limiter associated to the specified namespace
    */
   public QuotaLimiter getNamespaceLimiter(final String namespace) {
-    return getQuotaState(this.namespaceQuotaCache, namespace).getGlobalLimiter();
+    ensureInitialized();
+    // local reference because the chore thread may assign to namespaceQuotaCache
+    Map<String, QuotaState> cache = namespaceQuotaCache;
+    if (!cache.containsKey(namespace)) {
+      cache.put(namespace, new QuotaState());
+    }
+    return cache.get(namespace).getGlobalLimiter();
   }
 
   /**
@@ -181,7 +213,13 @@ public class QuotaCache implements Stoppable {
    * @return the limiter associated to the specified region server
    */
   public QuotaLimiter getRegionServerQuotaLimiter(final String regionServer) {
-    return getQuotaState(this.regionServerQuotaCache, regionServer).getGlobalLimiter();
+    ensureInitialized();
+    // local reference because the chore thread may assign to regionServerQuotaCache
+    Map<String, QuotaState> cache = regionServerQuotaCache;
+    if (!cache.containsKey(regionServer)) {
+      cache.put(regionServer, new QuotaState());
+    }
+    return cache.get(regionServer).getGlobalLimiter();
   }
 
   protected boolean isExceedThrottleQuotaEnabled() {
@@ -193,7 +231,7 @@ public class QuotaCache implements Stoppable {
    * username
    * @param ugi The request's UserGroupInformation
    */
-  private String getQuotaUserName(final UserGroupInformation ugi) {
+  String getQuotaUserName(final UserGroupInformation ugi) {
     if (userOverrideRequestAttributeKey == null) {
       return ugi.getShortUserName();
     }
@@ -210,14 +248,6 @@ public class QuotaCache implements Stoppable {
     return Bytes.toString(override);
   }
 
-  /**
-   * Returns the QuotaState requested. If the quota info is not in cache an empty one will be
-   * returned and the quota request will be enqueued for the next cache refresh.
-   */
-  private <K> QuotaState getQuotaState(final ConcurrentMap<K, QuotaState> quotasMap, final K key) {
-    return computeIfAbsent(quotasMap, key, QuotaState::new);
-  }
-
   void triggerCacheRefresh() {
     refreshChore.triggerNow();
   }
@@ -226,30 +256,28 @@ public class QuotaCache implements Stoppable {
     refreshChore.chore();
   }
 
-  long getLastUpdate() {
-    return refreshChore.lastUpdate;
-  }
-
+  /** visible for testing */
   Map<String, QuotaState> getNamespaceQuotaCache() {
     return namespaceQuotaCache;
   }
 
+  /** visible for testing */
   Map<String, QuotaState> getRegionServerQuotaCache() {
     return regionServerQuotaCache;
   }
 
+  /** visible for testing */
   Map<TableName, QuotaState> getTableQuotaCache() {
     return tableQuotaCache;
   }
 
+  /** visible for testing */
   Map<String, UserQuotaState> getUserQuotaCache() {
     return userQuotaCache;
   }
 
   // TODO: Remove this once we have the notification bus
   private class QuotaRefresherChore extends ScheduledChore {
-    private long lastUpdate = 0;
-
     // Querying cluster metrics so often, per-RegionServer, limits horizontal scalability.
     // So we cache the results to reduce that load.
     private final RefreshableExpiringValueCache<ClusterMetrics> tableRegionStatesClusterMetrics;
@@ -280,101 +308,49 @@ public class QuotaCache implements Stoppable {
     }
 
     @Override
-    @edu.umd.cs.findbugs.annotations.SuppressWarnings(value = "GC_UNRELATED_TYPES",
-        justification = "I do not understand why the complaints, it looks good to me -- FIX")
     protected void chore() {
-      while (TEST_BLOCK_REFRESH) {
-        LOG.info("TEST_BLOCK_REFRESH=true, so blocking QuotaCache refresh until it is false");
+      synchronized (this) {
+        LOG.info("Reloading quota cache from hbase:quota table");
+        updateQuotaFactors();
+
         try {
-          Thread.sleep(10);
-        } catch (InterruptedException e) {
-          throw new RuntimeException(e);
+          Map<String, UserQuotaState> newUserQuotaCache =
+            new ConcurrentHashMap<>(fetchUserQuotaStateEntries());
+          updateNewCacheFromOld(userQuotaCache, newUserQuotaCache);
+          userQuotaCache = newUserQuotaCache;
+        } catch (IOException e) {
+          LOG.error("Error while fetching user quotas", e);
         }
+
+        try {
+          Map<String, QuotaState> newRegionServerQuotaCache =
+            new ConcurrentHashMap<>(fetchRegionServerQuotaStateEntries());
+          updateNewCacheFromOld(regionServerQuotaCache, newRegionServerQuotaCache);
+          regionServerQuotaCache = newRegionServerQuotaCache;
+        } catch (IOException e) {
+          LOG.error("Error while fetching region server quotas", e);
+        }
+
+        try {
+          Map<TableName, QuotaState> newTableQuotaCache =
+            new ConcurrentHashMap<>(fetchTableQuotaStateEntries());
+          updateNewCacheFromOld(tableQuotaCache, newTableQuotaCache);
+          tableQuotaCache = newTableQuotaCache;
+        } catch (IOException e) {
+          LOG.error("Error while refreshing table quotas", e);
+        }
+
+        try {
+          Map<String, QuotaState> newNamespaceQuotaCache =
+            new ConcurrentHashMap<>(fetchNamespaceQuotaStateEntries());
+          updateNewCacheFromOld(namespaceQuotaCache, newNamespaceQuotaCache);
+          namespaceQuotaCache = newNamespaceQuotaCache;
+        } catch (IOException e) {
+          LOG.error("Error while refreshing namespace quotas", e);
+        }
+
+        fetchExceedThrottleQuota();
       }
-      // Prefetch online tables/namespaces
-      for (TableName table : ((HRegionServer) QuotaCache.this.rsServices).getOnlineTables()) {
-        if (table.isSystemTable()) {
-          continue;
-        }
-        QuotaCache.this.tableQuotaCache.computeIfAbsent(table, key -> new QuotaState());
-
-        final String ns = table.getNamespaceAsString();
-
-        QuotaCache.this.namespaceQuotaCache.computeIfAbsent(ns, key -> new QuotaState());
-      }
-
-      QuotaCache.this.regionServerQuotaCache
-        .computeIfAbsent(QuotaTableUtil.QUOTA_REGION_SERVER_ROW_KEY, key -> new QuotaState());
-
-      updateQuotaFactors();
-      fetchNamespaceQuotaState();
-      fetchTableQuotaState();
-      fetchUserQuotaState();
-      fetchRegionServerQuotaState();
-      fetchExceedThrottleQuota();
-      lastUpdate = EnvironmentEdgeManager.currentTime();
-    }
-
-    private void fetchNamespaceQuotaState() {
-      fetch("namespace", QuotaCache.this.namespaceQuotaCache, new Fetcher<String, QuotaState>() {
-        @Override
-        public Get makeGet(final Map.Entry<String, QuotaState> entry) {
-          return QuotaUtil.makeGetForNamespaceQuotas(entry.getKey());
-        }
-
-        @Override
-        public Map<String, QuotaState> fetchEntries(final List<Get> gets) throws IOException {
-          return QuotaUtil.fetchNamespaceQuotas(rsServices.getConnection(), gets,
-            machineQuotaFactor);
-        }
-      });
-    }
-
-    private void fetchTableQuotaState() {
-      fetch("table", QuotaCache.this.tableQuotaCache, new Fetcher<TableName, QuotaState>() {
-        @Override
-        public Get makeGet(final Map.Entry<TableName, QuotaState> entry) {
-          return QuotaUtil.makeGetForTableQuotas(entry.getKey());
-        }
-
-        @Override
-        public Map<TableName, QuotaState> fetchEntries(final List<Get> gets) throws IOException {
-          return QuotaUtil.fetchTableQuotas(rsServices.getConnection(), gets,
-            tableMachineQuotaFactors);
-        }
-      });
-    }
-
-    private void fetchUserQuotaState() {
-      final Set<String> namespaces = QuotaCache.this.namespaceQuotaCache.keySet();
-      final Set<TableName> tables = QuotaCache.this.tableQuotaCache.keySet();
-      fetch("user", QuotaCache.this.userQuotaCache, new Fetcher<String, UserQuotaState>() {
-        @Override
-        public Get makeGet(final Map.Entry<String, UserQuotaState> entry) {
-          return QuotaUtil.makeGetForUserQuotas(entry.getKey(), tables, namespaces);
-        }
-
-        @Override
-        public Map<String, UserQuotaState> fetchEntries(final List<Get> gets) throws IOException {
-          return QuotaUtil.fetchUserQuotas(rsServices.getConnection(), gets,
-            tableMachineQuotaFactors, machineQuotaFactor);
-        }
-      });
-    }
-
-    private void fetchRegionServerQuotaState() {
-      fetch("regionServer", QuotaCache.this.regionServerQuotaCache,
-        new Fetcher<String, QuotaState>() {
-          @Override
-          public Get makeGet(final Map.Entry<String, QuotaState> entry) {
-            return QuotaUtil.makeGetForRegionServerQuotas(entry.getKey());
-          }
-
-          @Override
-          public Map<String, QuotaState> fetchEntries(final List<Get> gets) throws IOException {
-            return QuotaUtil.fetchRegionServerQuotas(rsServices.getConnection(), gets);
-          }
-        });
     }
 
     private void fetchExceedThrottleQuota() {
@@ -386,48 +362,6 @@ public class QuotaCache implements Stoppable {
       }
     }
 
-    private <K, V extends QuotaState> void fetch(final String type,
-      final ConcurrentMap<K, V> quotasMap, final Fetcher<K, V> fetcher) {
-      long now = EnvironmentEdgeManager.currentTime();
-      long evictPeriod = getPeriod() * EVICT_PERIOD_FACTOR;
-      // Find the quota entries to update
-      List<Get> gets = new ArrayList<>();
-      List<K> toRemove = new ArrayList<>();
-      for (Map.Entry<K, V> entry : quotasMap.entrySet()) {
-        long lastQuery = entry.getValue().getLastQuery();
-        if (lastQuery > 0 && (now - lastQuery) >= evictPeriod) {
-          toRemove.add(entry.getKey());
-        } else {
-          gets.add(fetcher.makeGet(entry));
-        }
-      }
-
-      for (final K key : toRemove) {
-        if (LOG.isTraceEnabled()) {
-          LOG.trace("evict " + type + " key=" + key);
-        }
-        quotasMap.remove(key);
-      }
-
-      // fetch and update the quota entries
-      if (!gets.isEmpty()) {
-        try {
-          for (Map.Entry<K, V> entry : fetcher.fetchEntries(gets).entrySet()) {
-            V quotaInfo = quotasMap.putIfAbsent(entry.getKey(), entry.getValue());
-            if (quotaInfo != null) {
-              quotaInfo.update(entry.getValue());
-            }
-
-            if (LOG.isTraceEnabled()) {
-              LOG.trace("refresh " + type + " key=" + entry.getKey() + " quotas=" + quotaInfo);
-            }
-          }
-        } catch (IOException e) {
-          LOG.warn("Unable to read " + type + " from quota table", e);
-        }
-      }
-    }
-
     /**
      * Update quota factors which is used to divide cluster scope quota into machine scope quota For
      * user/namespace/user over namespace quota, use [1 / RSNum] as machine factor. For table/user
@@ -503,6 +437,20 @@ public class QuotaCache implements Stoppable {
     }
   }
 
+  /** visible for testing */
+  static <K, V extends QuotaState> void updateNewCacheFromOld(Map<K, V> oldCache,
+    Map<K, V> newCache) {
+    for (Map.Entry<K, V> entry : oldCache.entrySet()) {
+      K key = entry.getKey();
+      if (newCache.containsKey(key)) {
+        V newState = newCache.get(key);
+        V oldState = entry.getValue();
+        oldState.update(newState);
+        newCache.put(key, oldState);
+      }
+    }
+  }
+
   static class RefreshableExpiringValueCache<T> {
     private final String name;
     private final LoadingCache<String, Optional<T>> cache;
@@ -543,9 +491,4 @@ public class QuotaCache implements Stoppable {
     T get() throws Exception;
   }
 
-  static interface Fetcher<Key, Value> {
-    Get makeGet(Map.Entry<Key, Value> entry);
-
-    Map<Key, Value> fetchEntries(List<Get> gets) throws IOException;
-  }
 }
