@@ -21,8 +21,8 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
+import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.TableName;
-import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.apache.yetus.audience.InterfaceStability;
 
@@ -42,24 +42,18 @@ public class UserQuotaState extends QuotaState {
   private Map<TableName, QuotaLimiter> tableLimiters = null;
   private boolean bypassGlobals = false;
 
-  public UserQuotaState() {
-    super();
-  }
-
-  public UserQuotaState(final long updateTs) {
-    super(updateTs);
-  }
-
   @Override
   public synchronized String toString() {
     StringBuilder builder = new StringBuilder();
-    builder.append("UserQuotaState(ts=" + getLastUpdate());
-    if (bypassGlobals) builder.append(" bypass-globals");
+    builder.append("UserQuotaState(");
+    if (bypassGlobals) {
+      builder.append("bypass-globals");
+    }
 
     if (isBypass()) {
       builder.append(" bypass");
     } else {
-      if (getGlobalLimiterWithoutUpdatingLastQuery() != NoopQuotaLimiter.get()) {
+      if (getGlobalLimiter() != NoopQuotaLimiter.get()) {
         builder.append(" global-limiter");
       }
 
@@ -86,7 +80,7 @@ public class UserQuotaState extends QuotaState {
   /** Returns true if there is no quota information associated to this object */
   @Override
   public synchronized boolean isBypass() {
-    return !bypassGlobals && getGlobalLimiterWithoutUpdatingLastQuery() == NoopQuotaLimiter.get()
+    return !bypassGlobals && getGlobalLimiter() == NoopQuotaLimiter.get()
       && (tableLimiters == null || tableLimiters.isEmpty())
       && (namespaceLimiters == null || namespaceLimiters.isEmpty());
   }
@@ -96,8 +90,8 @@ public class UserQuotaState extends QuotaState {
   }
 
   @Override
-  public synchronized void setQuotas(final Quotas quotas) {
-    super.setQuotas(quotas);
+  public synchronized void setQuotas(Configuration conf, final Quotas quotas) {
+    super.setQuotas(conf, quotas);
     bypassGlobals = quotas.getBypassGlobals();
   }
 
@@ -105,30 +99,30 @@ public class UserQuotaState extends QuotaState {
    * Add the quota information of the specified table. (This operation is part of the QuotaState
    * setup)
    */
-  public synchronized void setQuotas(final TableName table, Quotas quotas) {
-    tableLimiters = setLimiter(tableLimiters, table, quotas);
+  public synchronized void setQuotas(Configuration conf, final TableName table, Quotas quotas) {
+    tableLimiters = setLimiter(conf, tableLimiters, table, quotas);
   }
 
   /**
    * Add the quota information of the specified namespace. (This operation is part of the QuotaState
    * setup)
    */
-  public void setQuotas(final String namespace, Quotas quotas) {
-    namespaceLimiters = setLimiter(namespaceLimiters, namespace, quotas);
+  public void setQuotas(Configuration conf, final String namespace, Quotas quotas) {
+    namespaceLimiters = setLimiter(conf, namespaceLimiters, namespace, quotas);
   }
 
   public boolean hasTableLimiters() {
     return tableLimiters != null && !tableLimiters.isEmpty();
   }
 
-  private <K> Map<K, QuotaLimiter> setLimiter(Map<K, QuotaLimiter> limiters, final K key,
-    final Quotas quotas) {
+  private <K> Map<K, QuotaLimiter> setLimiter(Configuration conf, Map<K, QuotaLimiter> limiters,
+    final K key, final Quotas quotas) {
     if (limiters == null) {
       limiters = new HashMap<>();
     }
 
     QuotaLimiter limiter =
-      quotas.hasThrottle() ? QuotaLimiterFactory.fromThrottle(quotas.getThrottle()) : null;
+      quotas.hasThrottle() ? QuotaLimiterFactory.fromThrottle(conf, quotas.getThrottle()) : null;
     if (limiter != null && !limiter.isBypass()) {
       limiters.put(key, limiter);
     } else {
@@ -191,7 +185,6 @@ public class UserQuotaState extends QuotaState {
    * @return the quota limiter for the specified table
    */
   public synchronized QuotaLimiter getTableLimiter(final TableName table) {
-    lastQuery = EnvironmentEdgeManager.currentTime();
     if (tableLimiters != null) {
       QuotaLimiter limiter = tableLimiters.get(table);
       if (limiter != null) return limiter;
@@ -200,6 +193,6 @@ public class UserQuotaState extends QuotaState {
       QuotaLimiter limiter = namespaceLimiters.get(table.getNamespaceAsString());
       if (limiter != null) return limiter;
     }
-    return getGlobalLimiterWithoutUpdatingLastQuery();
+    return getGlobalLimiter();
   }
 }
