@@ -22,6 +22,7 @@ import java.util.ArrayList;
 import java.util.Comparator;
 import java.util.List;
 import java.util.PriorityQueue;
+import java.util.function.IntConsumer;
 import org.apache.hadoop.hbase.Cell;
 import org.apache.hadoop.hbase.CellComparator;
 import org.apache.hadoop.hbase.regionserver.ScannerContext.NextState;
@@ -104,6 +105,11 @@ public class KeyValueHeap extends NonReversedNonLazyKeyValueScanner
     return !this.current.isFileScanner();
   }
 
+  @Override
+  public void recordBlockSize(IntConsumer blockSizeConsumer) {
+    this.current.recordBlockSize(blockSizeConsumer);
+  }
+
   @Override
   public Cell next() throws IOException {
     if (this.current == null) {
