@@ -19,8 +19,10 @@ package org.apache.hadoop.hbase.regionserver.wal;
 
 import static org.apache.hadoop.hbase.regionserver.wal.AbstractFSWAL.WAL_AVOID_LOCAL_WRITES_DEFAULT;
 import static org.apache.hadoop.hbase.regionserver.wal.AbstractFSWAL.WAL_AVOID_LOCAL_WRITES_KEY;
-import static org.apache.hadoop.hbase.regionserver.wal.ProtobufLogReader.DEFAULT_WAL_TRAILER_WARN_SIZE;
-import static org.apache.hadoop.hbase.regionserver.wal.ProtobufLogReader.WAL_TRAILER_WARN_SIZE;
+import static org.apache.hadoop.hbase.regionserver.wal.AbstractProtobufWALReader.DEFAULT_WAL_TRAILER_WARN_SIZE;
+import static org.apache.hadoop.hbase.regionserver.wal.AbstractProtobufWALReader.PB_WAL_COMPLETE_MAGIC;
+import static org.apache.hadoop.hbase.regionserver.wal.AbstractProtobufWALReader.PB_WAL_MAGIC;
+import static org.apache.hadoop.hbase.regionserver.wal.AbstractProtobufWALReader.WAL_TRAILER_WARN_SIZE;
 
 import java.io.IOException;
 import java.io.OutputStream;
@@ -29,7 +31,6 @@ import java.util.concurrent.atomic.AtomicLong;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.hbase.HBaseInterfaceAudience;
 import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.codec.Codec;
 import org.apache.hadoop.hbase.io.asyncfs.monitor.StreamSlowMonitor;
@@ -55,13 +56,14 @@ import org.apache.hadoop.hbase.shaded.protobuf.generated.WALProtos.WALTrailer;
 /**
  * Base class for Protobuf log writer.
  */
-@InterfaceAudience.LimitedPrivate(HBaseInterfaceAudience.CONFIG)
+@InterfaceAudience.Private
 public abstract class AbstractProtobufLogWriter {
 
   private static final Logger LOG = LoggerFactory.getLogger(AbstractProtobufLogWriter.class);
 
   protected CompressionContext compressionContext;
   protected Configuration conf;
+  protected Encryptor encryptor;
   protected Codec.Encoder cellEncoder;
   protected WALCellCodec.ByteStringCompressor compressor;
   protected boolean trailerWritten;
@@ -77,64 +79,53 @@ public abstract class AbstractProtobufLogWriter {
     return WALCellCodec.create(conf, null, compressionContext);
   }
 
-  private WALHeader buildWALHeader0(Configuration conf, WALHeader.Builder builder) {
-    if (!builder.hasWriterClsName()) {
-      builder.setWriterClsName(getWriterClassName());
-    }
-    if (!builder.hasCellCodecClsName()) {
-      builder.setCellCodecClsName(WALCellCodec.getWALCellCodecClass(conf).getName());
-    }
+  private WALHeader buildWALHeader(Configuration conf, WALHeader.Builder builder)
+    throws IOException {
+    builder.setWriterClsName(getWriterClassName());
+    builder.setCellCodecClsName(WALCellCodec.getWALCellCodecClass(conf).getName());
     return builder.build();
   }
 
-  protected WALHeader buildWALHeader(Configuration conf, WALHeader.Builder builder)
+  private WALHeader buildSecureWALHeader(Configuration conf, WALHeader.Builder builder)
     throws IOException {
-    return buildWALHeader0(conf, builder);
-  }
+    EncryptionTest.testKeyProvider(conf);
+    EncryptionTest.testCipherProvider(conf);
 
-  // should be called in sub classes's buildWALHeader method to build WALHeader for secure
-  // environment. Do not forget to override the setEncryptor method as it will be called in this
-  // method to init your encryptor.
-  protected final WALHeader buildSecureWALHeader(Configuration conf, WALHeader.Builder builder)
-    throws IOException {
-    builder.setWriterClsName(getWriterClassName());
-    if (conf.getBoolean(HConstants.ENABLE_WAL_ENCRYPTION, false)) {
-      EncryptionTest.testKeyProvider(conf);
-      EncryptionTest.testCipherProvider(conf);
-
-      // Get an instance of our cipher
-      final String cipherName =
-        conf.get(HConstants.CRYPTO_WAL_ALGORITHM_CONF_KEY, HConstants.CIPHER_AES);
-      Cipher cipher = Encryption.getCipher(conf, cipherName);
-      if (cipher == null) {
-        throw new RuntimeException("Cipher '" + cipherName + "' is not available");
-      }
-
-      // Generate a random encryption key for this WAL
-      Key key = cipher.getRandomKey();
-      builder.setEncryptionKey(UnsafeByteOperations.unsafeWrap(EncryptionUtil.wrapKey(conf,
-        conf.get(HConstants.CRYPTO_WAL_KEY_NAME_CONF_KEY,
-          conf.get(HConstants.CRYPTO_MASTERKEY_NAME_CONF_KEY, User.getCurrent().getShortName())),
-        key)));
-
-      // Set up the encryptor
-      Encryptor encryptor = cipher.getEncryptor();
-      encryptor.setKey(key);
-      setEncryptor(encryptor);
-      if (LOG.isTraceEnabled()) {
-        LOG.trace("Initialized secure protobuf WAL: cipher=" + cipher.getName());
-      }
+    // Get an instance of our cipher
+    final String cipherName =
+      conf.get(HConstants.CRYPTO_WAL_ALGORITHM_CONF_KEY, HConstants.CIPHER_AES);
+    Cipher cipher = Encryption.getCipher(conf, cipherName);
+    if (cipher == null) {
+      throw new RuntimeException("Cipher '" + cipherName + "' is not available");
     }
+
+    // Generate a random encryption key for this WAL
+    Key key = cipher.getRandomKey();
+    builder.setEncryptionKey(UnsafeByteOperations.unsafeWrap(EncryptionUtil.wrapKey(conf,
+      conf.get(HConstants.CRYPTO_WAL_KEY_NAME_CONF_KEY,
+        conf.get(HConstants.CRYPTO_MASTERKEY_NAME_CONF_KEY, User.getCurrent().getShortName())),
+      key)));
+
+    // Set up the encryptor
+    Encryptor encryptor = cipher.getEncryptor();
+    encryptor.setKey(key);
+    this.encryptor = encryptor;
+    if (LOG.isTraceEnabled()) {
+      LOG.trace("Initialized secure protobuf WAL: cipher={}", cipher.getName());
+    }
+    builder.setWriterClsName(getWriterClassName());
     builder.setCellCodecClsName(SecureWALCellCodec.class.getName());
-    return buildWALHeader0(conf, builder);
+    return builder.build();
   }
 
-  // override this if you need a encryptor
-  protected void setEncryptor(Encryptor encryptor) {
-  }
-
-  protected String getWriterClassName() {
-    return getClass().getSimpleName();
+  private String getWriterClassName() {
+    // class name which is recognized by hbase-1.x to avoid ProtobufLogReader throwing error:
+    // IOException: Got unknown writer class: AsyncProtobufLogWriter
+    if (encryptor == null) {
+      return "ProtobufLogWriter";
+    } else {
+      return "SecureProtobufLogWriter";
+    }
   }
 
   private boolean initializeCompressionContext(Configuration conf, Path path) throws IOException {
@@ -189,10 +180,13 @@ public abstract class AbstractProtobufLogWriter {
         headerBuilder.setValueCompressionAlgorithm(
           CompressionContext.getValueCompressionAlgorithm(conf).ordinal());
       }
-      length.set(writeMagicAndWALHeader(ProtobufLogReader.PB_WAL_MAGIC,
-        buildWALHeader(conf, headerBuilder)));
-
-      initAfterHeader(doCompress);
+      if (conf.getBoolean(HConstants.ENABLE_WAL_ENCRYPTION, false)) {
+        length.set(writeMagicAndWALHeader(PB_WAL_MAGIC, buildSecureWALHeader(conf, headerBuilder)));
+        secureInitAfterHeader(doCompress, encryptor);
+      } else {
+        length.set(writeMagicAndWALHeader(PB_WAL_MAGIC, buildWALHeader(conf, headerBuilder)));
+        initAfterHeader(doCompress);
+      }
 
       // instantiate trailer to default value.
       trailer = WALTrailer.newBuilder().build();
@@ -208,7 +202,7 @@ public abstract class AbstractProtobufLogWriter {
     }
   }
 
-  private void initAfterHeader0(boolean doCompress) throws IOException {
+  private void initAfterHeader(boolean doCompress) throws IOException {
     WALCellCodec codec = getCodec(conf, this.compressionContext);
     this.cellEncoder = codec.getEncoder(getOutputStreamForCellEncoder());
     if (doCompress) {
@@ -218,21 +212,15 @@ public abstract class AbstractProtobufLogWriter {
     }
   }
 
-  protected void initAfterHeader(boolean doCompress) throws IOException {
-    initAfterHeader0(doCompress);
-  }
-
-  // should be called in sub classes's initAfterHeader method to init SecureWALCellCodec.
-  protected final void secureInitAfterHeader(boolean doCompress, Encryptor encryptor)
-    throws IOException {
-    if (conf.getBoolean(HConstants.ENABLE_WAL_ENCRYPTION, false) && encryptor != null) {
+  private void secureInitAfterHeader(boolean doCompress, Encryptor encryptor) throws IOException {
+    if (encryptor != null) {
       WALCellCodec codec = SecureWALCellCodec.getCodec(this.conf, encryptor);
       this.cellEncoder = codec.getEncoder(getOutputStreamForCellEncoder());
       // We do not support compression
       this.compressionContext = null;
       this.compressor = WALCellCodec.getNoneCompressor();
     } else {
-      initAfterHeader0(doCompress);
+      initAfterHeader(doCompress);
     }
   }
 
@@ -248,7 +236,7 @@ public abstract class AbstractProtobufLogWriter {
     return builder.build();
   }
 
-  protected void writeWALTrailer() {
+  protected final void writeWALTrailer() {
     try {
       int trailerSize = 0;
       if (this.trailer == null) {
@@ -261,7 +249,7 @@ public abstract class AbstractProtobufLogWriter {
         LOG.warn("Please investigate WALTrailer usage. Trailer size > maximum size : " + trailerSize
           + " > " + this.trailerWarnSize);
       }
-      length.set(writeWALTrailerAndMagic(trailer, ProtobufLogReader.PB_WAL_COMPLETE_MAGIC));
+      length.set(writeWALTrailerAndMagic(trailer, PB_WAL_COMPLETE_MAGIC));
       this.trailerWritten = true;
     } catch (IOException ioe) {
       LOG.warn("Failed to write trailer, non-fatal, continuing...", ioe);
