@@ -149,6 +149,9 @@ public class RecoveredReplicationSource extends ReplicationSource {
       boolean allTasksDone = workerThreads.values().stream().allMatch(w -> w.isFinished());
       if (allTasksDone) {
         this.getSourceMetrics().clear();
+        if (this.getReplicationEndpoint() != null) {
+          this.getReplicationEndpoint().stop();
+        }
         manager.removeRecoveredSource(this);
         LOG.info("Finished recovering queue {} with the following stats: {}", queueId, getStats());
       }
