@@ -427,6 +427,7 @@ public class RegionReplicaReplicationEndpoint extends HBaseReplicationEndpoint {
       // invalidate the cache and check from meta
       RegionLocations locations = null;
       boolean useCache = true;
+      int retries = 0;
       while (true) {
         // get the replicas of the primary region
         try {
@@ -439,12 +440,13 @@ public class RegionReplicaReplicationEndpoint extends HBaseReplicationEndpoint {
           // Replicas can take a while to come online. The cache may have only the primary. If we
           // keep going to the cache, we will not learn of the replicas and their locations after
           // they come online.
-          if (useCache && locations.size() == 1 && TableName.isMetaTableName(tableName)) {
-            if (tableDescriptors.get(tableName).getRegionReplication() > 1) {
+          if (useCache && locations.size() == 1) {
+            if (tableDescriptors.get(tableName).getRegionReplication() > 1 && retries <= 3) {
               // Make an obnoxious log here. See how bad this issue is. Add a timer if happening
               // too much.
               LOG.info("Skipping location cache; only one location found for {}", tableName);
               useCache = false;
+              retries++;
               continue;
             }
           }
@@ -488,6 +490,13 @@ public class RegionReplicaReplicationEndpoint extends HBaseReplicationEndpoint {
       }
 
       if (locations.size() == 1) {
+        if (LOG.isTraceEnabled()) {
+          LOG.trace("Skipping {} entries in table {} because only one region location was found",
+            entries.size(), tableName);
+          for (Entry entry : entries) {
+            LOG.trace("Skipping: {}", entry);
+          }
+        }
         return;
       }
 
