@@ -69,6 +69,10 @@ class ReplicationSourceWALActionListener implements WALActionsListener {
     if (ReplicationUtils.isReplicationForBulkLoadDataEnabled(conf)) {
       return;
     }
+    // Allow replication marker row to pass through.
+    if (WALEdit.isReplicationMarkerEdit(logEdit)) {
+      return;
+    }
     // For replay, or if all the cells are markers, do not need to store replication scope.
     if (
       logEdit.isReplay() || logEdit.getCells().stream().allMatch(c -> WALEdit.isMetaEditFamily(c))
