@@ -24,6 +24,7 @@ import java.util.Map;
 import javax.security.sasl.Sasl;
 import javax.security.sasl.SaslException;
 import javax.security.sasl.SaslServer;
+import org.apache.hadoop.hbase.io.crypto.aes.CryptoAES;
 import org.apache.hadoop.hbase.security.provider.AttemptingUserProvidingSaslServer;
 import org.apache.hadoop.hbase.security.provider.SaslServerAuthenticationProvider;
 import org.apache.hadoop.security.token.SecretManager;
@@ -40,7 +41,7 @@ public class HBaseSaslRpcServer {
 
   private final AttemptingUserProvidingSaslServer serverWithProvider;
   private final SaslServer saslServer;
-
+  private CryptoAES cryptoAES;
   private final Map<String, String> saslProps;
 
   public HBaseSaslRpcServer(SaslServerAuthenticationProvider provider,
@@ -64,16 +65,28 @@ public class HBaseSaslRpcServer {
     SaslUtil.safeDispose(saslServer);
   }
 
+  public void switchToCryptoAES(CryptoAES cryptoAES) {
+    this.cryptoAES = cryptoAES;
+  }
+
   public String getAttemptingUser() {
     return serverWithProvider.getAttemptingUser().map(Object::toString).orElse("Unknown");
   }
 
   public byte[] wrap(byte[] buf, int off, int len) throws SaslException {
-    return saslServer.wrap(buf, off, len);
+    if (cryptoAES != null) {
+      return cryptoAES.wrap(buf, off, len);
+    } else {
+      return saslServer.wrap(buf, off, len);
+    }
   }
 
   public byte[] unwrap(byte[] buf, int off, int len) throws SaslException {
-    return saslServer.unwrap(buf, off, len);
+    if (cryptoAES != null) {
+      return cryptoAES.unwrap(buf, off, len);
+    } else {
+      return saslServer.unwrap(buf, off, len);
+    }
   }
 
   public String getNegotiatedQop() {
@@ -99,4 +112,18 @@ public class HBaseSaslRpcServer {
     }
     return tokenIdentifier;
   }
+
+  /**
+   * Unwrap InvalidToken exception, otherwise return the one passed in.
+   */
+  public static Throwable unwrap(Throwable e) {
+    Throwable cause = e;
+    while (cause != null) {
+      if (cause instanceof InvalidToken) {
+        return cause;
+      }
+      cause = cause.getCause();
+    }
+    return e;
+  }
 }
