@@ -34,7 +34,9 @@ import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.client.Admin;
 import org.apache.hadoop.hbase.client.Connection;
 import org.apache.hadoop.hbase.client.ConnectionFactory;
+import org.apache.hadoop.hbase.ipc.RpcServer;
 import org.apache.hadoop.hbase.security.User;
+import org.apache.hadoop.hbase.security.access.AccessChecker;
 import org.apache.hadoop.hbase.security.access.PermissionStorage;
 import org.apache.hadoop.hbase.security.access.ShadedAccessControlUtil;
 import org.apache.hadoop.hbase.security.access.UserPermission;
@@ -148,6 +150,8 @@ public final class SnapshotDescriptionUtils {
   @Deprecated
   public static final String SNAPSHOT_TIMEOUT_MILLIS_KEY = "hbase.snapshot.master.timeoutMillis";
 
+  public static final String SNAPSHOT_CORRUPTED_FILE = "_CORRUPTED";
+
   private SnapshotDescriptionUtils() {
     // private constructor for utility class
   }
@@ -239,6 +243,15 @@ public final class SnapshotDescriptionUtils {
     return getSpecifiedSnapshotDir(getWorkingSnapshotDir(rootDir, conf), snapshotName);
   }
 
+  /**
+   * Get the flag file path if the snapshot is corrupted
+   * @param workingDir the directory where we build the specific snapshot
+   * @return {@link Path} snapshot corrupted flag file path
+   */
+  public static Path getCorruptedFlagFileForSnapshot(final Path workingDir) {
+    return new Path(workingDir, SNAPSHOT_CORRUPTED_FILE);
+  }
+
   /**
    * Get the directory within the given filepath to store the snapshot instance
    * @param snapshotsDir directory to store snapshot directory within
@@ -309,15 +322,15 @@ public final class SnapshotDescriptionUtils {
         "Descriptor doesn't apply to a table, so we can't build it.");
     }
 
+    SnapshotDescription.Builder builder = snapshot.toBuilder();
+
     // set the creation time, if one hasn't been set
     long time = snapshot.getCreationTime();
     if (time == SnapshotDescriptionUtils.NO_SNAPSHOT_START_TIME_SPECIFIED) {
       time = EnvironmentEdgeManager.currentTime();
       LOG.debug("Creation time not specified, setting to:" + time + " (current time:"
         + EnvironmentEdgeManager.currentTime() + ").");
-      SnapshotDescription.Builder builder = snapshot.toBuilder();
       builder.setCreationTime(time);
-      snapshot = builder.build();
     }
 
     long ttl = snapshot.getTtl();
@@ -334,8 +347,21 @@ public final class SnapshotDescriptionUtils {
       }
       ttl = defaultSnapshotTtl;
     }
-    SnapshotDescription.Builder builder = snapshot.toBuilder();
     builder.setTtl(ttl);
+
+    if (!snapshot.hasVersion()) {
+      builder.setVersion(SnapshotDescriptionUtils.SNAPSHOT_LAYOUT_VERSION);
+      LOG.debug("Snapshot {} VERSION not specified, setting to {}", snapshot.getName(),
+        SnapshotDescriptionUtils.SNAPSHOT_LAYOUT_VERSION);
+    }
+
+    RpcServer.getRequestUser().ifPresent(user -> {
+      if (AccessChecker.isAuthorizationSupported(conf)) {
+        builder.setOwner(user.getShortName());
+        LOG.debug("Set {} as owner of Snapshot", user.getShortName());
+      }
+    });
+
     snapshot = builder.build();
 
     // set the acl to snapshot if security feature is enabled.
