@@ -45,6 +45,7 @@ import org.apache.hadoop.hbase.client.Connection;
 import org.apache.hadoop.hbase.client.ConnectionFactory;
 import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.util.Tool;
 import org.apache.hadoop.util.ToolRunner;
 import org.apache.yetus.audience.InterfaceAudience;
@@ -419,7 +420,7 @@ public class MajorCompactor extends Configured implements Tool {
       return -1;
     }
     String tableName = commandLine.getOptionValue("table");
-    String cf = commandLine.getOptionValue("cf", null);
+    String cf = commandLine.getOptionValue("cf");
     Set<String> families = Sets.newHashSet();
     if (cf != null) {
       Iterables.addAll(families, Splitter.on(",").split(cf));
@@ -427,8 +428,8 @@ public class MajorCompactor extends Configured implements Tool {
 
     Configuration configuration = getConf();
     int concurrency = Integer.parseInt(commandLine.getOptionValue("servers"));
-    long minModTime = Long.parseLong(
-      commandLine.getOptionValue("minModTime", String.valueOf(System.currentTimeMillis())));
+    long minModTime = Long.parseLong(commandLine.getOptionValue("minModTime",
+      String.valueOf(EnvironmentEdgeManager.currentTime())));
     String quorum =
       commandLine.getOptionValue("zk", configuration.get(HConstants.ZOOKEEPER_QUORUM));
     String rootDir = commandLine.getOptionValue("rootDir", configuration.get(HConstants.HBASE_DIR));
