@@ -18,13 +18,15 @@
 package org.apache.hadoop.hbase.client;
 
 import static org.apache.hadoop.hbase.client.AsyncProcess.START_LOG_ERRORS_AFTER_COUNT_KEY;
-import static org.hamcrest.CoreMatchers.instanceOf;
-import static org.hamcrest.CoreMatchers.startsWith;
 import static org.hamcrest.MatcherAssert.assertThat;
+import static org.hamcrest.Matchers.containsString;
+import static org.hamcrest.Matchers.instanceOf;
+import static org.hamcrest.Matchers.startsWith;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertThrows;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
@@ -105,6 +107,7 @@ public class TestAsyncReplicationAdminApi extends TestAsyncAdminBase {
         queueStorage.removeQueue(serverName, queue);
       }
     }
+    admin.replicationPeerModificationSwitch(true).join();
   }
 
   @Test
@@ -519,7 +522,7 @@ public class TestAsyncReplicationAdminApi extends TestAsyncAdminBase {
     }
   }
 
-  /*
+  /**
    * Tests that admin api throws ReplicationPeerNotFoundException if peer doesn't exist.
    */
   @Test
@@ -532,4 +535,19 @@ public class TestAsyncReplicationAdminApi extends TestAsyncAdminBase {
       assertThat(e.getCause(), instanceOf(ReplicationPeerNotFoundException.class));
     }
   }
+
+  @Test
+  public void testReplicationPeerModificationSwitch() throws Exception {
+    assertTrue(admin.isReplicationPeerModificationEnabled().get());
+    // disable modification, should returns true as it is enabled by default and the above
+    // assertion has confirmed it
+    assertTrue(admin.replicationPeerModificationSwitch(false).get());
+    ExecutionException error = assertThrows(ExecutionException.class, () -> admin
+      .addReplicationPeer(ID_ONE, ReplicationPeerConfig.newBuilder().setClusterKey(KEY_ONE).build())
+      .get());
+    assertThat(error.getCause().getMessage(),
+      containsString("Replication peer modification disabled"));
+    // enable again, and the previous value should be false
+    assertFalse(admin.replicationPeerModificationSwitch(true).get());
+  }
 }
