@@ -17,9 +17,10 @@
  */
 package org.apache.hadoop.hbase.coprocessor;
 
+import static org.junit.Assert.assertTrue;
+
 import java.io.IOException;
 import java.util.Optional;
-import junit.framework.TestCase;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hbase.Coprocessor;
@@ -39,13 +40,16 @@ import org.apache.hadoop.hbase.regionserver.RegionServerServices;
 import org.apache.hadoop.hbase.testclassification.CoprocessorTests;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
+import org.apache.hadoop.hbase.util.Threads;
 import org.apache.hadoop.hbase.wal.WALEdit;
 import org.junit.ClassRule;
+import org.junit.Test;
 import org.junit.experimental.categories.Category;
 import org.mockito.Mockito;
 
 @Category({ CoprocessorTests.class, SmallTests.class })
-public class TestRegionObserverStacking extends TestCase {
+public class TestRegionObserverStacking {
 
   @ClassRule
   public static final HBaseClassTestRule CLASS_RULE =
@@ -65,11 +69,8 @@ public class TestRegionObserverStacking extends TestCase {
     @Override
     public void postPut(final ObserverContext<RegionCoprocessorEnvironment> c, final Put put,
       final WALEdit edit, final Durability durability) throws IOException {
-      id = System.currentTimeMillis();
-      try {
-        Thread.sleep(10);
-      } catch (InterruptedException ex) {
-      }
+      id = EnvironmentEdgeManager.currentTime();
+      Threads.sleepWithoutInterrupt(10);
     }
   }
 
@@ -84,11 +85,8 @@ public class TestRegionObserverStacking extends TestCase {
     @Override
     public void postPut(final ObserverContext<RegionCoprocessorEnvironment> c, final Put put,
       final WALEdit edit, final Durability durability) throws IOException {
-      id = System.currentTimeMillis();
-      try {
-        Thread.sleep(10);
-      } catch (InterruptedException ex) {
-      }
+      id = EnvironmentEdgeManager.currentTime();
+      Threads.sleepWithoutInterrupt(10);
     }
   }
 
@@ -103,11 +101,8 @@ public class TestRegionObserverStacking extends TestCase {
     @Override
     public void postPut(final ObserverContext<RegionCoprocessorEnvironment> c, final Put put,
       final WALEdit edit, final Durability durability) throws IOException {
-      id = System.currentTimeMillis();
-      try {
-        Thread.sleep(10);
-      } catch (InterruptedException ex) {
-      }
+      id = EnvironmentEdgeManager.currentTime();
+      Threads.sleepWithoutInterrupt(10);
     }
   }
 
@@ -132,6 +127,7 @@ public class TestRegionObserverStacking extends TestCase {
     return r;
   }
 
+  @Test
   public void testRegionObserverStacking() throws Exception {
     byte[] ROW = Bytes.toBytes("testRow");
     byte[] TABLE = Bytes.toBytes(this.getClass().getSimpleName());
