@@ -141,8 +141,8 @@ public class TestWALObserver {
     this.hbaseRootDir = CommonFSUtils.getRootDir(conf);
     this.hbaseWALRootDir = CommonFSUtils.getWALRootDir(conf);
     this.oldLogDir = new Path(this.hbaseWALRootDir, HConstants.HREGION_OLDLOGDIR_NAME);
-    String serverName =
-      ServerName.valueOf(currentTest.getMethodName(), 16010, System.currentTimeMillis()).toString();
+    String serverName = ServerName
+      .valueOf(currentTest.getMethodName(), 16010, EnvironmentEdgeManager.currentTime()).toString();
     this.logDir =
       new Path(this.hbaseWALRootDir, AbstractFSWALProvider.getWALDirectoryName(serverName));
 
@@ -346,7 +346,8 @@ public class TestWALObserver {
         LOG.info("WALSplit path == " + p);
         // Make a new wal for new region open.
         final WALFactory wals2 = new WALFactory(conf,
-          ServerName.valueOf(currentTest.getMethodName() + "2", 16010, System.currentTimeMillis())
+          ServerName
+            .valueOf(currentTest.getMethodName() + "2", 16010, EnvironmentEdgeManager.currentTime())
             .toString());
         WAL wal2 = wals2.getWAL(null);
         HRegion region = HRegion.openHRegion(newConf, FileSystem.get(newConf), hbaseRootDir, hri,
