@@ -19,6 +19,7 @@ package org.apache.hadoop.hbase.io.hfile;
 
 import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
@@ -39,6 +40,7 @@ import org.apache.hadoop.hbase.MultithreadedTestUtil.TestThread;
 import org.apache.hadoop.hbase.io.ByteBuffAllocator;
 import org.apache.hadoop.hbase.io.HeapSize;
 import org.apache.hadoop.hbase.io.compress.Compression;
+import org.apache.hadoop.hbase.io.encoding.DataBlockEncoding;
 import org.apache.hadoop.hbase.io.hfile.bucket.BucketCache;
 import org.apache.hadoop.hbase.nio.ByteBuff;
 import org.apache.hadoop.hbase.util.Bytes;
@@ -161,6 +163,8 @@ public class CacheTestUtils {
       }
     }
 
+    testConvertToJSON(toBeTested);
+
   }
 
   public static void hammerSingleKey(final BlockCache toBeTested, int numThreads, int numQueries)
@@ -279,7 +283,8 @@ public class CacheTestUtils {
     return generateBlocksForPath(blockSize, numBlocks, null);
   }
 
-  public static HFileBlockPair[] generateBlocksForPath(int blockSize, int numBlocks, Path path) {
+  public static HFileBlockPair[] generateBlocksForPath(int blockSize, int numBlocks, Path path,
+    boolean encoded) {
     HFileBlockPair[] returnedBlocks = new HFileBlockPair[numBlocks];
     Random rand = ThreadLocalRandom.current();
     HashSet<String> usedStrings = new HashSet<>();
@@ -298,12 +303,13 @@ public class CacheTestUtils {
       HFileContext meta =
         new HFileContextBuilder().withHBaseCheckSum(false).withIncludesMvcc(includesMemstoreTS)
           .withIncludesTags(false).withCompression(Compression.Algorithm.NONE)
-          .withBytesPerCheckSum(0).withChecksumType(ChecksumType.NULL).build();
-      HFileBlock generated =
-        new HFileBlock(BlockType.DATA, onDiskSizeWithoutHeader, uncompressedSizeWithoutHeader,
-          prevBlockOffset, ByteBuff.wrap(cachedBuffer), HFileBlock.DONT_FILL_HEADER, blockSize,
-          onDiskSizeWithoutHeader + HConstants.HFILEBLOCK_HEADER_SIZE, -1, meta,
-          ByteBuffAllocator.HEAP);
+          .withDataBlockEncoding(DataBlockEncoding.FAST_DIFF).withBytesPerCheckSum(0)
+          .withChecksumType(ChecksumType.NULL).build();
+      HFileBlock generated = new HFileBlock(encoded ? BlockType.ENCODED_DATA : BlockType.DATA,
+        onDiskSizeWithoutHeader, uncompressedSizeWithoutHeader, prevBlockOffset,
+        ByteBuff.wrap(cachedBuffer), HFileBlock.DONT_FILL_HEADER, blockSize,
+        onDiskSizeWithoutHeader + HConstants.HFILEBLOCK_HEADER_SIZE, -1, meta,
+        ByteBuffAllocator.HEAP);
       String key = null;
       long offset = 0;
       if (path != null) {
@@ -317,12 +323,17 @@ public class CacheTestUtils {
         }
       }
       returnedBlocks[i] = new HFileBlockPair();
-      returnedBlocks[i].blockName = new BlockCacheKey(key, offset);
+      returnedBlocks[i].blockName =
+        new BlockCacheKey(key, offset, true, encoded ? BlockType.ENCODED_DATA : BlockType.DATA);
       returnedBlocks[i].block = generated;
     }
     return returnedBlocks;
   }
 
+  public static HFileBlockPair[] generateBlocksForPath(int blockSize, int numBlocks, Path path) {
+    return generateBlocksForPath(blockSize, numBlocks, path, false);
+  }
+
   public static class HFileBlockPair {
     BlockCacheKey blockName;
     HFileBlock block;
@@ -351,4 +362,13 @@ public class CacheTestUtils {
       }
     }
   }
+
+  public static void testConvertToJSON(BlockCache toBeTested) {
+    try {
+      String json = BlockCacheUtil.toJSON(toBeTested);
+      assertNotNull(json);
+    } catch (Exception e) {
+      fail("conversion to JSON should not fail, exception is:" + e);
+    }
+  }
 }
