@@ -268,6 +268,10 @@ public class TestBytesReadFromFs {
 
     CacheConfig cacheConfig = new CacheConfig(conf);
     HFile.Reader reader = new HFilePreadReader(readerContext, hfile, cacheConfig, conf);
+    // Since HBASE-28466, we call fileInfo.initMetaAndIndex inside HFilePreadReader,
+    // which reads some blocks and increment the counters, so we need to reset it here.
+    ThreadLocalServerSideScanMetrics.getBytesReadFromFsAndReset();
+    ThreadLocalServerSideScanMetrics.getBlockReadOpsCountAndReset();
     HFileBlock.FSReader blockReader = reader.getUncachedBlockReader();
 
     // Create iterator for reading root index block
@@ -327,7 +331,7 @@ public class TestBytesReadFromFs {
     readLoadOnOpenDataSection(path, true);
 
     CacheConfig cacheConf = new CacheConfig(conf);
-    StoreFileInfo storeFileInfo = new StoreFileInfo(conf, fs, path, true);
+    StoreFileInfo storeFileInfo = StoreFileInfo.createStoreFileInfoForHFile(conf, fs, path, true);
     HStoreFile sf = new HStoreFile(storeFileInfo, bt, cacheConf);
 
     // Read HFile trailer and load-on-open data section
