@@ -137,7 +137,7 @@ public class TestHFileDataBlockEncoder {
   }
 
   /**
-   * Test encoding. n
+   * Test encoding.
    */
   @Test
   public void testEncoding() throws IOException {
@@ -147,7 +147,7 @@ public class TestHFileDataBlockEncoder {
 
   /**
    * Test encoding with offheap keyvalue. This test just verifies if the encoders work with DBB and
-   * does not use the getXXXArray() API n
+   * does not use the getXXXArray() API
    */
   @Test
   public void testEncodingWithOffheapKeyValue() throws IOException {
@@ -239,7 +239,7 @@ public class TestHFileDataBlockEncoder {
         HFileDataBlockEncoder dbe = (diskAlgo == DataBlockEncoding.NONE)
           ? NoOpDataBlockEncoder.INSTANCE
           : new HFileDataBlockEncoderImpl(diskAlgo);
-        configurations.add(new Object[] { dbe, new Boolean(includesMemstoreTS) });
+        configurations.add(new Object[] { dbe, includesMemstoreTS });
       }
     }
 
