@@ -92,9 +92,9 @@ public class TestHFileEncryption {
     cryptoContext.setKey(key);
   }
 
-  private int writeBlock(FSDataOutputStream os, HFileContext fileContext, int size)
-    throws IOException {
-    HFileBlock.Writer hbw = new HFileBlock.Writer(null, fileContext);
+  private int writeBlock(Configuration conf, FSDataOutputStream os, HFileContext fileContext,
+    int size) throws IOException {
+    HFileBlock.Writer hbw = new HFileBlock.Writer(conf, null, fileContext);
     DataOutputStream dos = hbw.startWriting(BlockType.DATA);
     for (int j = 0; j < size; j++) {
       dos.writeInt(j);
@@ -112,7 +112,8 @@ public class TestHFileEncryption {
     HFileBlock b = hbr.readBlockData(pos, -1, false, false, true);
     assertEquals(0, HFile.getAndResetChecksumFailuresCount());
     b.sanityCheck();
-    assertFalse(b.isUnpacked());
+    assertFalse(
+      (b.getHFileContext().getCompression() != Compression.Algorithm.NONE) && b.isUnpacked());
     b = b.unpack(ctx, hbr);
     LOG.info(
       "Read a block at " + pos + " with" + " onDiskSizeWithHeader=" + b.getOnDiskSizeWithHeader()
@@ -136,7 +137,7 @@ public class TestHFileEncryption {
     for (int i = 0; i < blocks; i++) {
       blockSizes[i] = (1024 + rand.nextInt(1024 * 63)) / Bytes.SIZEOF_INT;
     }
-    for (Compression.Algorithm compression : TestHFileBlock.COMPRESSION_ALGORITHMS) {
+    for (Compression.Algorithm compression : HBaseTestingUtility.COMPRESSION_ALGORITHMS) {
       Path path = new Path(TEST_UTIL.getDataTestDir(), "block_v3_" + compression + "_AES");
       LOG.info("testDataBlockEncryption: encryption=AES compression=" + compression);
       long totalSize = 0;
@@ -145,7 +146,7 @@ public class TestHFileEncryption {
       FSDataOutputStream os = fs.create(path);
       try {
         for (int i = 0; i < blocks; i++) {
-          totalSize += writeBlock(os, fileContext, blockSizes[i]);
+          totalSize += writeBlock(TEST_UTIL.getConfiguration(), os, fileContext, blockSizes[i]);
         }
       } finally {
         os.close();
@@ -155,8 +156,8 @@ public class TestHFileEncryption {
         new ReaderContextBuilder().withInputStreamWrapper(new FSDataInputStreamWrapper(is))
           .withFilePath(path).withFileSystem(fs).withFileSize(totalSize).build();
       try {
-        HFileBlock.FSReaderImpl hbr =
-          new HFileBlock.FSReaderImpl(context, fileContext, ByteBuffAllocator.HEAP);
+        HFileBlock.FSReaderImpl hbr = new HFileBlock.FSReaderImpl(context, fileContext,
+          ByteBuffAllocator.HEAP, TEST_UTIL.getConfiguration());
         long pos = 0;
         for (int i = 0; i < blocks; i++) {
           pos += readAndVerifyBlock(pos, fileContext, hbr, blockSizes[i]);
@@ -210,14 +211,14 @@ public class TestHFileEncryption {
     Configuration conf = TEST_UTIL.getConfiguration();
     CacheConfig cacheConf = new CacheConfig(conf);
     for (DataBlockEncoding encoding : DataBlockEncoding.values()) {
-      for (Compression.Algorithm compression : TestHFileBlock.COMPRESSION_ALGORITHMS) {
+      for (Compression.Algorithm compression : HBaseTestingUtility.COMPRESSION_ALGORITHMS) {
         HFileContext fileContext = new HFileContextBuilder().withBlockSize(4096) // small blocks
           .withEncryptionContext(cryptoContext).withCompression(compression)
           .withDataBlockEncoding(encoding).build();
         // write a new test HFile
         LOG.info("Writing with " + fileContext);
-        Path path =
-          new Path(TEST_UTIL.getDataTestDir(), TEST_UTIL.getRandomUUID().toString() + ".hfile");
+        Path path = new Path(TEST_UTIL.getDataTestDir(),
+          HBaseTestingUtility.getRandomUUID().toString() + ".hfile");
         FSDataOutputStream out = fs.create(path);
         HFile.Writer writer = HFile.getWriterFactory(conf, cacheConf).withOutputStream(out)
           .withFileContext(fileContext).create();
@@ -238,7 +239,7 @@ public class TestHFileEncryption {
         try {
           FixedFileTrailer trailer = reader.getTrailer();
           assertNotNull(trailer.getEncryptionKey());
-          scanner = reader.getScanner(false, false);
+          scanner = reader.getScanner(conf, false, false);
           assertTrue("Initial seekTo failed", scanner.seekTo());
           do {
             Cell kv = scanner.getCell();
@@ -258,7 +259,7 @@ public class TestHFileEncryption {
         Random rand = ThreadLocalRandom.current();
         reader = HFile.createReader(fs, path, cacheConf, true, conf);
         try {
-          scanner = reader.getScanner(false, true);
+          scanner = reader.getScanner(conf, false, true);
           assertTrue("Initial seekTo failed", scanner.seekTo());
           for (i = 0; i < 100; i++) {
             KeyValue kv = testKvs.get(rand.nextInt(testKvs.size()));
