@@ -33,6 +33,8 @@ import static org.hamcrest.Matchers.containsString;
 import static org.hamcrest.Matchers.everyItem;
 import static org.hamcrest.Matchers.greaterThanOrEqualTo;
 import static org.hamcrest.Matchers.hasItem;
+import static org.hamcrest.Matchers.instanceOf;
+import static org.hamcrest.Matchers.startsWith;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
@@ -41,8 +43,10 @@ import static org.junit.Assert.assertThrows;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.spy;
 import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.when;
 import static org.mockito.internal.verification.VerificationModeFactory.times;
 
 import io.opentelemetry.api.common.AttributeKey;
@@ -66,11 +70,16 @@ import org.apache.hadoop.hbase.HBaseConfiguration;
 import org.apache.hadoop.hbase.KeyValue;
 import org.apache.hadoop.hbase.MatcherPredicate;
 import org.apache.hadoop.hbase.Server;
+import org.apache.hadoop.hbase.ServerName;
 import org.apache.hadoop.hbase.Waiter;
 import org.apache.hadoop.hbase.ipc.RpcServer.BlockingServiceAndInterface;
 import org.apache.hadoop.hbase.nio.ByteBuff;
+import org.apache.hadoop.hbase.regionserver.HRegionServer;
+import org.apache.hadoop.hbase.security.User;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.io.compress.GzipCodec;
+import org.apache.hadoop.ipc.RemoteException;
 import org.apache.hadoop.util.StringUtils;
 import org.hamcrest.Matcher;
 import org.junit.Before;
@@ -82,8 +91,9 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.hbase.thirdparty.com.google.common.collect.ImmutableList;
 import org.apache.hbase.thirdparty.com.google.common.collect.Lists;
+import org.apache.hbase.thirdparty.com.google.protobuf.BlockingRpcChannel;
+import org.apache.hbase.thirdparty.com.google.protobuf.RpcChannel;
 import org.apache.hbase.thirdparty.com.google.protobuf.ServiceException;
-import org.apache.hbase.thirdparty.io.netty.channel.Channel;
 
 import org.apache.hadoop.hbase.shaded.ipc.protobuf.generated.TestProtos.EchoRequestProto;
 import org.apache.hadoop.hbase.shaded.ipc.protobuf.generated.TestProtos.EchoResponseProto;
@@ -93,6 +103,9 @@ import org.apache.hadoop.hbase.shaded.ipc.protobuf.generated.TestProtos.PauseReq
 import org.apache.hadoop.hbase.shaded.ipc.protobuf.generated.TestRpcServiceProtos.TestProtobufRpcProto.BlockingInterface;
 import org.apache.hadoop.hbase.shaded.ipc.protobuf.generated.TestRpcServiceProtos.TestProtobufRpcProto.Interface;
 import org.apache.hadoop.hbase.shaded.protobuf.ProtobufUtil;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.RegistryProtos.ConnectionRegistryService;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.RegistryProtos.GetConnectionRegistryRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.RegistryProtos.GetConnectionRegistryResponse;
 
 /**
  * Some basic ipc tests.
@@ -106,12 +119,17 @@ public abstract class AbstractTestIPC {
 
   protected static final Configuration CONF = HBaseConfiguration.create();
 
-  private RpcServer createRpcServer(Server server, String name,
+  protected RpcServer createRpcServer(Server server, String name,
     List<BlockingServiceAndInterface> services, InetSocketAddress bindAddress, Configuration conf,
     RpcScheduler scheduler) throws IOException {
     return RpcServerFactory.createRpcServer(server, name, services, bindAddress, conf, scheduler);
   }
 
+  private RpcServer createRpcServer(String name, List<BlockingServiceAndInterface> services,
+    InetSocketAddress bindAddress, Configuration conf, RpcScheduler scheduler) throws IOException {
+    return createRpcServer(null, name, services, bindAddress, conf, scheduler);
+  }
+
   protected abstract AbstractRpcClient<?> createRpcClientNoCodec(Configuration conf);
 
   @Rule
@@ -130,11 +148,11 @@ public abstract class AbstractTestIPC {
    */
   @Test
   public void testNoCodec() throws IOException, ServiceException {
-    Configuration conf = HBaseConfiguration.create();
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
-    try (AbstractRpcClient<?> client = createRpcClientNoCodec(conf)) {
+    try (AbstractRpcClient<?> client = createRpcClientNoCodec(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       HBaseRpcController pcrc = new HBaseRpcControllerImpl();
@@ -156,18 +174,18 @@ public abstract class AbstractTestIPC {
    */
   @Test
   public void testCompressCellBlock() throws IOException, ServiceException {
-    Configuration conf = new Configuration(HBaseConfiguration.create());
-    conf.set("hbase.client.rpc.compressor", GzipCodec.class.getCanonicalName());
+    Configuration clientConf = new Configuration(CONF);
+    clientConf.set("hbase.client.rpc.compressor", GzipCodec.class.getCanonicalName());
     List<Cell> cells = new ArrayList<>();
     int count = 3;
     for (int i = 0; i < count; i++) {
       cells.add(CELL);
     }
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
 
-    try (AbstractRpcClient<?> client = createRpcClient(conf)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       HBaseRpcController pcrc = new HBaseRpcControllerImpl(CellUtil.createCellScanner(cells));
@@ -192,11 +210,11 @@ public abstract class AbstractTestIPC {
 
   @Test
   public void testRTEDuringConnectionSetup() throws Exception {
-    Configuration conf = HBaseConfiguration.create();
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
-    try (AbstractRpcClient<?> client = createRpcClientRTEDuringConnectionSetup(conf)) {
+    try (AbstractRpcClient<?> client = createRpcClientRTEDuringConnectionSetup(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       stub.ping(null, EmptyRequestProto.getDefaultInstance());
@@ -214,12 +232,13 @@ public abstract class AbstractTestIPC {
    */
   @Test
   public void testRpcScheduler() throws IOException, ServiceException, InterruptedException {
+    Configuration clientConf = new Configuration(CONF);
     RpcScheduler scheduler = spy(new FifoRpcScheduler(CONF, 1));
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, scheduler);
     verify(scheduler).init(any(RpcScheduler.Context.class));
-    try (AbstractRpcClient<?> client = createRpcClient(CONF)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       verify(scheduler).start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
@@ -237,12 +256,12 @@ public abstract class AbstractTestIPC {
   /** Tests that the rpc scheduler is called when requests arrive. */
   @Test
   public void testRpcMaxRequestSize() throws IOException, ServiceException {
-    Configuration conf = new Configuration(CONF);
-    conf.setInt(RpcServer.MAX_REQUEST_SIZE, 1000);
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    clientConf.setInt(RpcServer.MAX_REQUEST_SIZE, 1000);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
-      new InetSocketAddress("localhost", 0), conf, new FifoRpcScheduler(conf, 1));
-    try (AbstractRpcClient<?> client = createRpcClient(conf)) {
+      new InetSocketAddress("localhost", 0), clientConf, new FifoRpcScheduler(clientConf, 1));
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       StringBuilder message = new StringBuilder(1200);
@@ -271,11 +290,12 @@ public abstract class AbstractTestIPC {
   @Test
   public void testRpcServerForNotNullRemoteAddressInCallObject()
     throws IOException, ServiceException {
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
     InetSocketAddress localAddr = new InetSocketAddress("localhost", 0);
-    try (AbstractRpcClient<?> client = createRpcClient(CONF)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       assertEquals(localAddr.getAddress().getHostAddress(),
@@ -287,10 +307,11 @@ public abstract class AbstractTestIPC {
 
   @Test
   public void testRemoteError() throws IOException, ServiceException {
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
-    try (AbstractRpcClient<?> client = createRpcClient(CONF)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       stub.error(null, EmptyRequestProto.getDefaultInstance());
@@ -306,10 +327,11 @@ public abstract class AbstractTestIPC {
 
   @Test
   public void testTimeout() throws IOException {
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
-    try (AbstractRpcClient<?> client = createRpcClient(CONF)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       HBaseRpcController pcrc = new HBaseRpcControllerImpl();
@@ -364,39 +386,7 @@ public abstract class AbstractTestIPC {
     }
   }
 
-  private static class FailingNettyRpcServer extends NettyRpcServer {
-
-    FailingNettyRpcServer(Server server, String name,
-      List<RpcServer.BlockingServiceAndInterface> services, InetSocketAddress bindAddress,
-      Configuration conf, RpcScheduler scheduler) throws IOException {
-      super(server, name, services, bindAddress, conf, scheduler, true);
-    }
-
-    static final class FailingConnection extends NettyServerRpcConnection {
-      private FailingConnection(FailingNettyRpcServer rpcServer, Channel channel) {
-        super(rpcServer, channel);
-      }
-
-      @Override
-      public void processRequest(ByteBuff buf) throws IOException, InterruptedException {
-        // this will throw exception after the connection header is read, and an RPC is sent
-        // from client
-        throw new DoNotRetryIOException("Failing for test");
-      }
-    }
-
-    @Override
-    protected NettyRpcServerPreambleHandler createNettyRpcServerPreambleHandler() {
-      return new NettyRpcServerPreambleHandler(FailingNettyRpcServer.this) {
-        @Override
-        protected NettyServerRpcConnection createNettyServerRpcConnection(Channel channel) {
-          return new FailingConnection(FailingNettyRpcServer.this, channel);
-        }
-      };
-    }
-  }
-
-  private RpcServer createTestFailingRpcServer(final String name,
+  protected RpcServer createTestFailingRpcServer(final String name,
     final List<BlockingServiceAndInterface> services, final InetSocketAddress bindAddress,
     Configuration conf, RpcScheduler scheduler) throws IOException {
     if (rpcServerImpl.equals(NettyRpcServer.class)) {
@@ -409,12 +399,12 @@ public abstract class AbstractTestIPC {
   /** Tests that the connection closing is handled by the client with outstanding RPC calls */
   @Test
   public void testConnectionCloseWithOutstandingRPCs() throws InterruptedException, IOException {
-    Configuration conf = new Configuration(CONF);
+    Configuration clientConf = new Configuration(CONF);
     RpcServer rpcServer = createTestFailingRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
 
-    try (AbstractRpcClient<?> client = createRpcClient(conf)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       EchoRequestProto param = EchoRequestProto.newBuilder().setMessage("hello").build();
@@ -429,11 +419,11 @@ public abstract class AbstractTestIPC {
 
   @Test
   public void testAsyncEcho() throws IOException {
-    Configuration conf = HBaseConfiguration.create();
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
-    try (AbstractRpcClient<?> client = createRpcClient(conf)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       Interface stub = newStub(client, rpcServer.getListenerAddress());
       int num = 10;
@@ -447,10 +437,11 @@ public abstract class AbstractTestIPC {
         callbackList.add(done);
       }
       for (int i = 0; i < num; i++) {
+        EchoResponseProto resp = callbackList.get(i).get();
         HBaseRpcController pcrc = pcrcList.get(i);
+        assertEquals("hello-" + i, resp.getMessage());
         assertFalse(pcrc.failed());
         assertNull(pcrc.cellScanner());
-        assertEquals("hello-" + i, callbackList.get(i).get().getMessage());
       }
     } finally {
       rpcServer.stop();
@@ -459,8 +450,9 @@ public abstract class AbstractTestIPC {
 
   @Test
   public void testAsyncRemoteError() throws IOException {
-    AbstractRpcClient<?> client = createRpcClient(CONF);
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    AbstractRpcClient<?> client = createRpcClient(clientConf);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
     try {
@@ -483,10 +475,11 @@ public abstract class AbstractTestIPC {
 
   @Test
   public void testAsyncTimeout() throws IOException {
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
-    try (AbstractRpcClient<?> client = createRpcClient(CONF)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       Interface stub = newStub(client, rpcServer.getListenerAddress());
       List<HBaseRpcController> pcrcList = new ArrayList<>();
@@ -564,10 +557,11 @@ public abstract class AbstractTestIPC {
 
   @Test
   public void testTracingSuccessIpc() throws IOException, ServiceException {
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
-    try (AbstractRpcClient<?> client = createRpcClient(CONF)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       stub.pause(null, PauseRequestProto.newBuilder().setMs(100).build());
@@ -592,10 +586,11 @@ public abstract class AbstractTestIPC {
 
   @Test
   public void testTracingErrorIpc() throws IOException {
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer",
+    Configuration clientConf = new Configuration(CONF);
+    RpcServer rpcServer = createRpcServer("testRpcServer",
       Lists.newArrayList(new RpcServer.BlockingServiceAndInterface(SERVICE, null)),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
-    try (AbstractRpcClient<?> client = createRpcClient(CONF)) {
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
       rpcServer.start();
       BlockingInterface stub = newBlockingStub(client, rpcServer.getListenerAddress());
       // use the ISA from the running server so that we can get the port selected.
@@ -628,7 +623,7 @@ public abstract class AbstractTestIPC {
   @Test
   public void testBadPreambleHeader() throws Exception {
     Configuration clientConf = new Configuration(CONF);
-    RpcServer rpcServer = createRpcServer(null, "testRpcServer", Collections.emptyList(),
+    RpcServer rpcServer = createRpcServer("testRpcServer", Collections.emptyList(),
       new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
     try (AbstractRpcClient<?> client = createBadAuthRpcClient(clientConf)) {
       rpcServer.start();
@@ -650,4 +645,62 @@ public abstract class AbstractTestIPC {
       rpcServer.stop();
     }
   }
+
+  /**
+   * Testcase for getting connection registry information through connection preamble header, see
+   * HBASE-25051 for more details.
+   */
+  @Test
+  public void testGetConnectionRegistry() throws IOException, ServiceException {
+    Configuration clientConf = new Configuration(CONF);
+    String clusterId = "test_cluster_id";
+    HRegionServer server = mock(HRegionServer.class);
+    when(server.getClusterId()).thenReturn(clusterId);
+    // do not need any services
+    RpcServer rpcServer = createRpcServer(server, "testRpcServer", Collections.emptyList(),
+      new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
+      rpcServer.start();
+      InetSocketAddress addr = rpcServer.getListenerAddress();
+      BlockingRpcChannel channel =
+        client.createBlockingRpcChannel(ServerName.valueOf(addr.getHostName(), addr.getPort(),
+          EnvironmentEdgeManager.currentTime()), User.getCurrent(), 0);
+      ConnectionRegistryService.BlockingInterface stub =
+        ConnectionRegistryService.newBlockingStub(channel);
+      GetConnectionRegistryResponse resp =
+        stub.getConnectionRegistry(null, GetConnectionRegistryRequest.getDefaultInstance());
+      assertEquals(clusterId, resp.getClusterId());
+    }
+  }
+
+  /**
+   * Test server does not support getting connection registry information through connection
+   * preamble header, i.e, a new client connecting to an old server. We simulate this by using a
+   * Server without implementing the ConnectionRegistryEndpoint interface.
+   */
+  @Test
+  public void testGetConnectionRegistryError() throws IOException, ServiceException {
+    Configuration clientConf = new Configuration(CONF);
+    // do not need any services
+    RpcServer rpcServer = createRpcServer("testRpcServer", Collections.emptyList(),
+      new InetSocketAddress("localhost", 0), CONF, new FifoRpcScheduler(CONF, 1));
+    try (AbstractRpcClient<?> client = createRpcClient(clientConf)) {
+      rpcServer.start();
+      InetSocketAddress addr = rpcServer.getListenerAddress();
+      RpcChannel channel = client.createRpcChannel(ServerName.valueOf(addr.getHostName(),
+        addr.getPort(), EnvironmentEdgeManager.currentTime()), User.getCurrent(), 0);
+      ConnectionRegistryService.Interface stub = ConnectionRegistryService.newStub(channel);
+      HBaseRpcController pcrc = new HBaseRpcControllerImpl();
+      BlockingRpcCallback<GetConnectionRegistryResponse> done = new BlockingRpcCallback<>();
+      stub.getConnectionRegistry(pcrc, GetConnectionRegistryRequest.getDefaultInstance(), done);
+      // should have failed so no response
+      assertNull(done.get());
+      assertTrue(pcrc.failed());
+      // should be a FatalConnectionException
+      assertThat(pcrc.getFailed(), instanceOf(RemoteException.class));
+      assertEquals(FatalConnectionException.class.getName(),
+        ((RemoteException) pcrc.getFailed()).getClassName());
+      assertThat(pcrc.getFailed().getMessage(), startsWith("Expected HEADER="));
+    }
+  }
 }
