@@ -49,7 +49,7 @@ public class TestBlockingIPC extends AbstractTestIPC {
   protected BlockingRpcClient createRpcClientNoCodec(Configuration conf) {
     return new BlockingRpcClient(conf) {
       @Override
-      Codec getCodec() {
+      protected Codec getCodec() {
         return null;
       }
     };
@@ -66,7 +66,7 @@ public class TestBlockingIPC extends AbstractTestIPC {
     return new BlockingRpcClient(conf) {
 
       @Override
-      boolean isTcpNoDelay() {
+      protected boolean isTcpNoDelay() {
         throw new RuntimeException("Injected fault");
       }
     };
