@@ -30,8 +30,6 @@ import java.util.List;
 import java.util.NavigableSet;
 import java.util.concurrent.atomic.LongAdder;
 import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hbase.HBaseTestingUtility;
 import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.KeyValue;
@@ -53,12 +51,12 @@ import org.apache.hadoop.hbase.regionserver.MultiVersionConcurrencyControl;
 import org.apache.hadoop.hbase.regionserver.Region;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.CommonFSUtils;
+import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.JVMClusterUtil.MasterThread;
 import org.apache.hadoop.hbase.util.JVMClusterUtil.RegionServerThread;
 import org.apache.hadoop.hbase.util.Threads;
 import org.apache.hadoop.hbase.wal.WAL;
 import org.apache.hadoop.hbase.wal.WALEdit;
-import org.apache.hadoop.hbase.wal.WALFactory;
 import org.apache.hadoop.hbase.wal.WALKeyImpl;
 import org.apache.hadoop.hbase.zookeeper.ZKUtil;
 import org.junit.After;
@@ -373,9 +371,10 @@ public abstract class AbstractTestDLS {
         row = Arrays.copyOfRange(row, 3, 8); // use last 5 bytes because
         // HBaseTestingUtility.createMultiRegions use 5 bytes key
         byte[] qualifier = Bytes.toBytes("c" + Integer.toString(i));
-        e.add(new KeyValue(row, COLUMN_FAMILY, qualifier, System.currentTimeMillis(), value));
+        e.add(
+          new KeyValue(row, COLUMN_FAMILY, qualifier, EnvironmentEdgeManager.currentTime(), value));
         log.appendData(curRegionInfo, new WALKeyImpl(curRegionInfo.getEncodedNameAsBytes(),
-          tableName, System.currentTimeMillis(), mvcc), e);
+          tableName, EnvironmentEdgeManager.currentTime(), mvcc), e);
         if (0 == i % syncEvery) {
           log.sync();
         }
@@ -400,19 +399,6 @@ public abstract class AbstractTestDLS {
     return;
   }
 
-  private int countWAL(Path log, FileSystem fs, Configuration conf) throws IOException {
-    int count = 0;
-    try (WAL.Reader in = WALFactory.createReader(fs, log, conf)) {
-      WAL.Entry e;
-      while ((e = in.next()) != null) {
-        if (!WALEdit.isMetaEditFamily(e.getEdit().getCells().get(0))) {
-          count++;
-        }
-      }
-    }
-    return count;
-  }
-
   private void blockUntilNoRIT() throws Exception {
     TEST_UTIL.waitUntilNoRegionsInTransition(60000);
   }
@@ -430,12 +416,12 @@ public abstract class AbstractTestDLS {
 
   private void waitForCounter(LongAdder ctr, long oldval, long newval, long timems)
     throws InterruptedException {
-    long curt = System.currentTimeMillis();
+    long curt = EnvironmentEdgeManager.currentTime();
     long endt = curt + timems;
     while (curt < endt) {
       if (ctr.sum() == oldval) {
         Thread.sleep(100);
-        curt = System.currentTimeMillis();
+        curt = EnvironmentEdgeManager.currentTime();
       } else {
         assertEquals(newval, ctr.sum());
         return;
