@@ -49,11 +49,10 @@ public class TestBalancerRejection extends BalancerTestBase {
   public static final HBaseClassTestRule CLASS_RULE =
     HBaseClassTestRule.forClass(TestBalancerRejection.class);
 
-  static class MockCostFunction extends StochasticLoadBalancer.CostFunction {
+  static class MockCostFunction extends CostFunction {
     public static double mockCost;
 
     public MockCostFunction(Configuration c) {
-      super(c);
     }
 
     @Override
@@ -61,11 +60,6 @@ public class TestBalancerRejection extends BalancerTestBase {
       return mockCost;
     }
 
-    @Override
-    boolean isNeeded() {
-      return super.isNeeded();
-    }
-
     @Override
     float getMultiplier() {
       return 1;
@@ -79,7 +73,7 @@ public class TestBalancerRejection extends BalancerTestBase {
       conf.setBoolean(BaseLoadBalancer.BALANCER_REJECTION_BUFFER_ENABLED, true);
       conf.set(StochasticLoadBalancer.COST_FUNCTIONS_COST_FUNCTIONS_KEY,
         MockCostFunction.class.getName());
-      loadBalancer.setConf(conf);
+      loadBalancer.onConfigurationChange(conf);
       // Simulate 2 servers with 5 regions.
       Map<ServerName, List<RegionInfo>> servers = mockClusterServers(new int[] { 5, 5 });
       Map<TableName, Map<ServerName, List<RegionInfo>>> LoadOfAllTable =
@@ -93,7 +87,7 @@ public class TestBalancerRejection extends BalancerTestBase {
       // Reject case 2: Cost < minCostNeedBalance
       MockCostFunction.mockCost = 1;
       conf.setFloat("hbase.master.balancer.stochastic.minCostNeedBalance", Float.MAX_VALUE);
-      loadBalancer.setConf(conf);
+      loadBalancer.onConfigurationChange(conf);
       Assert.assertNull(loadBalancer.balanceCluster(LoadOfAllTable));
 
       // NamedQueue is an async Producer-consumer Pattern, waiting here until it completed
@@ -109,7 +103,7 @@ public class TestBalancerRejection extends BalancerTestBase {
     } finally {
       conf.unset(StochasticLoadBalancer.COST_FUNCTIONS_COST_FUNCTIONS_KEY);
       conf.unset(BaseLoadBalancer.BALANCER_REJECTION_BUFFER_ENABLED);
-      loadBalancer.setConf(conf);
+      loadBalancer.onConfigurationChange(conf);
     }
   }
 
