@@ -45,6 +45,7 @@ import org.apache.hadoop.hbase.procedure2.ProcedureTestingUtility;
 import org.apache.hadoop.hbase.testclassification.LargeTests;
 import org.apache.hadoop.hbase.testclassification.MasterTests;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.hamcrest.core.IsInstanceOf;
 import org.hamcrest.core.StringStartsWith;
 import org.junit.After;
@@ -195,8 +196,8 @@ public class TestLockProcedure {
    * @throws TimeoutException if lock couldn't be acquired.
    */
   private boolean awaitForLocked(long procId, long timeoutInMs) throws Exception {
-    long deadline = System.currentTimeMillis() + timeoutInMs;
-    while (System.currentTimeMillis() < deadline) {
+    long deadline = EnvironmentEdgeManager.currentTime() + timeoutInMs;
+    while (EnvironmentEdgeManager.currentTime() < deadline) {
       LockHeartbeatResponse response = masterRpcService.lockHeartbeat(null,
         LockHeartbeatRequest.newBuilder().setProcId(procId).build());
       if (response.getLockStatus() == LockHeartbeatResponse.LockStatus.LOCKED) {
@@ -300,7 +301,7 @@ public class TestLockProcedure {
     // Acquire namespace lock, then queue other locks.
     long nsProcId = queueLock(nsLock);
     assertTrue(awaitForLocked(nsProcId, 2000));
-    long start = System.currentTimeMillis();
+    long start = EnvironmentEdgeManager.currentTime();
     sendHeartbeatAndCheckLocked(nsProcId, true);
     long table1ProcId = queueLock(tableLock1);
     long table2ProcId = queueLock(tableLock2);
@@ -308,7 +309,7 @@ public class TestLockProcedure {
     long regions2ProcId = queueLock(regionsLock2);
 
     // Assert tables & region locks are waiting because of namespace lock.
-    long now = System.currentTimeMillis();
+    long now = EnvironmentEdgeManager.currentTime();
     // leave extra 10 msec in case more than half the HEARTBEAT_TIMEOUT has passed
     Thread
       .sleep(Math.min(HEARTBEAT_TIMEOUT / 2, Math.max(HEARTBEAT_TIMEOUT - (now - start) - 10, 0)));
