@@ -18,8 +18,6 @@
 package org.apache.hadoop.hbase.master.snapshot;
 
 import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertThrows;
-import static org.junit.Assert.assertTrue;
 
 import java.util.HashMap;
 import java.util.Map;
@@ -31,7 +29,6 @@ import org.apache.hadoop.hbase.client.Put;
 import org.apache.hadoop.hbase.client.Table;
 import org.apache.hadoop.hbase.client.TableDescriptor;
 import org.apache.hadoop.hbase.client.TableDescriptorBuilder;
-import org.apache.hadoop.hbase.snapshot.HBaseSnapshotException;
 import org.apache.hadoop.hbase.snapshot.SnapshotTTLExpiredException;
 import org.apache.hadoop.hbase.testclassification.MediumTests;
 import org.apache.hadoop.hbase.util.Bytes;
@@ -104,17 +101,12 @@ public class TestTakeSnapshotHandler {
     assertEquals(-1, UTIL.getAdmin().getDescriptor(cloned).getMaxFileSize());
   }
 
-  @Test
+  @Test(expected = SnapshotTTLExpiredException.class)
   public void testSnapshotEarlyExpiration() throws Exception {
     UTIL.startMiniCluster();
     Map<String, Object> snapshotProps = new HashMap<>();
     snapshotProps.put("TTL", 1L);
-    HBaseSnapshotException hBaseSnapshotException = assertThrows(HBaseSnapshotException.class,
-      () -> createTableInsertDataAndTakeSnapshot(snapshotProps));
-    assertTrue(
-      hBaseSnapshotException.toString().contains(SnapshotTTLExpiredException.class.getName()));
-    assertTrue(hBaseSnapshotException.getMessage()
-      .contains("TTL for snapshot 'snaptestSnapshotEarlyExpiration' has already expired"));
+    createTableInsertDataAndTakeSnapshot(snapshotProps);
   }
 
   @After
