@@ -34,6 +34,7 @@ import org.apache.hadoop.hbase.io.hfile.HFileContext;
 import org.apache.hadoop.hbase.io.hfile.HFileContextBuilder;
 import org.apache.hadoop.hbase.regionserver.BloomType;
 import org.apache.hadoop.hbase.regionserver.HStoreFile;
+import org.apache.hadoop.hbase.regionserver.StoreFileInfo;
 import org.apache.hadoop.hbase.regionserver.StoreFileScanner;
 import org.apache.hadoop.hbase.regionserver.StoreFileWriter;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
@@ -67,8 +68,9 @@ public class TestMobFile {
     String caseName = testName.getMethodName();
     MobTestUtil.writeStoreFile(writer, caseName);
 
-    MobFile mobFile =
-      new MobFile(new HStoreFile(fs, writer.getPath(), conf, cacheConf, BloomType.NONE, true));
+    StoreFileInfo storeFileInfo =
+      StoreFileInfo.createStoreFileInfoForHFile(conf, fs, writer.getPath(), true);
+    MobFile mobFile = new MobFile(new HStoreFile(storeFileInfo, BloomType.NONE, cacheConf));
     byte[] family = Bytes.toBytes(caseName);
     byte[] qualify = Bytes.toBytes(caseName);
 
@@ -116,8 +118,9 @@ public class TestMobFile {
       .withFileContext(meta).build();
     MobTestUtil.writeStoreFile(writer, testName.getMethodName());
 
-    MobFile mobFile =
-      new MobFile(new HStoreFile(fs, writer.getPath(), conf, cacheConf, BloomType.NONE, true));
+    StoreFileInfo storeFileInfo =
+      StoreFileInfo.createStoreFileInfoForHFile(conf, fs, writer.getPath(), true);
+    MobFile mobFile = new MobFile(new HStoreFile(storeFileInfo, BloomType.NONE, cacheConf));
     assertNotNull(mobFile.getScanner());
     assertTrue(mobFile.getScanner() instanceof StoreFileScanner);
   }
