@@ -23,6 +23,7 @@ import static org.junit.Assert.assertTrue;
 
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.HBaseClassTestRule;
+import org.apache.hadoop.hbase.HBaseConfiguration;
 import org.apache.hadoop.hbase.testclassification.RegionServerTests;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
@@ -41,6 +42,7 @@ public class TestDefaultOperationQuota {
   public static final HBaseClassTestRule CLASS_RULE =
     HBaseClassTestRule.forClass(TestDefaultOperationQuota.class);
 
+  private static final Configuration conf = HBaseConfiguration.create();
   private static final int DEFAULT_REQUESTS_PER_SECOND = 1000;
   private static ManualEnvironmentEdge envEdge = new ManualEnvironmentEdge();
   static {
@@ -150,7 +152,7 @@ public class TestDefaultOperationQuota {
     QuotaProtos.Throttle throttle =
       QuotaProtos.Throttle.newBuilder().setReadNum(QuotaProtos.TimedQuota.newBuilder()
         .setSoftLimit(limit).setTimeUnit(HBaseProtos.TimeUnit.SECONDS).build()).build();
-    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(throttle);
+    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(conf, throttle);
     DefaultOperationQuota quota =
       new DefaultOperationQuota(new Configuration(), 65536, DEFAULT_REQUESTS_PER_SECOND, limiter);
 
@@ -172,7 +174,7 @@ public class TestDefaultOperationQuota {
     QuotaProtos.Throttle throttle =
       QuotaProtos.Throttle.newBuilder().setWriteNum(QuotaProtos.TimedQuota.newBuilder()
         .setSoftLimit(limit).setTimeUnit(HBaseProtos.TimeUnit.SECONDS).build()).build();
-    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(throttle);
+    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(conf, throttle);
     DefaultOperationQuota quota =
       new DefaultOperationQuota(new Configuration(), 65536, DEFAULT_REQUESTS_PER_SECOND, limiter);
 
@@ -194,7 +196,7 @@ public class TestDefaultOperationQuota {
     QuotaProtos.Throttle throttle =
       QuotaProtos.Throttle.newBuilder().setReadNum(QuotaProtos.TimedQuota.newBuilder()
         .setSoftLimit(limit).setTimeUnit(HBaseProtos.TimeUnit.SECONDS).build()).build();
-    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(throttle);
+    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(conf, throttle);
     DefaultOperationQuota quota =
       new DefaultOperationQuota(new Configuration(), 65536, DEFAULT_REQUESTS_PER_SECOND, limiter);
 
@@ -216,7 +218,7 @@ public class TestDefaultOperationQuota {
     QuotaProtos.Throttle throttle =
       QuotaProtos.Throttle.newBuilder().setWriteNum(QuotaProtos.TimedQuota.newBuilder()
         .setSoftLimit(limit).setTimeUnit(HBaseProtos.TimeUnit.SECONDS).build()).build();
-    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(throttle);
+    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(conf, throttle);
     DefaultOperationQuota quota =
       new DefaultOperationQuota(new Configuration(), 65536, DEFAULT_REQUESTS_PER_SECOND, limiter);
 
@@ -238,7 +240,7 @@ public class TestDefaultOperationQuota {
     QuotaProtos.Throttle throttle =
       QuotaProtos.Throttle.newBuilder().setWriteSize(QuotaProtos.TimedQuota.newBuilder()
         .setSoftLimit(limit).setTimeUnit(HBaseProtos.TimeUnit.SECONDS).build()).build();
-    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(throttle);
+    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(conf, throttle);
     DefaultOperationQuota quota =
       new DefaultOperationQuota(new Configuration(), 65536, DEFAULT_REQUESTS_PER_SECOND, limiter);
 
@@ -261,7 +263,7 @@ public class TestDefaultOperationQuota {
     QuotaProtos.Throttle throttle =
       QuotaProtos.Throttle.newBuilder().setReadSize(QuotaProtos.TimedQuota.newBuilder()
         .setSoftLimit(limit).setTimeUnit(HBaseProtos.TimeUnit.SECONDS).build()).build();
-    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(throttle);
+    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(conf, throttle);
     DefaultOperationQuota quota = new DefaultOperationQuota(new Configuration(), (int) blockSize,
       DEFAULT_REQUESTS_PER_SECOND, limiter);
 
@@ -284,7 +286,7 @@ public class TestDefaultOperationQuota {
     QuotaProtos.Throttle throttle =
       QuotaProtos.Throttle.newBuilder().setReqSize(QuotaProtos.TimedQuota.newBuilder()
         .setSoftLimit(limit).setTimeUnit(HBaseProtos.TimeUnit.SECONDS).build()).build();
-    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(throttle);
+    QuotaLimiter limiter = TimeBasedLimiter.fromThrottle(conf, throttle);
     DefaultOperationQuota quota = new DefaultOperationQuota(new Configuration(), (int) blockSize,
       DEFAULT_REQUESTS_PER_SECOND, limiter);
 
