@@ -18,6 +18,7 @@
 package org.apache.hadoop.hbase.regionserver;
 
 import java.io.IOException;
+import java.net.UnknownHostException;
 import java.util.Arrays;
 import java.util.Map;
 import java.util.Optional;
@@ -30,6 +31,7 @@ import org.apache.hadoop.hbase.CellBuilderType;
 import org.apache.hadoop.hbase.HBaseTestingUtility;
 import org.apache.hadoop.hbase.HDFSBlocksDistribution;
 import org.apache.hadoop.hbase.io.hfile.CacheConfig;
+import org.apache.hadoop.hbase.regionserver.storefiletracker.StoreFileTracker;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.DNS;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
@@ -52,9 +54,13 @@ public class MockHStoreFile extends HStoreFile {
   boolean compactedAway;
 
   MockHStoreFile(HBaseTestingUtility testUtil, Path testPath, long length, long ageInDisk,
-    boolean isRef, long sequenceid) throws IOException {
-    super(testUtil.getTestFileSystem(), testPath, testUtil.getConfiguration(),
-      new CacheConfig(testUtil.getConfiguration()), BloomType.NONE, true);
+    boolean isRef, long sequenceid, StoreFileInfo storeFileInfo) throws IOException {
+    super(storeFileInfo, BloomType.NONE, new CacheConfig(testUtil.getConfiguration()));
+    setMockHStoreFileVals(length, isRef, ageInDisk, sequenceid, isMajor, testUtil);
+  }
+
+  private void setMockHStoreFileVals(long length, boolean isRef, long ageInDisk, long sequenceid,
+    boolean isMajor, HBaseTestingUtility testUtil) throws UnknownHostException {
     this.length = length;
     this.isRef = isRef;
     this.ageInDisk = ageInDisk;
@@ -67,6 +73,13 @@ public class MockHStoreFile extends HStoreFile {
     modificationTime = EnvironmentEdgeManager.currentTime();
   }
 
+  MockHStoreFile(HBaseTestingUtility testUtil, Path testPath, long length, long ageInDisk,
+    boolean isRef, long sequenceid, StoreFileTracker tracker) throws IOException {
+    super(testUtil.getTestFileSystem(), testPath, testUtil.getConfiguration(),
+      new CacheConfig(testUtil.getConfiguration()), BloomType.NONE, true, tracker);
+    setMockHStoreFileVals(length, isRef, ageInDisk, sequenceid, isMajor, testUtil);
+  }
+
   void setLength(long newLen) {
     this.length = newLen;
   }
