@@ -650,6 +650,7 @@ public class TestBytesReadServerSideScanMetrics {
       LOG.info("Bytes read from fs: " + scanMetrics.bytesReadFromFs.get());
       LOG.info("Bytes read from block cache: " + scanMetrics.bytesReadFromBlockCache.get());
       LOG.info("Bytes read from memstore: " + scanMetrics.bytesReadFromMemstore.get());
+      LOG.info("Count of bytes scanned: " + scanMetrics.countOfBlockBytesScanned.get());
       LOG.info("StoreFileScanners seek count: " + StoreFileScanner.getSeekCount());
     }
     return scanMetrics;
@@ -873,8 +874,7 @@ public class TestBytesReadServerSideScanMetrics {
       return;
     }
     Path path = hfileReader.getPath();
-    BlockCacheKey key =
-      new BlockCacheKey(path.getName(), block.getOffset(), true, block.getBlockType());
+    BlockCacheKey key = new BlockCacheKey(path, block.getOffset(), true, block.getBlockType());
     HFileBlock cachedBlock = (HFileBlock) blockCache.getBlock(key, true, false, true);
     Assert.assertNotNull(cachedBlock);
     Assert.assertEquals(block.getOnDiskSizeWithHeader(), cachedBlock.getOnDiskSizeWithHeader());
