@@ -121,8 +121,9 @@ public class TestReversibleScanners {
           .withOutputDir(hfilePath).withFileContext(hFileContext).build();
       writeStoreFile(writer);
 
-      HStoreFile sf = new HStoreFile(fs, writer.getPath(), TEST_UTIL.getConfiguration(), cacheConf,
-        BloomType.NONE, true);
+      StoreFileInfo storeFileInfo = StoreFileInfo
+        .createStoreFileInfoForHFile(TEST_UTIL.getConfiguration(), fs, writer.getPath(), true);
+      HStoreFile sf = new HStoreFile(storeFileInfo, BloomType.NONE, cacheConf);
 
       List<StoreFileScanner> scanners = StoreFileScanner.getScannersForStoreFiles(
         Collections.singletonList(sf), false, true, false, false, Long.MAX_VALUE);
@@ -172,11 +173,13 @@ public class TestReversibleScanners {
     MemStore memstore = new DefaultMemStore();
     writeMemstoreAndStoreFiles(memstore, new StoreFileWriter[] { writer1, writer2 });
 
-    HStoreFile sf1 = new HStoreFile(fs, writer1.getPath(), TEST_UTIL.getConfiguration(), cacheConf,
-      BloomType.NONE, true);
+    StoreFileInfo storeFileInfo1 = StoreFileInfo
+      .createStoreFileInfoForHFile(TEST_UTIL.getConfiguration(), fs, writer1.getPath(), true);
+    HStoreFile sf1 = new HStoreFile(storeFileInfo1, BloomType.NONE, cacheConf);
 
-    HStoreFile sf2 = new HStoreFile(fs, writer2.getPath(), TEST_UTIL.getConfiguration(), cacheConf,
-      BloomType.NONE, true);
+    StoreFileInfo storeFileInfo2 = StoreFileInfo
+      .createStoreFileInfoForHFile(TEST_UTIL.getConfiguration(), fs, writer2.getPath(), true);
+    HStoreFile sf2 = new HStoreFile(storeFileInfo2, BloomType.NONE, cacheConf);
     /**
      * Test without MVCC
      */
@@ -252,11 +255,13 @@ public class TestReversibleScanners {
     MemStore memstore = new DefaultMemStore();
     writeMemstoreAndStoreFiles(memstore, new StoreFileWriter[] { writer1, writer2 });
 
-    HStoreFile sf1 = new HStoreFile(fs, writer1.getPath(), TEST_UTIL.getConfiguration(), cacheConf,
-      BloomType.NONE, true);
+    StoreFileInfo storeFileInfo1 = StoreFileInfo
+      .createStoreFileInfoForHFile(TEST_UTIL.getConfiguration(), fs, writer1.getPath(), true);
+    HStoreFile sf1 = new HStoreFile(storeFileInfo1, BloomType.NONE, cacheConf);
 
-    HStoreFile sf2 = new HStoreFile(fs, writer2.getPath(), TEST_UTIL.getConfiguration(), cacheConf,
-      BloomType.NONE, true);
+    StoreFileInfo storeFileInfo2 = StoreFileInfo
+      .createStoreFileInfoForHFile(TEST_UTIL.getConfiguration(), fs, writer2.getPath(), true);
+    HStoreFile sf2 = new HStoreFile(storeFileInfo2, BloomType.NONE, cacheConf);
 
     ScanInfo scanInfo = new ScanInfo(TEST_UTIL.getConfiguration(), FAMILYNAME, 0, Integer.MAX_VALUE,
       Long.MAX_VALUE, KeepDeletedCells.FALSE, HConstants.DEFAULT_BLOCKSIZE, 0,
