@@ -19,6 +19,7 @@ package org.apache.hadoop.hbase.regionserver.throttle;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.io.IOException;
 import java.util.List;
@@ -125,7 +126,14 @@ public class TestFlushWithThroughputController {
         table.put(new Put(Bytes.toBytes(i * 10 + j)).addColumn(family, qualifier, value));
       }
       long startTime = System.nanoTime();
-      hbtu.getAdmin().flush(tableName);
+      hbtu.getHBaseCluster().getRegions(tableName).stream().findFirst().ifPresent(r -> {
+        try {
+          r.flush(true);
+        } catch (IOException e) {
+          LOG.error("Failed flush region {}", r, e);
+          fail("Failed flush region " + r.getRegionInfo().getRegionNameAsString());
+        }
+      });
       duration += System.nanoTime() - startTime;
     }
     HStore store = getStoreWithName(tableName);
