@@ -52,8 +52,8 @@ public class TestAsyncFSWALDurability extends WALDurabilityTestBase<CustomAsyncF
   }
 
   @AfterClass
-  public static void tearDownAfterClass() {
-    GROUP.shutdownGracefully();
+  public static void tearDownAfterClass() throws Exception {
+    GROUP.shutdownGracefully().get();
   }
 
   @Override
@@ -128,15 +128,15 @@ class CustomAsyncFSWAL extends AsyncFSWAL {
   }
 
   @Override
-  public void sync(boolean forceSync) throws IOException {
+  protected void doSync(boolean forceSync) throws IOException {
     syncFlag = forceSync;
-    super.sync(forceSync);
+    super.doSync(forceSync);
   }
 
   @Override
-  public void sync(long txid, boolean forceSync) throws IOException {
+  protected void doSync(long txid, boolean forceSync) throws IOException {
     syncFlag = forceSync;
-    super.sync(txid, forceSync);
+    super.doSync(txid, forceSync);
   }
 
   void resetSyncFlag() {
