@@ -29,6 +29,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Objects;
+import java.util.concurrent.TimeUnit;
 import org.apache.commons.lang3.mutable.MutableObject;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.Cell;
@@ -260,7 +261,8 @@ public class TestMetaRegionReplicaReplicationEndpoint {
           HTU.compact(tableName, false);
         }
       }
-      HTU.getHBaseCluster().getMaster().getLogCleaner().runCleaner();
+      HTU.getHBaseCluster().getMaster().getLogCleaner().triggerCleanerNow().get(1,
+        TimeUnit.SECONDS);
       ((ReplicationPeerImpl) source.replicationPeer).setPeerState(true);
       // now loads more data without flushing nor compacting
       for (int i = 5; i < 10; i++) {
