@@ -123,7 +123,7 @@ public abstract class CompressedWALTestBase {
     wals.shutdown();
 
     // Confirm the WAL can be read back
-    try (WAL.Reader reader = wals.createReader(TEST_UTIL.getTestFileSystem(), walPath)) {
+    try (WALStreamReader reader = wals.createStreamReader(TEST_UTIL.getTestFileSystem(), walPath)) {
       int count = 0;
       WAL.Entry entry = new WAL.Entry();
       while (reader.next(entry) != null) {
