@@ -166,6 +166,16 @@ module Hbase
       end
     end
 
+    #----------------------------------------------------------------------------------------------
+    # Requests a region truncate
+    def truncate_region(region_name)
+      begin
+        org.apache.hadoop.hbase.util.FutureUtils.get(@admin.truncateRegionAsync(region_name.to_java_bytes))
+      rescue java.lang.IllegalArgumentException, org.apache.hadoop.hbase.UnknownRegionException
+        @admin.truncate_region(region_name.to_java_bytes)
+      end
+    end
+
     #----------------------------------------------------------------------------------------------
     # Enable/disable one split or merge switch
     # Returns previous switch setting.
@@ -1507,6 +1517,8 @@ module Hbase
 
     # Parse arguments and update HTableDescriptor accordingly
     def update_htd_from_arg(htd, arg)
+      htd.setErasureCodingPolicy(arg.delete(org.apache.hadoop.hbase.HTableDescriptor::ERASURE_CODING_POLICY)) \
+        if arg.include?(org.apache.hadoop.hbase.HTableDescriptor::ERASURE_CODING_POLICY)
       htd.setOwnerString(arg.delete(org.apache.hadoop.hbase.HTableDescriptor::OWNER)) if arg.include?(org.apache.hadoop.hbase.HTableDescriptor::OWNER)
       htd.setMaxFileSize(arg.delete(org.apache.hadoop.hbase.HTableDescriptor::MAX_FILESIZE)) if arg.include?(org.apache.hadoop.hbase.HTableDescriptor::MAX_FILESIZE)
       htd.setReadOnly(JBoolean.valueOf(arg.delete(org.apache.hadoop.hbase.HTableDescriptor::READONLY))) if arg.include?(org.apache.hadoop.hbase.HTableDescriptor::READONLY)
@@ -1835,6 +1847,12 @@ module Hbase
     def flush_master_store()
       @admin.flushMasterStore()
     end
+
+    #----------------------------------------------------------------------------------------------
+    # Returns a list of enable or disabled tables in hbase
+    def list_tables_by_state(is_enabled)
+      @admin.listTableNamesByState(is_enabled).map(&:getNameAsString)
+    end
   end
   # rubocop:enable Metrics/ClassLength
 end
