@@ -20,7 +20,6 @@ package org.apache.hadoop.hbase.client;
 import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.testclassification.ClientTests;
 import org.apache.hadoop.hbase.testclassification.LargeTests;
-import org.junit.BeforeClass;
 import org.junit.ClassRule;
 import org.junit.experimental.categories.Category;
 
@@ -30,17 +29,8 @@ public class TestListTablesShell extends AbstractTestShell {
   public static final HBaseClassTestRule CLASS_RULE =
     HBaseClassTestRule.forClass(TestListTablesShell.class);
 
-  @BeforeClass
-  public static void setUpBeforeClass() throws Exception {
-    setUpConfig();
-
-    TEST_UTIL.startMiniCluster(3);
-
-    setUpJRubyRuntime();
-  }
-
   @Override
-  protected String getIncludeList() {
+  public String getIncludeList() {
     return "list_tables_test.rb";
   }
 }
