@@ -1,5 +1,4 @@
-/**
- *
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -29,7 +28,6 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Queue;
 import java.util.concurrent.TimeUnit;
-
 import org.apache.commons.lang3.NotImplementedException;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.CompareOperator;
@@ -87,17 +85,16 @@ public class ThriftTable implements Table {
   private final int scannerCaching;
 
   public ThriftTable(TableName tableName, THBaseService.Client client, TTransport tTransport,
-      Configuration conf) {
+    Configuration conf) {
     this.tableName = tableName;
     this.tableNameInBytes = ByteBuffer.wrap(tableName.toBytes());
     this.conf = conf;
     this.tTransport = tTransport;
     this.client = client;
-    this.scannerCaching = conf.getInt(HBASE_THRIFT_CLIENT_SCANNER_CACHING,
-        HBASE_THRIFT_CLIENT_SCANNER_CACHING_DEFAULT);
+    this.scannerCaching =
+      conf.getInt(HBASE_THRIFT_CLIENT_SCANNER_CACHING, HBASE_THRIFT_CLIENT_SCANNER_CACHING_DEFAULT);
     this.operationTimeout = conf.getInt(HConstants.HBASE_CLIENT_OPERATION_TIMEOUT,
-        HConstants.DEFAULT_HBASE_CLIENT_OPERATION_TIMEOUT);
-
+      HConstants.DEFAULT_HBASE_CLIENT_OPERATION_TIMEOUT);
 
   }
 
@@ -114,8 +111,8 @@ public class ThriftTable implements Table {
   @Override
   public TableDescriptor getDescriptor() throws IOException {
     try {
-      TTableDescriptor tableDescriptor = client
-          .getTableDescriptor(ThriftUtilities.tableNameFromHBase(tableName));
+      TTableDescriptor tableDescriptor =
+        client.getTableDescriptor(ThriftUtilities.tableNameFromHBase(tableName));
       return ThriftUtilities.tableDescriptorFromThrift(tableDescriptor);
     } catch (TException e) {
       throw new IOException(e);
@@ -127,7 +124,7 @@ public class ThriftTable implements Table {
     TGet tGet = ThriftUtilities.getFromHBase(get);
     try {
       return client.exists(tableNameInBytes, tGet);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
@@ -135,31 +132,29 @@ public class ThriftTable implements Table {
   @Override
   public boolean[] exists(List<Get> gets) throws IOException {
     List<TGet> tGets = new ArrayList<>();
-    for (Get get: gets) {
+    for (Get get : gets) {
       tGets.add(ThriftUtilities.getFromHBase(get));
     }
     try {
       List<Boolean> results = client.existsAll(tableNameInBytes, tGets);
       return Booleans.toArray(results);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
 
   @Override
-  public void batch(List<? extends Row> actions, Object[] results)
-      throws IOException {
+  public void batch(List<? extends Row> actions, Object[] results) throws IOException {
     throw new IOException("Batch not supported in ThriftTable, use put(List<Put> puts), "
-        + "get(List<Get> gets) or delete(List<Delete> deletes) respectively");
-
+      + "get(List<Get> gets) or delete(List<Delete> deletes) respectively");
 
   }
 
   @Override
   public <R> void batchCallback(List<? extends Row> actions, Object[] results,
-      Batch.Callback<R> callback) throws IOException {
+    Batch.Callback<R> callback) throws IOException {
     throw new IOException("BatchCallback not supported in ThriftTable, use put(List<Put> puts), "
-        + "get(List<Get> gets) or delete(List<Delete> deletes) respectively");
+      + "get(List<Get> gets) or delete(List<Delete> deletes) respectively");
   }
 
   @Override
@@ -168,7 +163,7 @@ public class ThriftTable implements Table {
     try {
       TResult tResult = client.get(tableNameInBytes, tGet);
       return ThriftUtilities.resultFromThrift(tResult);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
@@ -179,28 +174,26 @@ public class ThriftTable implements Table {
     try {
       List<TResult> results = client.getMultiple(tableNameInBytes, tGets);
       return ThriftUtilities.resultsFromThrift(results);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
 
   /**
-   * A scanner to perform scan from thrift server
-   * getScannerResults is used in this scanner
+   * A scanner to perform scan from thrift server getScannerResults is used in this scanner
    */
   private class Scanner implements ResultScanner {
     protected TScan scan;
     protected Result lastResult = null;
     protected final Queue<Result> cache = new ArrayDeque<>();;
 
-
     public Scanner(Scan scan) throws IOException {
       if (scan.getBatch() > 0) {
         throw new IOException("Batch is not supported in Scanner");
       }
       if (scan.getCaching() <= 0) {
         scan.setCaching(scannerCaching);
-      } else if (scan.getCaching() == 1 && scan.isReversed()){
+      } else if (scan.getCaching() == 1 && scan.isReversed()) {
         // for reverse scan, we need to pass the last row to the next scanner
         // we need caching number bigger than 1
         scan.setCaching(scan.getCaching() + 1);
@@ -208,14 +201,13 @@ public class ThriftTable implements Table {
       this.scan = ThriftUtilities.scanFromHBase(scan);
     }
 
-
     @Override
     public Result next() throws IOException {
       if (cache.size() == 0) {
         setupNextScanner();
         try {
-          List<TResult> tResults = client
-              .getScannerResults(tableNameInBytes, scan, scan.getCaching());
+          List<TResult> tResults =
+            client.getScannerResults(tableNameInBytes, scan, scan.getCaching());
           Result[] results = ThriftUtilities.resultsFromThrift(tResults);
           boolean firstKey = true;
           for (Result result : results) {
@@ -242,7 +234,7 @@ public class ThriftTable implements Table {
       if (cache.size() > 0) {
         return cache.poll();
       } else {
-        //scan finished
+        // scan finished
         return null;
       }
     }
@@ -262,11 +254,11 @@ public class ThriftTable implements Table {
     }
 
     private void setupNextScanner() {
-      //if lastResult is null null, it means it is not the fist scan
-      if (lastResult!= null) {
+      // if lastResult is null null, it means it is not the fist scan
+      if (lastResult != null) {
         byte[] lastRow = lastResult.getRow();
         if (scan.isReversed()) {
-          //for reverse scan, we can't find the closet row before this row
+          // for reverse scan, we can't find the closet row before this row
           scan.setStartRow(lastRow);
         } else {
           scan.setStartRow(createClosestRowAfter(lastRow));
@@ -274,7 +266,6 @@ public class ThriftTable implements Table {
       }
     }
 
-
     /**
      * Create the closest row after the specified row
      */
@@ -310,7 +301,7 @@ public class ThriftTable implements Table {
     TPut tPut = ThriftUtilities.putFromHBase(put);
     try {
       client.put(tableNameInBytes, tPut);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
@@ -320,7 +311,7 @@ public class ThriftTable implements Table {
     List<TPut> tPuts = ThriftUtilities.putsFromHBase(puts);
     try {
       client.putMultiple(tableNameInBytes, tPuts);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
@@ -330,7 +321,7 @@ public class ThriftTable implements Table {
     TDelete tDelete = ThriftUtilities.deleteFromHBase(delete);
     try {
       client.deleteSingle(tableNameInBytes, tDelete);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
@@ -340,7 +331,7 @@ public class ThriftTable implements Table {
     List<TDelete> tDeletes = ThriftUtilities.deletesFromHBase(deletes);
     try {
       client.deleteMultiple(tableNameInBytes, tDeletes);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
@@ -360,8 +351,8 @@ public class ThriftTable implements Table {
 
     @Override
     public CheckAndMutateBuilder qualifier(byte[] qualifier) {
-      this.qualifier = Preconditions.checkNotNull(qualifier, "qualifier is null. Consider using" +
-          " an empty byte array, or just do not call this method if you want a null qualifier");
+      this.qualifier = Preconditions.checkNotNull(qualifier, "qualifier is null. Consider using"
+        + " an empty byte array, or just do not call this method if you want a null qualifier");
       return this;
     }
 
@@ -385,8 +376,8 @@ public class ThriftTable implements Table {
     }
 
     private void preCheck() {
-      Preconditions.checkNotNull(op, "condition is null. You need to specify the condition by" +
-          " calling ifNotExists/ifEquals/ifMatches before executing the request");
+      Preconditions.checkNotNull(op, "condition is null. You need to specify the condition by"
+        + " calling ifNotExists/ifEquals/ifMatches before executing the request");
     }
 
     @Override
@@ -412,15 +403,14 @@ public class ThriftTable implements Table {
     }
   }
 
-
   @Override
   public boolean checkAndMutate(byte[] row, byte[] family, byte[] qualifier, CompareOperator op,
-      byte[] value, RowMutations mutation) throws IOException {
+    byte[] value, RowMutations mutation) throws IOException {
     try {
-      ByteBuffer valueBuffer = value == null? null : ByteBuffer.wrap(value);
+      ByteBuffer valueBuffer = value == null ? null : ByteBuffer.wrap(value);
       return client.checkAndMutate(tableNameInBytes, ByteBuffer.wrap(row), ByteBuffer.wrap(family),
-          ByteBuffer.wrap(qualifier), ThriftUtilities.compareOpFromHBase(op), valueBuffer,
-          ThriftUtilities.rowMutationsFromHBase(mutation));
+        ByteBuffer.wrap(qualifier), ThriftUtilities.compareOpFromHBase(op), valueBuffer,
+        ThriftUtilities.rowMutationsFromHBase(mutation));
     } catch (TException e) {
       throw new IOException(e);
     }
@@ -452,7 +442,7 @@ public class ThriftTable implements Table {
     try {
       client.mutateRow(tableNameInBytes, tRowMutations);
       return Result.EMPTY_RESULT;
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
@@ -463,7 +453,7 @@ public class ThriftTable implements Table {
     try {
       TResult tResult = client.append(tableNameInBytes, tAppend);
       return ThriftUtilities.resultFromThrift(tResult);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
@@ -474,7 +464,7 @@ public class ThriftTable implements Table {
     try {
       TResult tResult = client.increment(tableNameInBytes, tIncrement);
       return ThriftUtilities.resultFromThrift(tResult);
-    }  catch (TException e) {
+    } catch (TException e) {
       throw new IOException(e);
     }
   }
