@@ -805,8 +805,20 @@ public class RecoverableZooKeeper {
     return zk == null ? null : zk.getState();
   }
 
+  /**
+   * Returns the wrapped ZooKeeper client. If the wrapped client hasn't been created yet then tries
+   * create it first.
+   * @return the wrapped ZK client of null if the creation has failed.
+   */
   public synchronized ZooKeeper getZooKeeper() {
-    return zk;
+    // Callers expect an initialized ZooKeeper instance
+    // Pre HBASE-28529 the constructor used to call checkZk()
+    try {
+      return checkZk();
+    } catch (Exception x) {
+      LOG.warn("Failed to initialize Zookeeper object, returning null", x);
+      return null;
+    }
   }
 
   public synchronized byte[] getSessionPasswd() {
